@@ -3,11 +3,11 @@ import typing as t
 from datetime import datetime, timedelta
 
 import pandas as pd
+from helper import iter_dates  # type: ignore
 
-from examples.sushi.helper import iter_dates
 from sqlmesh import ExecutionContext, model
-from sqlmesh.core.model import IncrementalByTimeRangeKind
-from sqlmesh.utils.date import to_ds
+from sqlmesh.core.model.kind import ModelKindName
+from sqlmesh.utils.date import to_date
 
 CUSTOMERS = list(range(0, 100))
 WAITERS = list(range(0, 10))
@@ -16,24 +16,32 @@ WAITERS = list(range(0, 10))
 @model(
     "sushi.orders",
     description="Table of sushi orders.",
-    kind=IncrementalByTimeRangeKind(time_column="ds"),
-    start="3 months ago",
+    kind=dict(
+        name=ModelKindName.INCREMENTAL_BY_TIME_RANGE, time_column="event_date", batch_size=30
+    ),
+    start="1 week ago",
     cron="@daily",
-    batch_size=30,
+    grains=[
+        "id AS order_id",
+    ],
+    references=[
+        "customer_id",
+        "waiter_id",
+    ],
     columns={
         "id": "int",
         "customer_id": "int",
         "waiter_id": "int",
         "start_ts": "int",
         "end_ts": "int",
-        "ds": "text",
+        "event_date": "date",
     },
 )
 def execute(
     context: ExecutionContext,
     start: datetime,
     end: datetime,
-    latest: datetime,
+    execution_time: datetime,
     **kwargs: t.Any,
 ) -> pd.DataFrame:
     dfs = []
@@ -54,7 +62,7 @@ def execute(
                     "waiter_id": random.choices(WAITERS, k=num_orders),
                     "start_ts": start_ts,
                     "end_ts": end_ts,
-                    "ds": to_ds(dt),
+                    "event_date": to_date(dt),
                 }
             )
             .reset_index()
