@@ -4,5 +4,7 @@ MODEL (
     path '../seeds/waiter_names.csv',
     batch_size 5
   ),
-  owner jen
+  owner jen,
+  grain id,
+  description 'List of waiter names'
 )
