@@ -11,6 +11,7 @@ from sqlmesh import configure_logging
 from sqlmesh.cli import error_handler
 from sqlmesh.cli import options as opt
 from sqlmesh.cli.example_project import ProjectTemplate, init_example_project
+from sqlmesh.core.analytics import cli_analytics
 from sqlmesh.core.config import load_configs
 from sqlmesh.core.context import Context
 from sqlmesh.utils.date import TimeLike
@@ -121,6 +122,7 @@ def cli(
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def init(
     ctx: click.Context, sql_dialect: t.Optional[str] = None, template: t.Optional[str] = None
 ) -> None:
@@ -146,6 +148,7 @@ def init(
 @click.option("--no-format", is_flag=True, help="Disable fancy formatting of the query.")
 @click.pass_context
 @error_handler
+@cli_analytics
 def render(
     ctx: click.Context,
     model: str,
@@ -184,6 +187,7 @@ def render(
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def evaluate(
     ctx: click.Context,
     model: str,
@@ -200,7 +204,10 @@ def evaluate(
         execution_time=execution_time,
         limit=limit,
     )
-    ctx.obj.console.log_success(df)
+    if hasattr(df, "show"):
+        df.show(limit)
+    else:
+        ctx.obj.console.log_success(df)
 
 
 @cli.command("format")
@@ -250,6 +257,7 @@ def evaluate(
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def format(ctx: click.Context, **kwargs: t.Any) -> None:
     """Format all SQL models."""
     ctx.obj.format(**{k: v for k, v in kwargs.items() if v is not None})
@@ -259,6 +267,7 @@ def format(ctx: click.Context, **kwargs: t.Any) -> None:
 @click.argument("environment")
 @click.pass_context
 @error_handler
+@cli_analytics
 def diff(ctx: click.Context, environment: t.Optional[str] = None) -> None:
     """Show the diff between the local state and the target environment."""
     if ctx.obj.diff(environment, detailed=True):
@@ -303,6 +312,12 @@ def diff(ctx: click.Context, environment: t.Optional[str] = None) -> None:
     help="Create a plan for forward-only changes.",
     default=None,
 )
+@click.option(
+    "--allow-destructive-model",
+    type=str,
+    multiple=True,
+    help="Allow destructive forward-only changes to models whose names match the expression.",
+)
 @click.option(
     "--effective-from",
     type=str,
@@ -365,6 +380,7 @@ def diff(ctx: click.Context, environment: t.Optional[str] = None) -> None:
 @opt.verbose
 @click.pass_context
 @error_handler
+@cli_analytics
 def plan(
     ctx: click.Context, verbose: bool, environment: t.Optional[str] = None, **kwargs: t.Any
 ) -> None:
@@ -372,12 +388,14 @@ def plan(
     context = ctx.obj
     restate_models = kwargs.pop("restate_model") or None
     select_models = kwargs.pop("select_model") or None
+    allow_destructive_models = kwargs.pop("allow_destructive_model") or None
     backfill_models = kwargs.pop("backfill_model") or None
     context.console.verbose = verbose
     context.plan(
         environment,
         restate_models=restate_models,
         select_models=select_models,
+        allow_destructive_models=allow_destructive_models,
         backfill_models=backfill_models,
         **kwargs,
     )
@@ -395,6 +413,7 @@ def plan(
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def run(ctx: click.Context, environment: t.Optional[str] = None, **kwargs: t.Any) -> None:
     """Evaluate missing intervals for the target environment."""
     context = ctx.obj
@@ -413,6 +432,7 @@ def run(ctx: click.Context, environment: t.Optional[str] = None, **kwargs: t.Any
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def invalidate(ctx: click.Context, environment: str, **kwargs: t.Any) -> None:
     """Invalidate the target environment, forcing its removal during the next run of the janitor process."""
     context = ctx.obj
@@ -429,6 +449,7 @@ def invalidate(ctx: click.Context, environment: str, **kwargs: t.Any) -> None:
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def dag(ctx: click.Context, file: str, select_model: t.List[str]) -> None:
     """Render the DAG as an html file."""
     rendered_dag_path = ctx.obj.render_dag(file, select_model)
@@ -488,6 +509,7 @@ def dag(ctx: click.Context, file: str, select_model: t.List[str]) -> None:
 )
 @click.pass_obj
 @error_handler
+@cli_analytics
 def create_test(
     obj: Context,
     model: str,
@@ -522,6 +544,7 @@ def create_test(
 @click.argument("tests", nargs=-1)
 @click.pass_obj
 @error_handler
+@cli_analytics
 def test(
     obj: Context,
     k: t.List[str],
@@ -552,6 +575,7 @@ def test(
 @opt.execution_time
 @click.pass_obj
 @error_handler
+@cli_analytics
 def audit(
     obj: Context,
     models: t.Iterator[str],
@@ -567,6 +591,7 @@ def audit(
 @click.argument("sql")
 @click.pass_context
 @error_handler
+@cli_analytics
 def fetchdf(ctx: click.Context, sql: str) -> None:
     """Run a SQL query and display the results."""
     context = ctx.obj
@@ -574,15 +599,21 @@ def fetchdf(ctx: click.Context, sql: str) -> None:
 
 
 @cli.command("info")
+@click.option(
+    "--skip-connection",
+    is_flag=True,
+    help="Skip the connection test.",
+)
 @click.pass_obj
 @error_handler
-def info(obj: Context) -> None:
+@cli_analytics
+def info(obj: Context, skip_connection: bool) -> None:
     """
     Print information about a SQLMesh project.
 
-    Includes counts of project models and macros and connection tests for the data warehouse and test runner.
+    Includes counts of project models and macros and connection tests for the data warehouse.
     """
-    obj.print_info()
+    obj.print_info(skip_connection=skip_connection)
 
 
 @cli.command("ui")
@@ -606,6 +637,7 @@ def info(obj: Context) -> None:
 )
 @click.pass_context
 @error_handler
+@cli_analytics
 def ui(ctx: click.Context, host: str, port: int, mode: str) -> None:
     """Start a browser-based SQLMesh UI."""
     try:
@@ -636,6 +668,7 @@ def ui(ctx: click.Context, host: str, port: int, mode: str) -> None:
 @cli.command("migrate")
 @click.pass_context
 @error_handler
+@cli_analytics
 def migrate(ctx: click.Context) -> None:
     """Migrate SQLMesh to the current running version."""
     ctx.obj.migrate()
@@ -644,6 +677,7 @@ def migrate(ctx: click.Context) -> None:
 @cli.command("rollback")
 @click.pass_obj
 @error_handler
+@cli_analytics
 def rollback(obj: Context) -> None:
     """Rollback SQLMesh to the previous migration."""
     obj.rollback()
@@ -652,6 +686,7 @@ def rollback(obj: Context) -> None:
 @cli.command("create_external_models")
 @click.pass_obj
 @error_handler
+@cli_analytics
 def create_external_models(obj: Context) -> None:
     """Create a schema file containing external model schemas."""
     obj.create_external_models()
@@ -683,8 +718,16 @@ def create_external_models(obj: Context) -> None:
     is_flag=True,
     help="Show a sample of the rows that differ. With many columns, the output can be very wide.",
 )
+@click.option(
+    "-d",
+    "--decimals",
+    type=int,
+    default=3,
+    help="The number of decimal places to keep when comparing floating point columns. Default: 3",
+)
 @click.pass_obj
 @error_handler
+@cli_analytics
 def table_diff(
     obj: Context, source_to_target: str, model: t.Optional[str], **kwargs: t.Any
 ) -> None:
@@ -712,6 +755,7 @@ def table_diff(
 )
 @click.pass_obj
 @error_handler
+@cli_analytics
 def rewrite(obj: Context, sql: str, read: str = "", write: str = "") -> None:
     """Rewrite a SQL expression with semantic references into an executable query.
 
@@ -740,6 +784,7 @@ def rewrite(obj: Context, sql: str, read: str = "", write: str = "") -> None:
 @opt.verbose
 @click.pass_context
 @error_handler
+@cli_analytics
 def prompt(
     ctx: click.Context, prompt: str, evaluate: bool, temperature: float, verbose: bool
 ) -> None:
@@ -764,6 +809,7 @@ def prompt(
 @cli.command("clean")
 @click.pass_obj
 @error_handler
+@cli_analytics
 def clean(obj: Context) -> None:
     """Clears the SQLMesh cache and any build artifacts."""
     obj.clear_caches()
@@ -779,6 +825,7 @@ def clean(obj: Context) -> None:
 )
 @click.pass_obj
 @error_handler
+@cli_analytics
 def table_name(obj: Context, model_name: str, dev: bool) -> None:
     """Prints the name of the physical table for the given model."""
     print(obj.table_name(model_name, dev))
