@@ -7,7 +7,7 @@ from sqlmesh.core.audit.definition import ModelAudit
 # not_null(columns=(column_1, column_2))
 not_null_audit = ModelAudit(
     name="not_null",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -27,7 +27,7 @@ WHERE @AND(
 # unique_values(columns=(column_1, column_2))
 unique_values_audit = ModelAudit(
     name="unique_values",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM (
@@ -37,7 +37,7 @@ FROM (
       c -> row_number() OVER (PARTITION BY c ORDER BY c) AS rank_@c
     )
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 )
 WHERE @REDUCE(
   @EACH(
@@ -52,7 +52,7 @@ WHERE @REDUCE(
 # accepted_values(column=column_name, is_in=(1, 2, 3))
 accepted_values_audit = ModelAudit(
     name="accepted_values",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -63,13 +63,13 @@ WHERE @AND(@column NOT IN @is_in, @condition)
 # number_of_rows(threshold=100)
 number_of_rows_audit = ModelAudit(
     name="number_of_rows",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT COUNT(*)
 FROM (
   SELECT 1
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
   LIMIT @threshold + 1
 )
 HAVING COUNT(*) <= @threshold
@@ -83,16 +83,21 @@ HAVING COUNT(*) <= @threshold
 # ))
 forall_audit = ModelAudit(
     name="forall",
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
-WHERE @REDUCE(
-  @EACH(
-    @criteria,
-    c -> NOT (c)
-  ),
-  (l, r) -> l OR r
-)
+WHERE
+  @AND(
+    @REDUCE(
+      @EACH(
+        @criteria,
+        c -> NOT (c)
+      ),
+      (l, r) -> l OR r
+    ),
+    @condition,
+  )
     """,
 )
 
@@ -105,7 +110,7 @@ accepted_range_audit = ModelAudit(
         "min_v": exp.null(),
         "max_v": exp.null(),
         "inclusive": exp.true(),
-        "condition": exp.null(),
+        "condition": exp.true(),
     },
     query="""
 SELECT *
@@ -126,11 +131,11 @@ WHERE
 # at_least_one(column=column_name)
 at_least_one_audit = ModelAudit(
     name="at_least_one",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT 1
 FROM @this_model
-@WHERE(@condition) @condition
+WHERE @condition
 GROUP BY 1
 HAVING COUNT(@column) = 0
     """,
@@ -139,13 +144,13 @@ HAVING COUNT(@column) = 0
 # not_constant(column=column_name)
 not_constant_audit = ModelAudit(
     name="not_constant",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT 1
 FROM (
   SELECT COUNT(DISTINCT @column) AS t_cardinality
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 ) AS r
 WHERE r.t_cardinality <= 1
     """,
@@ -154,7 +159,7 @@ WHERE r.t_cardinality <= 1
 # not_empty_string(column=column_name)
 not_empty_string_audit = ModelAudit(
     name="not_empty_string",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -165,7 +170,7 @@ WHERE @AND(@column = '', @condition)
 # not_null_proportion(column=column_name, threshold=0.9)
 not_null_proportion_audit = ModelAudit(
     name="not_null_proportion",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM (
@@ -174,7 +179,7 @@ FROM (
     count(@column) as cnt_not_null,
     count(*) - count(@column) as cnt_null
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 ) AS s
 WHERE s.cnt_not_null <= s.cnt_tot * @threshold
     """,
@@ -183,7 +188,7 @@ WHERE s.cnt_not_null <= s.cnt_tot * @threshold
 # not_accepted_values(column=column_name, is_in=(1, 2, 3))
 not_accepted_values_audit = ModelAudit(
     name="not_accepted_values",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -195,7 +200,7 @@ WHERE @AND(@column IN @is_in, @condition)
 # TODO: support grouping
 sequential_values_audit = ModelAudit(
     name="sequential_values",
-    defaults={"interval": exp.Literal.number(1), "condition": exp.null()},
+    defaults={"interval": exp.Literal.number(1), "condition": exp.true()},
     query="""
 WITH windowed AS (
   SELECT
@@ -204,7 +209,7 @@ WITH windowed AS (
       ORDER BY @column
     ) AS prv
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 ), validation_errors AS (
     SELECT *
     FROM windowed
@@ -219,11 +224,11 @@ FROM validation_errors
 # unique_combination_of_columns(columns=(column_1, column_2))
 unique_combination_of_columns_audit = ModelAudit(
     name="unique_combination_of_columns",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT @EACH(@columns, c -> c)
 FROM @this_model
-@WHERE(@condition) @condition
+WHERE @condition
 GROUP BY @EACH(@columns, c -> c)
 HAVING COUNT(*) > 1
     """,
@@ -233,7 +238,7 @@ HAVING COUNT(*) > 1
 # TODO: make inclusivity configurable
 mutually_exclusive_ranges_audit = ModelAudit(
     name="mutually_exclusive_ranges",
-    defaults={"partition_clause": exp.false(), "condition": exp.null()},
+    defaults={"partition_clause": exp.false(), "condition": exp.true()},
     query="""
 WITH window_functions AS (
   SELECT
@@ -248,7 +253,7 @@ WITH window_functions AS (
       ORDER BY @lower_bound_column desc, @upper_bound_column desc
     ) = 1 AS is_last_record
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 ), calc AS (
   SELECT
     *,
@@ -279,7 +284,7 @@ FROM validation_errors
 # valid_uuid(column=column_name)
 valid_uuid_audit = ModelAudit(
     name="valid_uuid",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -294,7 +299,7 @@ WHERE @AND(
 # valid_url(column=column_name)
 valid_url_audit = ModelAudit(
     name="valid_url",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query=r"""
 SELECT *
 FROM @this_model
@@ -315,7 +320,7 @@ WHERE NOT @column IN ('GET', 'POST', 'PUT', 'DELETE', 'PATCH', 'HEAD', 'OPTIONS'
 # valid_email(column=column_name)
 valid_email_audit = ModelAudit(
     name="valid_email",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query=r"""
 SELECT *
 FROM @this_model
@@ -326,7 +331,7 @@ WHERE @AND(NOT REGEXP_LIKE(@column, '^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9
 # match_regex_pattern_list(column=column_name, patterns=('^pattern_1', 'pattern_2$'))
 match_regex_pattern_list_audit = ModelAudit(
     name="match_regex_pattern_list",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -346,7 +351,7 @@ WHERE @AND(
 # not_match_regex_pattern_list(column=column_name, patterns=('^pattern_1', 'pattern_2$'))
 not_match_regex_pattern_list_audit = ModelAudit(
     name="not_match_regex_pattern_list",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -385,7 +390,7 @@ WHERE @condition AND (
 # not_match_like_pattern_list(column=column_name, patterns=('%pattern_1%', 'pattern_2%'))
 not_match_like_pattern_list_audit = ModelAudit(
     name="not_match_like_pattern_list",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -405,14 +410,14 @@ WHERE @AND(
 # z_score_audit(column=column_name, threshold=3)
 z_score_audit = ModelAudit(
     name="z_score",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 WITH stats AS (
   SELECT
     AVG(@column) AS mean_@column,
     STDDEV(@column) AS stddev_@column
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 )
 SELECT
   @column,
@@ -429,7 +434,7 @@ string_length_between_audit = ModelAudit(
         "min_v": exp.null(),
         "max_v": exp.null(),
         "inclusive": exp.true(),
-        "condition": exp.null(),
+        "condition": exp.true(),
     },
     query="""
 SELECT *
@@ -450,7 +455,7 @@ WHERE
 # string_length_equal_audit(column=column_name, v=22)
 string_length_equal_audit = ModelAudit(
     name="string_length_equal",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 SELECT *
 FROM @this_model
@@ -465,14 +470,14 @@ stddev_in_range_audit = ModelAudit(
         "min_v": exp.null(),
         "max_v": exp.null(),
         "inclusive": exp.true(),
-        "condition": exp.null(),
+        "condition": exp.true(),
     },
     query="""
 SELECT *
 FROM (
   SELECT STDDEV(@column) AS stddev_@column
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 )
 WHERE
   @OR(
@@ -491,14 +496,14 @@ mean_in_range_audit = ModelAudit(
         "min_v": exp.null(),
         "max_v": exp.null(),
         "inclusive": exp.true(),
-        "condition": exp.null(),
+        "condition": exp.true(),
     },
     query="""
 SELECT *
 FROM (
   SELECT AVG(@column) AS mean_@column
   FROM @this_model
-  @WHERE(@condition) @condition
+  WHERE @condition
 )
 WHERE
   @OR(
@@ -513,7 +518,7 @@ WHERE
 # kl_divergence(column=age, target_column=normalized_age, threshold=0.1)
 kl_divergence_audit = ModelAudit(
     name="kl_divergence",
-    defaults={"condition": exp.null()},
+    defaults={"condition": exp.true()},
     query="""
 WITH
   table_a AS (
@@ -521,7 +526,7 @@ WITH
       @source_column,
       COUNT(*) AS num_rows
     FROM @this_model
-    @WHERE(@condition) @condition
+    WHERE @condition
     GROUP BY @source_column
   ),
   table_b AS (
@@ -529,7 +534,7 @@ WITH
       @target_column,
       COUNT(*) AS num_rows
     FROM @this_model
-    @WHERE(@condition) @condition
+    WHERE @condition
     GROUP BY @target_column
   ),
   table_a_with_p AS (
@@ -612,7 +617,7 @@ chi_square_audit = ModelAudit(
         "@def(r, (SELECT COUNT(DISTINCT x_b) FROM contingency_table))",
         "@def(E, (tot_a * tot_b / g_t))",
     ],
-    defaults={"dependent": exp.true(), "condition": exp.null()},
+    defaults={"dependent": exp.true(), "condition": exp.true()},
     query="""
 WITH
   samples AS (
