@@ -8,6 +8,7 @@ import pathlib
 import sys
 import typing as t
 from enum import Enum
+from functools import partial
 
 from pydantic import Field
 from sqlglot import exp
@@ -43,6 +44,7 @@ class ConnectionConfig(abc.ABC, BaseConfig):
     type_: str
     concurrent_tasks: int
     register_comments: bool
+    pre_ping: bool
 
     @property
     @abc.abstractmethod
@@ -84,20 +86,31 @@ class ConnectionConfig(abc.ABC, BaseConfig):
         """Whether this connection is recommended for being used as a state sync for production state syncs"""
         return self.type_ in RECOMMENDED_STATE_SYNC_ENGINES
 
+    @property
+    def _connection_factory_with_kwargs(self) -> t.Callable[[], t.Any]:
+        """A function that is called to return a connection object for the given Engine Adapter"""
+        return partial(
+            self._connection_factory,
+            **{
+                **self._static_connection_kwargs,
+                **{k: v for k, v in self.dict().items() if k in self._connection_kwargs_keys},
+            },
+        )
+
+    def connection_validator(self) -> t.Callable[[], None]:
+        """A function that validates the connection configuration"""
+        return self.create_engine_adapter().ping
+
     def create_engine_adapter(self, register_comments_override: bool = False) -> EngineAdapter:
         """Returns a new instance of the Engine Adapter."""
         return self._engine_adapter(
-            lambda: self._connection_factory(
-                **{
-                    **self._static_connection_kwargs,
-                    **{k: v for k, v in self.dict().items() if k in self._connection_kwargs_keys},
-                }
-            ),
+            self._connection_factory_with_kwargs,
             multithreaded=self.concurrent_tasks > 1,
             cursor_kwargs=self._cursor_kwargs,
             default_catalog=self.get_catalog(),
             cursor_init=self._cursor_init,
             register_comments=register_comments_override or self.register_comments,
+            pre_ping=self.pre_ping,
             **self._extra_engine_config,
         )
 
@@ -120,6 +133,7 @@ class BaseDuckDBConnectionConfig(ConnectionConfig):
         connector_config: A dictionary of configuration to pass into the duckdb connector.
         concurrent_tasks: The maximum number of tasks that can use this connection concurrently.
         register_comments: Whether or not to register model comments with the SQL engine.
+        pre_ping: Whether or not to pre-ping the connection before starting a new transaction to ensure it is still alive.
     """
 
     extensions: t.List[str] = []
@@ -127,6 +141,7 @@ class BaseDuckDBConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: Literal[1] = 1
     register_comments: bool = True
+    pre_ping: Literal[False] = False
 
     @property
     def _engine_adapter(self) -> t.Type[EngineAdapter]:
@@ -158,21 +173,28 @@ class BaseDuckDBConnectionConfig(ConnectionConfig):
                 except Exception as e:
                     raise ConfigError(f"Failed to set connector config {field} to {setting}: {e}")
 
-            for i, (alias, path) in enumerate((getattr(self, "catalogs", None) or {}).items()):
+            for i, (alias, path_options) in enumerate(
+                (getattr(self, "catalogs", None) or {}).items()
+            ):
                 # we parse_identifier and generate to ensure that `alias` has exactly one set of quotes
                 # regardless of whether it comes in quoted or not
                 alias = exp.parse_identifier(alias, dialect="duckdb").sql(
                     identify=True, dialect="duckdb"
                 )
                 try:
-                    cursor.execute(f"""ATTACH '{path}' AS {alias}""")
+                    query = (
+                        path_options.to_sql(alias)
+                        if isinstance(path_options, DuckDBAttachOptions)
+                        else f"ATTACH '{path_options}' AS {alias}"
+                    )
+                    cursor.execute(query)
                 except BinderException as e:
                     # If a user tries to create a catalog pointing at `:memory:` and with the name `memory`
                     # then we don't want to raise since this happens by default. They are just doing this to
                     # set it as the default catalog.
                     if not (
                         'database with name "memory" already exists' in str(e)
-                        and path == ":memory:"
+                        and path_options == ":memory:"
                     ):
                         raise e
                 if i == 0 and not getattr(self, "database", None):
@@ -201,10 +223,32 @@ class MotherDuckConnectionConfig(BaseDuckDBConnectionConfig):
     @property
     def _static_connection_kwargs(self) -> t.Dict[str, t.Any]:
         """kwargs that are for execution config only"""
+        from sqlmesh import __version__
+
         connection_str = f"md:{self.database}"
         if self.token:
             connection_str += f"?motherduck_token={self.token}"
-        return {"database": connection_str}
+        return {
+            "database": connection_str,
+            "config": {"custom_user_agent": f"SQLMesh/{__version__}"},
+        }
+
+
+class DuckDBAttachOptions(BaseConfig):
+    type: str
+    path: str
+    read_only: bool = False
+
+    def to_sql(self, alias: str) -> str:
+        options = []
+        # 'duckdb' is actually not a supported type, but we'd like to allow it for
+        # fully qualified attach options or integration testing, similar to duckdb-dbt
+        if self.type != "duckdb":
+            options.append(f"TYPE {self.type.upper()}")
+        if self.read_only:
+            options.append("READ_ONLY")
+        options_sql = f" ({', '.join(options)})" if options else ""
+        return f"ATTACH '{self.path}' AS {alias}{options_sql}"
 
 
 class DuckDBConnectionConfig(BaseDuckDBConnectionConfig):
@@ -216,7 +260,7 @@ class DuckDBConnectionConfig(BaseDuckDBConnectionConfig):
     """
 
     database: t.Optional[str] = None
-    catalogs: t.Optional[t.Dict[str, str]] = None
+    catalogs: t.Optional[t.Dict[str, t.Union[str, DuckDBAttachOptions]]] = None
 
     type_: Literal["duckdb"] = Field(alias="type", default="duckdb")
 
@@ -246,18 +290,19 @@ class DuckDBConnectionConfig(BaseDuckDBConnectionConfig):
             data_files.add(self.database)
         data_files.discard(":memory:")
         for data_file in data_files:
-            if adapter := DuckDBConnectionConfig._data_file_to_adapter.get(data_file):
-                logger.info(
-                    f"Using existing DuckDB adapter due to overlapping data file: {data_file}"
-                )
+            key = data_file if isinstance(data_file, str) else data_file.path
+            if adapter := DuckDBConnectionConfig._data_file_to_adapter.get(key):
+                logger.info(f"Using existing DuckDB adapter due to overlapping data file: {key}")
                 return adapter
+
         if data_files:
             logger.info(f"Creating new DuckDB adapter for data files: {data_files}")
         else:
             logger.info("Creating new DuckDB adapter for in-memory database")
         adapter = super().create_engine_adapter(register_comments_override)
         for data_file in data_files:
-            DuckDBConnectionConfig._data_file_to_adapter[data_file] = adapter
+            key = data_file if isinstance(data_file, str) else data_file.path
+            DuckDBConnectionConfig._data_file_to_adapter[key] = adapter
         return adapter
 
     def get_catalog(self) -> t.Optional[str]:
@@ -287,6 +332,7 @@ class SnowflakeConnectionConfig(ConnectionConfig):
         private_key_path: The optional path to the private key to use for authentication. This would be used instead of `private_key`.
         private_key_passphrase: The optional passphrase to use to decrypt `private_key` or `private_key_path`. Keys can be created without encryption so only provide this if needed.
         register_comments: Whether or not to register model comments with the SQL engine.
+        pre_ping: Whether or not to pre-ping the connection before starting a new transaction to ensure it is still alive.
     """
 
     account: str
@@ -305,6 +351,7 @@ class SnowflakeConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = False
 
     type_: Literal["snowflake"] = Field(alias="type", default="snowflake")
 
@@ -464,6 +511,8 @@ class DatabricksConnectionConfig(ConnectionConfig):
             Defaults to deriving the cluster id from the `http_path` value.
         force_databricks_connect: Force all queries to run using Databricks Connect instead of the SQL connector.
         disable_databricks_connect: Even if databricks connect is installed, do not use it.
+        disable_spark_session: Do not use SparkSession if it is available (like when running in a notebook).
+        pre_ping: Whether or not to pre-ping the connection before starting a new transaction to ensure it is still alive.
     """
 
     server_hostname: t.Optional[str] = None
@@ -477,9 +526,11 @@ class DatabricksConnectionConfig(ConnectionConfig):
     databricks_connect_cluster_id: t.Optional[str] = None
     force_databricks_connect: bool = False
     disable_databricks_connect: bool = False
+    disable_spark_session: bool = False
 
     concurrent_tasks: int = 1
     register_comments: bool = True
+    pre_ping: Literal[False] = False
 
     type_: Literal["databricks"] = Field(alias="type", default="databricks")
 
@@ -489,12 +540,11 @@ class DatabricksConnectionConfig(ConnectionConfig):
     @model_validator(mode="before")
     @model_validator_v1_args
     def _databricks_connect_validator(cls, values: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
-        from sqlmesh import RuntimeEnv
         from sqlmesh.core.engine_adapter.databricks import DatabricksEngineAdapter
 
-        runtime_env = RuntimeEnv.get()
-
-        if runtime_env.is_databricks:
+        if DatabricksEngineAdapter.can_access_spark_session(
+            bool(values.get("disable_spark_session"))
+        ):
             return values
         server_hostname, http_path, access_token = (
             values.get("server_hostname"),
@@ -505,16 +555,15 @@ class DatabricksConnectionConfig(ConnectionConfig):
             raise ValueError(
                 "`server_hostname`, `http_path`, and `access_token` are required for Databricks connections when not running in a notebook"
             )
-        if DatabricksEngineAdapter.can_access_spark_session:
+        if DatabricksEngineAdapter.can_access_databricks_connect(
+            bool(values.get("disable_databricks_connect"))
+        ):
             if not values.get("databricks_connect_server_hostname"):
                 values["databricks_connect_server_hostname"] = f"https://{server_hostname}"
             if not values.get("databricks_connect_access_token"):
                 values["databricks_connect_access_token"] = access_token
             if not values.get("databricks_connect_cluster_id"):
                 values["databricks_connect_cluster_id"] = http_path.split("/")[-1]
-        if not values.get("session_configuration"):
-            values["session_configuration"] = {}
-        values["session_configuration"]["spark.sql.sources.partitionOverwriteMode"] = "dynamic"
         return values
 
     @property
@@ -539,14 +588,18 @@ class DatabricksConnectionConfig(ConnectionConfig):
         return {
             k: v
             for k, v in self.dict().items()
-            if k.startswith("databricks_connect_") or k in ("catalog", "disable_databricks_connect")
+            if k.startswith("databricks_connect_")
+            or k in ("catalog", "disable_databricks_connect", "disable_spark_session")
         }
 
     @property
     def use_spark_session_only(self) -> bool:
-        from sqlmesh import RuntimeEnv
+        from sqlmesh.core.engine_adapter.databricks import DatabricksEngineAdapter
 
-        return RuntimeEnv.get().is_databricks or self.force_databricks_connect
+        return (
+            DatabricksEngineAdapter.can_access_spark_session(self.disable_spark_session)
+            or self.force_databricks_connect
+        )
 
     @property
     def _connection_factory(self) -> t.Callable:
@@ -561,12 +614,14 @@ class DatabricksConnectionConfig(ConnectionConfig):
 
     @property
     def _static_connection_kwargs(self) -> t.Dict[str, t.Any]:
-        from sqlmesh import RuntimeEnv
+        from sqlmesh.core.engine_adapter.databricks import DatabricksEngineAdapter
 
         if not self.use_spark_session_only:
-            return {}
+            return {
+                "_user_agent_entry": "sqlmesh",
+            }
 
-        if RuntimeEnv.get().is_databricks:
+        if DatabricksEngineAdapter.can_access_spark_session(self.disable_spark_session):
             from pyspark.sql import SparkSession
 
             return dict(
@@ -581,7 +636,9 @@ class DatabricksConnectionConfig(ConnectionConfig):
                 host=self.databricks_connect_server_hostname,
                 token=self.databricks_connect_access_token,
                 cluster_id=self.databricks_connect_cluster_id,
-            ).getOrCreate(),
+            )
+            .userAgent("sqlmesh")
+            .getOrCreate(),
             catalog=self.catalog,
         )
 
@@ -644,6 +701,7 @@ class BigQueryConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 1
     register_comments: bool = True
+    pre_ping: Literal[False] = False
 
     type_: Literal["bigquery"] = Field(alias="type", default="bigquery")
 
@@ -730,6 +788,7 @@ class GCPPostgresConnectionConfig(ConnectionConfig):
         password: The postgres user's password. Only needed when the user is a postgres user.
         enable_iam_auth: Set to True when user is an IAM user.
         db: Name of the db to connect to.
+        pre_ping: Whether or not to pre-ping the connection before starting a new transaction to ensure it is still alive.
     """
 
     instance_connection_string: str
@@ -743,6 +802,7 @@ class GCPPostgresConnectionConfig(ConnectionConfig):
     type_: Literal["gcp_postgres"] = Field(alias="type", default="gcp_postgres")
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = True
 
     @model_validator(mode="before")
     @model_validator_v1_args
@@ -817,6 +877,7 @@ class RedshiftConnectionConfig(ConnectionConfig):
         is_serverless: Redshift end-point is serverless or provisional. Default value false.
         serverless_acct_id: The account ID of the serverless. Default value None
         serverless_work_group: The name of work group for serverless end point. Default value None.
+        pre_ping: Whether or not to pre-ping the connection before starting a new transaction to ensure it is still alive.
     """
 
     user: t.Optional[str] = None
@@ -843,6 +904,7 @@ class RedshiftConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = False
 
     type_: Literal["redshift"] = Field(alias="type", default="redshift")
 
@@ -896,6 +958,7 @@ class PostgresConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = True
 
     type_: Literal["postgres"] = Field(alias="type", default="postgres")
 
@@ -934,6 +997,7 @@ class MySQLConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = True
 
     type_: Literal["mysql"] = Field(alias="type", default="mysql")
 
@@ -986,6 +1050,7 @@ class MSSQLConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: bool = True
 
     type_: Literal["mssql"] = Field(alias="type", default="mssql")
 
@@ -1028,6 +1093,7 @@ class SparkConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: Literal[False] = False
 
     type_: Literal["spark"] = Field(alias="type", default="spark")
 
@@ -1140,6 +1206,7 @@ class TrinoConnectionConfig(ConnectionConfig):
 
     concurrent_tasks: int = 4
     register_comments: bool = True
+    pre_ping: Literal[False] = False
 
     type_: Literal["trino"] = Field(alias="type", default="trino")
 
