@@ -464,7 +464,7 @@ class TerminalConsole(Console):
         """Indicates that a new migration progress has begun."""
         if self.migration_progress is None:
             self.migration_progress = Progress(
-                TextColumn(f"[bold blue]Migrating snapshots", justify="right"),
+                TextColumn("[bold blue]Migrating snapshots", justify="right"),
                 BarColumn(bar_width=40),
                 "[progress.percentage]{task.percentage:>3.1f}%",
                 "•",
@@ -476,7 +476,7 @@ class TerminalConsole(Console):
 
             self.migration_progress.start()
             self.migration_task = self.migration_progress.add_task(
-                f"Migrating snapshots...",
+                "Migrating snapshots...",
                 total=total_tasks,
             )
 
@@ -590,7 +590,7 @@ class TerminalConsole(Console):
         environment_naming_info: EnvironmentNamingInfo,
         default_catalog: t.Optional[str],
     ) -> Tree:
-        ignored = Tree(f"[bold][ignored]Ignored Models (Expected Plan Start):")
+        ignored = Tree("[bold][ignored]Ignored Models (Expected Plan Start):")
         for s_id in ignored_snapshot_ids:
             snapshot = snapshots[s_id]
             ignored.add(
@@ -642,7 +642,7 @@ class TerminalConsole(Console):
 
         tree = Tree(f"[bold]{header}:")
         if added_snapshot_ids:
-            added_tree = Tree(f"[bold][added]Added:")
+            added_tree = Tree("[bold][added]Added:")
             for s_id in added_snapshot_ids:
                 snapshot = context_diff.snapshots[s_id]
                 added_tree.add(
@@ -650,7 +650,7 @@ class TerminalConsole(Console):
                 )
             tree.add(added_tree)
         if removed_snapshot_ids:
-            removed_tree = Tree(f"[bold][removed]Removed:")
+            removed_tree = Tree("[bold][removed]Removed:")
             for s_id in removed_snapshot_ids:
                 snapshot_table_info = context_diff.removed_snapshots[s_id]
                 removed_tree.add(
@@ -658,9 +658,9 @@ class TerminalConsole(Console):
                 )
             tree.add(removed_tree)
         if modified_snapshot_ids:
-            direct = Tree(f"[bold][direct]Directly Modified:")
-            indirect = Tree(f"[bold][indirect]Indirectly Modified:")
-            metadata = Tree(f"[bold][metadata]Metadata Updated:")
+            direct = Tree("[bold][direct]Directly Modified:")
+            indirect = Tree("[bold][indirect]Indirectly Modified:")
+            metadata = Tree("[bold][metadata]Metadata Updated:")
             for s_id in modified_snapshot_ids:
                 name = s_id.name
                 display_name = context_diff.snapshots[s_id].display_name(
@@ -745,7 +745,7 @@ class TerminalConsole(Console):
             for child_sid in sorted(plan.indirectly_modified.get(snapshot.snapshot_id, set())):
                 child_snapshot = plan.context_diff.snapshots[child_sid]
                 if not indirect_tree:
-                    indirect_tree = Tree(f"[indirect]Indirectly Modified Children:")
+                    indirect_tree = Tree("[indirect]Indirectly Modified Children:")
                     tree.add(indirect_tree)
                 indirect_tree.add(
                     f"[indirect]{child_snapshot.display_name(plan.environment_naming_info, default_catalog)}"
@@ -771,7 +771,7 @@ class TerminalConsole(Console):
             for child_sid in sorted(plan.indirectly_modified.get(snapshot.snapshot_id, set())):
                 child_snapshot = context_diff.snapshots[child_sid]
                 if not indirect_tree:
-                    indirect_tree = Tree(f"[indirect]Indirectly Modified Children:")
+                    indirect_tree = Tree("[indirect]Indirectly Modified Children:")
                     tree.add(indirect_tree)
                 child_category_str = SNAPSHOT_CHANGE_CATEGORY_STR[child_snapshot.change_category]
                 indirect_tree.add(
@@ -868,7 +868,7 @@ class TerminalConsole(Console):
 
     def _prompt_promote(self, plan_builder: PlanBuilder) -> None:
         if self._confirm(
-            f"Apply - Virtual Update",
+            "Apply - Virtual Update",
         ):
             plan_builder.apply()
 
@@ -966,9 +966,22 @@ class TerminalConsole(Console):
             target_name = row_diff.target_alias.upper()
 
         tree = Tree("[b]Row Counts:[/b]")
-        tree.add(f" [b][blue]COMMON[/blue]:[/b] {row_diff.join_count} rows")
-        tree.add(f" [b][yellow]{source_name} ONLY[/yellow]:[/b] {row_diff.s_only_count} rows")
-        tree.add(f" [b][green]{target_name} ONLY[/green]:[/b] {row_diff.t_only_count} rows")
+        if row_diff.full_match_count:
+            tree.add(
+                f" [b][cyan]FULL MATCH[/cyan]:[/b] {row_diff.full_match_count} rows ({row_diff.full_match_pct}%)"
+            )
+        if row_diff.partial_match_count:
+            tree.add(
+                f" [b][blue]PARTIAL MATCH[/blue]:[/b] {row_diff.partial_match_count} rows ({row_diff.partial_match_pct}%)"
+            )
+        if row_diff.s_only_count:
+            tree.add(
+                f" [b][yellow]{source_name} ONLY[/yellow]:[/b] {row_diff.s_only_count} rows ({row_diff.s_only_pct}%)"
+            )
+        if row_diff.t_only_count:
+            tree.add(
+                f" [b][green]{target_name} ONLY[/green]:[/b] {row_diff.t_only_count} rows ({row_diff.t_only_pct}%)"
+            )
         self.console.print("\n", tree)
 
         self.console.print("\n[b][blue]COMMON ROWS[/blue] column comparison stats:[/b]")
@@ -1072,10 +1085,12 @@ class NotebookMagicConsole(TerminalConsole):
         **kwargs: t.Any,
     ) -> None:
         import ipywidgets as widgets
+        from IPython import get_ipython
         from IPython.display import display as ipython_display
 
         super().__init__(console, **kwargs)
-        self.display = display or get_ipython().user_ns.get("display", ipython_display)  # type: ignore
+
+        self.display = display or get_ipython().user_ns.get("display", ipython_display)
         self.missing_dates_output = widgets.Output()
         self.dynamic_options_after_categorization_output = widgets.VBox()
 
@@ -1406,16 +1421,16 @@ class MarkdownConsole(CaptureTerminalConsole):
         ignored_snapshot_ids = ignored_snapshot_ids or set()
         if context_diff.is_new_environment:
             self._print(
-                f"**New environment `{context_diff.environment}` will be created from `{context_diff.create_from}`**\n\n"
+                f"**New environment `{context_diff.environment}` will be created from `{context_diff.create_from}`**\n"
             )
             if not context_diff.has_snapshot_changes:
                 return
 
         if not context_diff.has_changes:
-            self._print(f"**No differences when compared to `{context_diff.environment}`**\n\n")
+            self._print(f"**No differences when compared to `{context_diff.environment}`**\n")
             return
 
-        self._print(f"**Summary of differences against `{context_diff.environment}`:**\n\n")
+        self._print(f"**Summary of differences against `{context_diff.environment}`:**\n")
 
         added_snapshots = {
             context_diff.snapshots[s_id]
@@ -1424,21 +1439,19 @@ class MarkdownConsole(CaptureTerminalConsole):
         }
         added_snapshot_models = {s for s in added_snapshots if s.is_model}
         if added_snapshot_models:
-            self._print(f"**Added Models:**\n")
-            for snapshot in added_snapshot_models:
+            self._print("\n**Added Models:**")
+            for snapshot in sorted(added_snapshot_models):
                 self._print(
-                    f"- {snapshot.display_name(environment_naming_info, default_catalog)}\n"
+                    f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`"
                 )
-            self._print("\n")
 
         added_snapshot_audits = {s for s in added_snapshots if s.is_audit}
         if added_snapshot_audits:
-            self._print(f"**Added Standalone Audits:**\n")
-            for snapshot in added_snapshot_audits:
+            self._print("\n**Added Standalone Audits:**")
+            for snapshot in sorted(added_snapshot_audits):
                 self._print(
-                    f"- {snapshot.display_name(environment_naming_info, default_catalog)}\n"
+                    f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`"
                 )
-            self._print("\n")
 
         removed_snapshot_table_infos = {
             snapshot_table_info
@@ -1447,21 +1460,19 @@ class MarkdownConsole(CaptureTerminalConsole):
         }
         removed_model_snapshot_table_infos = {s for s in removed_snapshot_table_infos if s.is_model}
         if removed_model_snapshot_table_infos:
-            self._print(f"**Removed Models:**\n")
-            for snapshot_table_info in removed_model_snapshot_table_infos:
+            self._print("\n**Removed Models:**")
+            for snapshot_table_info in sorted(removed_model_snapshot_table_infos):
                 self._print(
-                    f"- {snapshot_table_info.display_name(environment_naming_info, default_catalog)}\n"
+                    f"- `{snapshot_table_info.display_name(environment_naming_info, default_catalog)}`"
                 )
-            self._print("\n")
 
         removed_audit_snapshot_table_infos = {s for s in removed_snapshot_table_infos if s.is_audit}
         if removed_audit_snapshot_table_infos:
-            self._print(f"**Removed Standalone Audits:**\n")
-            for snapshot_table_info in removed_audit_snapshot_table_infos:
+            self._print("\n**Removed Standalone Audits:**")
+            for snapshot_table_info in sorted(removed_audit_snapshot_table_infos):
                 self._print(
-                    f"- {snapshot_table_info.display_name(environment_naming_info, default_catalog)}\n"
+                    f"- `{snapshot_table_info.display_name(environment_naming_info, default_catalog)}`"
                 )
-            self._print("\n")
 
         modified_snapshots = {
             current_snapshot
@@ -1480,43 +1491,39 @@ class MarkdownConsole(CaptureTerminalConsole):
                 elif context_diff.metadata_updated(snapshot.name):
                     metadata_modified.append(snapshot)
             if directly_modified:
-                self._print(f"**Directly Modified:**\n")
-                for snapshot in directly_modified:
+                self._print("\n**Directly Modified:**")
+                for snapshot in sorted(directly_modified):
                     self._print(
-                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`\n"
+                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`"
                     )
                     if not no_diff:
-                        self._print(f"```diff\n{context_diff.text_diff(snapshot.name)}\n```\n")
-                self._print("\n")
+                        self._print(f"```diff\n{context_diff.text_diff(snapshot.name)}\n```")
             if indirectly_modified:
-                self._print(f"**Indirectly Modified:**\n")
-                for snapshot in indirectly_modified:
+                self._print("\n**Indirectly Modified:**")
+                for snapshot in sorted(indirectly_modified):
                     self._print(
-                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`\n"
+                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`"
                     )
-                self._print("\n")
             if metadata_modified:
-                self._print(f"**Metadata Updated:**\n")
-                for snapshot in metadata_modified:
+                self._print("\n**Metadata Updated:**")
+                for snapshot in sorted(metadata_modified):
                     self._print(
-                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`\n"
+                        f"- `{snapshot.display_name(environment_naming_info, default_catalog)}`"
                     )
-                self._print("\n")
         if ignored_snapshot_ids:
-            self._print(f"**Ignored Models (Expected Plan Start):**\n")
-            for s_id in ignored_snapshot_ids:
+            self._print("\n**Ignored Models (Expected Plan Start):**")
+            for s_id in sorted(ignored_snapshot_ids):
                 snapshot = context_diff.snapshots[s_id]
                 self._print(
-                    f"- `{snapshot.display_name(environment_naming_info, default_catalog)}` ({snapshot.get_latest(start_date(snapshot, context_diff.snapshots.values()))})\n"
+                    f"- `{snapshot.display_name(environment_naming_info, default_catalog)}` ({snapshot.get_latest(start_date(snapshot, context_diff.snapshots.values()))})"
                 )
-            self._print("\n")
 
     def _show_missing_dates(self, plan: Plan, default_catalog: t.Optional[str]) -> None:
         """Displays the models with missing dates."""
         missing_intervals = plan.missing_intervals
         if not missing_intervals:
             return
-        self._print("**Models needing backfill (missing dates):**\n\n")
+        self._print("\n**Models needing backfill (missing dates):**")
         for missing in missing_intervals:
             snapshot = plan.context_diff.snapshots[missing.snapshot_id]
             if not snapshot.is_model:
@@ -1527,9 +1534,8 @@ class MarkdownConsole(CaptureTerminalConsole):
                 preview_modifier = " (**preview**)"
 
             self._print(
-                f"* `{snapshot.display_name(plan.environment_naming_info, default_catalog)}`: {missing.format_intervals(snapshot.node.interval_unit)}{preview_modifier}\n"
+                f"* `{snapshot.display_name(plan.environment_naming_info, default_catalog)}`: {missing.format_intervals(snapshot.node.interval_unit)}{preview_modifier}"
             )
-        self._print("\n")
 
     def _show_categorized_snapshots(self, plan: Plan, default_catalog: t.Optional[str]) -> None:
         context_diff = plan.context_diff
@@ -1545,7 +1551,7 @@ class MarkdownConsole(CaptureTerminalConsole):
             for child_sid in sorted(plan.indirectly_modified.get(snapshot.snapshot_id, set())):
                 child_snapshot = context_diff.snapshots[child_sid]
                 if not indirect_tree:
-                    indirect_tree = Tree(f"[indirect]Indirectly Modified Children:")
+                    indirect_tree = Tree("[indirect]Indirectly Modified Children:")
                     tree.add(indirect_tree)
                 child_category_str = SNAPSHOT_CHANGE_CATEGORY_STR[child_snapshot.change_category]
                 indirect_tree.add(
@@ -1660,7 +1666,7 @@ class DatabricksMagicConsole(CaptureTerminalConsole):
     ) -> None:
         """Indicates that a new creation progress has begun."""
         self.model_creation_status = (0, total_tasks)
-        print(f"Starting Creating New Model Versions")
+        print("Starting Creating New Model Versions")
 
     def update_creation_progress(self, snapshot: SnapshotInfoLike) -> None:
         """Update the snapshot creation progress."""
@@ -1701,7 +1707,7 @@ class DatabricksMagicConsole(CaptureTerminalConsole):
     def start_migration_progress(self, total_tasks: int) -> None:
         """Indicates that a new migration progress has begun."""
         self.migration_status = (0, total_tasks)
-        print(f"Starting Migration")
+        print("Starting Migration")
 
     def update_migration_progress(self, num_tasks: int) -> None:
         """Update the migration progress."""
