@@ -40,18 +40,28 @@ DEFAULT_LOG_FILE_DIR = "logs"
 """The default directory for log files."""
 
 AUDITS = "audits"
+CACHE = ".cache"
+EXTERNAL_MODELS = "external_models"
 MACROS = "macros"
 METRICS = "metrics"
 MODELS = "models"
-EXTERNAL_MODELS = "external_models"
 SEEDS = "seeds"
 TESTS = "tests"
-CACHE = ".cache"
-SCHEMA_YAML = "schema.yaml"
 
+EXTERNAL_MODELS_YAML = "external_models.yaml"
+EXTERNAL_MODELS_DEPRECATED_YAML = "schema.yaml"
 
 DEFAULT_SCHEMA = "default"
 
 SQLMESH_VARS = "__sqlmesh__vars__"
 VAR = "var"
 GATEWAY = "gateway"
+
+SQLMESH_MACRO = "__sqlmesh__macro__"
+SQLMESH_BUILTIN = "__sqlmesh__builtin__"
+
+
+BUILTIN = "builtin"
+AIRFLOW = "airflow"
+DBT = "dbt"
+NATIVE = "native"
