@@ -52,7 +52,9 @@ import pandas as pd
 from sqlglot import exp
 from sqlglot.lineage import GraphHTML
 
+from sqlmesh.core import analytics
 from sqlmesh.core import constants as c
+from sqlmesh.core.analytics import python_api_analytics
 from sqlmesh.core.audit import Audit, StandaloneAudit
 from sqlmesh.core.config import CategorizerConfig, Config, load_configs
 from sqlmesh.core.config.loader import C
@@ -79,7 +81,7 @@ from sqlmesh.core.notification_target import (
 from sqlmesh.core.plan import Plan, PlanBuilder
 from sqlmesh.core.reference import ReferenceGraph
 from sqlmesh.core.scheduler import Scheduler
-from sqlmesh.core.schema_loader import create_schema_file
+from sqlmesh.core.schema_loader import create_external_models_file
 from sqlmesh.core.selector import Selector
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
@@ -118,7 +120,12 @@ from sqlmesh.utils.jinja import JinjaMacroRegistry
 if t.TYPE_CHECKING:
     from typing_extensions import Literal
 
-    from sqlmesh.core.engine_adapter._typing import DF, PySparkDataFrame, PySparkSession
+    from sqlmesh.core.engine_adapter._typing import (
+        DF,
+        PySparkDataFrame,
+        PySparkSession,
+        SnowparkSession,
+    )
     from sqlmesh.core.snapshot import Node
 
     ModelOrSnapshot = t.Union[str, Model, Snapshot]
@@ -150,6 +157,11 @@ class BaseContext(abc.ABC):
         """Returns the spark session if it exists."""
         return self.engine_adapter.spark
 
+    @property
+    def snowpark(self) -> t.Optional[SnowparkSession]:
+        """Returns the snowpark session if it exists."""
+        return self.engine_adapter.snowpark
+
     @property
     def default_catalog(self) -> t.Optional[str]:
         raise NotImplementedError
@@ -314,6 +326,9 @@ class GenericContext(BaseContext, t.Generic[C]):
 
         self.path, self.config = t.cast(t.Tuple[Path, C], next(iter(self.configs.items())))
 
+        if self.config.disable_anonymized_analytics:
+            analytics.disable_analytics()
+
         self.gateway = gateway
         self._scheduler = self.config.get_scheduler(self.gateway)
         self.environment_ttl = self.config.environment_ttl
@@ -324,12 +339,9 @@ class GenericContext(BaseContext, t.Generic[C]):
         self.concurrent_tasks = concurrent_tasks or self._connection_config.concurrent_tasks
         self._engine_adapter = engine_adapter or self._connection_config.create_engine_adapter()
 
-        test_connection_config = self.config.get_test_connection(
+        self._test_connection_config = self.config.get_test_connection(
             self.gateway, self.default_catalog, default_catalog_dialect=self.engine_adapter.DIALECT
         )
-        self._test_engine_adapter = test_connection_config.create_engine_adapter(
-            register_comments_override=False
-        )
 
         self._snapshot_evaluator: t.Optional[SnapshotEvaluator] = None
 
@@ -385,6 +397,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             default_catalog=self.default_catalog,
         )
 
+    @python_api_analytics
     def upsert_model(self, model: t.Union[str, Model], **kwargs: t.Any) -> Model:
         """Update or insert a model.
 
@@ -470,6 +483,7 @@ class GenericContext(BaseContext, t.Generic[C]):
 
     def load(self, update_schemas: bool = True) -> GenericContext[C]:
         """Load all files in the context's path."""
+        load_start_ts = time.perf_counter()
         with sys_path(*self.configs):
             gc.disable()
             project = self._loader.load(self, update_schemas)
@@ -493,8 +507,23 @@ class GenericContext(BaseContext, t.Generic[C]):
                     f"Models and Standalone audits cannot have the same name: {duplicates}"
                 )
 
+        analytics.collector.on_project_loaded(
+            project_type=(
+                c.DBT if type(self._loader).__name__.lower().startswith(c.DBT) else c.NATIVE
+            ),
+            models_count=len(self._models),
+            audits_count=len(self._audits),
+            standalone_audits_count=len(self._standalone_audits),
+            macros_count=len(self._macros),
+            jinja_macros_count=len(self._jinja_macros.root_macros),
+            load_time_sec=time.perf_counter() - load_start_ts,
+            state_sync_fingerprint=self._scheduler.state_sync_fingerprint(self),
+            project_name=self.config.project,
+        )
+
         return self
 
+    @python_api_analytics
     def run(
         self,
         environment: t.Optional[str] = None,
@@ -523,6 +552,11 @@ class GenericContext(BaseContext, t.Generic[C]):
         self.notification_target_manager.notify(
             NotificationEvent.RUN_START, environment=environment
         )
+        analytics_run_id = analytics.collector.on_run_start(
+            engine_type=self.snapshot_evaluator.adapter.dialect,
+            state_sync_type=self.state_sync.state_type(),
+        )
+
         success = False
         try:
             success = self._run(
@@ -538,6 +572,7 @@ class GenericContext(BaseContext, t.Generic[C]):
                 NotificationEvent.RUN_FAILURE, traceback.format_exc()
             )
             logger.error(f"Run Failure: {traceback.format_exc()}")
+            analytics.collector.on_run_end(run_id=analytics_run_id, succeeded=False, error=e)
             raise e
 
         if success:
@@ -550,6 +585,8 @@ class GenericContext(BaseContext, t.Generic[C]):
                 NotificationEvent.RUN_FAILURE, "See console logs for details."
             )
 
+        analytics.collector.on_run_end(run_id=analytics_run_id, succeeded=success)
+
         return success
 
     @t.overload
@@ -678,6 +715,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             self._default_catalog = self._scheduler.get_default_catalog(self)
         return self._default_catalog
 
+    @python_api_analytics
     def render(
         self,
         model_or_snapshot: ModelOrSnapshot,
@@ -738,6 +776,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             **kwargs,
         )
 
+    @python_api_analytics
     def evaluate(
         self,
         model_or_snapshot: ModelOrSnapshot,
@@ -774,6 +813,7 @@ class GenericContext(BaseContext, t.Generic[C]):
 
         return df
 
+    @python_api_analytics
     def format(
         self,
         transpile: t.Optional[str] = None,
@@ -809,6 +849,7 @@ class GenericContext(BaseContext, t.Generic[C]):
                     file.write("\n")
                 file.truncate()
 
+    @python_api_analytics
     def plan(
         self,
         environment: t.Optional[str] = None,
@@ -822,6 +863,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         no_gaps: bool = False,
         skip_backfill: bool = False,
         forward_only: t.Optional[bool] = None,
+        allow_destructive_models: t.Optional[t.Collection[str]] = None,
         no_prompts: t.Optional[bool] = None,
         auto_apply: t.Optional[bool] = None,
         no_auto_categorization: t.Optional[bool] = None,
@@ -858,6 +900,7 @@ class GenericContext(BaseContext, t.Generic[C]):
                 snapshots for same models.
             skip_backfill: Whether to skip the backfill step. Default: False.
             forward_only: Whether the purpose of the plan is to make forward only changes.
+            allow_destructive_models: Models whose forward-only changes are allowed to be destructive.
             no_prompts: Whether to disable interactive prompts for the backfill time range. Please note that
                 if this flag is set to true and there are uncategorized changes the plan creation will
                 fail. Default: False.
@@ -889,6 +932,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             no_gaps=no_gaps,
             skip_backfill=skip_backfill,
             forward_only=forward_only,
+            allow_destructive_models=allow_destructive_models,
             no_auto_categorization=no_auto_categorization,
             effective_from=effective_from,
             include_unmodified=include_unmodified,
@@ -909,6 +953,7 @@ class GenericContext(BaseContext, t.Generic[C]):
 
         return plan_builder.build()
 
+    @python_api_analytics
     def plan_builder(
         self,
         environment: t.Optional[str] = None,
@@ -922,6 +967,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         no_gaps: bool = False,
         skip_backfill: bool = False,
         forward_only: t.Optional[bool] = None,
+        allow_destructive_models: t.Optional[t.Collection[str]] = None,
         no_auto_categorization: t.Optional[bool] = None,
         effective_from: t.Optional[TimeLike] = None,
         include_unmodified: t.Optional[bool] = None,
@@ -952,6 +998,7 @@ class GenericContext(BaseContext, t.Generic[C]):
                 snapshots for same models.
             skip_backfill: Whether to skip the backfill step. Default: False.
             forward_only: Whether the purpose of the plan is to make forward only changes.
+            allow_destructive_models: Models whose forward-only changes are allowed to be destructive.
             no_auto_categorization: Indicates whether to disable automatic categorization of model
                 changes (breaking / non-breaking). If not provided, then the corresponding configuration
                 option determines the behavior.
@@ -987,6 +1034,13 @@ class GenericContext(BaseContext, t.Generic[C]):
 
         model_selector = self._new_selector()
 
+        if allow_destructive_models:
+            expanded_destructive_models = model_selector.expand_model_selections(
+                allow_destructive_models
+            )
+        else:
+            expanded_destructive_models = None
+
         if backfill_models:
             backfill_models = model_selector.expand_model_selections(backfill_models)
         else:
@@ -1007,10 +1061,6 @@ class GenericContext(BaseContext, t.Generic[C]):
         expanded_restate_models = None
         if restate_models is not None:
             expanded_restate_models = model_selector.expand_model_selections(restate_models)
-            if not expanded_restate_models:
-                self.console.log_error(
-                    f"Provided restated models do not match any models. No models will be included in plan. Provided: {', '.join(restate_models)}"
-                )
 
         snapshots = self._snapshots(models_override)
         context_diff = self._context_diff(
@@ -1068,6 +1118,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             forward_only=(
                 forward_only if forward_only is not None else self.config.plan.forward_only
             ),
+            allow_destructive_models=expanded_destructive_models,
             environment_ttl=environment_ttl,
             environment_suffix_target=self.config.environment_suffix_target,
             environment_catalog_mapping=self.config.environment_catalog_mapping,
@@ -1086,6 +1137,8 @@ class GenericContext(BaseContext, t.Generic[C]):
             ),
             end_bounded=not run,
             ensure_finalized_snapshots=self.config.plan.use_finalized_state,
+            engine_schema_differ=self.engine_adapter.SCHEMA_DIFFER,
+            console=self.console,
         )
 
     def apply(
@@ -1111,20 +1164,28 @@ class GenericContext(BaseContext, t.Generic[C]):
         if plan.uncategorized:
             raise UncategorizedPlanError("Can't apply a plan with uncategorized changes.")
         self.notification_target_manager.notify(
-            NotificationEvent.APPLY_START, environment=plan.environment_naming_info.name
+            NotificationEvent.APPLY_START,
+            environment=plan.environment_naming_info.name,
+            plan_id=plan.plan_id,
         )
         try:
             self._apply(plan, circuit_breaker)
         except Exception as e:
             self.notification_target_manager.notify(
-                NotificationEvent.APPLY_FAILURE, traceback.format_exc()
+                NotificationEvent.APPLY_FAILURE,
+                environment=plan.environment_naming_info.name,
+                plan_id=plan.plan_id,
+                exc=traceback.format_exc(),
             )
             logger.error(f"Apply Failure: {traceback.format_exc()}")
             raise e
         self.notification_target_manager.notify(
-            NotificationEvent.APPLY_END, environment=plan.environment_naming_info.name
+            NotificationEvent.APPLY_END,
+            environment=plan.environment_naming_info.name,
+            plan_id=plan.plan_id,
         )
 
+    @python_api_analytics
     def invalidate_environment(self, name: str, sync: bool = False) -> None:
         """Invalidates the target environment by setting its expiration timestamp to now.
 
@@ -1140,6 +1201,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         else:
             self.console.log_success(f"Environment '{name}' has been invalidated.")
 
+    @python_api_analytics
     def diff(self, environment: t.Optional[str] = None, detailed: bool = False) -> bool:
         """Show a diff of the current context with a given environment.
 
@@ -1165,6 +1227,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         )
         return context_diff.has_changes
 
+    @python_api_analytics
     def table_diff(
         self,
         source: str,
@@ -1175,6 +1238,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         limit: int = 20,
         show: bool = True,
         show_sample: bool = True,
+        decimals: int = 3,
     ) -> TableDiff:
         """Show a diff between two tables.
 
@@ -1188,6 +1252,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             limit: The limit of the sample dataframe.
             show: Show the table diff output in the console.
             show_sample: Show the sample dataframe in the console. Requires show=True.
+            decimals: The number of decimal places to keep when comparing floating point columns.
 
         Returns:
             The TableDiff object containing schema and summary differences.
@@ -1232,12 +1297,14 @@ class GenericContext(BaseContext, t.Generic[C]):
             target_alias=target_alias,
             model_name=model.name if model_or_snapshot else None,
             limit=limit,
+            decimals=decimals,
         )
         if show:
             self.console.show_schema_diff(table_diff.schema_diff())
             self.console.show_row_diff(table_diff.row_diff(), show_sample=show_sample)
         return table_diff
 
+    @python_api_analytics
     def get_dag(
         self, select_models: t.Optional[t.Collection[str]] = None, **options: t.Any
     ) -> GraphHTML:
@@ -1286,6 +1353,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             },
         )
 
+    @python_api_analytics
     def render_dag(self, path: str, select_models: t.Optional[t.Collection[str]] = None) -> None:
         """Render the dag as HTML and save it to a file.
 
@@ -1305,6 +1373,7 @@ class GenericContext(BaseContext, t.Generic[C]):
         with open(path, "w", encoding="utf-8") as file:
             file.write(str(self.get_dag(select_models)))
 
+    @python_api_analytics
     def create_test(
         self,
         model: str,
@@ -1336,20 +1405,27 @@ class GenericContext(BaseContext, t.Generic[C]):
             for dep, query in input_queries.items()
         }
 
-        generate_test(
-            model=self.get_model(model, raise_if_missing=True),
-            input_queries=input_queries,
-            models=self._models,
-            engine_adapter=self._engine_adapter,
-            test_engine_adapter=self._test_engine_adapter,
-            project_path=self.path,
-            overwrite=overwrite,
-            variables=variables,
-            path=path,
-            name=name,
-            include_ctes=include_ctes,
-        )
+        try:
+            test_adapter = self._test_connection_config.create_engine_adapter(
+                register_comments_override=False
+            )
+            generate_test(
+                model=self.get_model(model, raise_if_missing=True),
+                input_queries=input_queries,
+                models=self._models,
+                engine_adapter=self._engine_adapter,
+                test_engine_adapter=test_adapter,
+                project_path=self.path,
+                overwrite=overwrite,
+                variables=variables,
+                path=path,
+                name=name,
+                include_ctes=include_ctes,
+            )
+        finally:
+            test_adapter.close()
 
+    @python_api_analytics
     def test(
         self,
         match_patterns: t.Optional[t.List[str]] = None,
@@ -1365,45 +1441,48 @@ class GenericContext(BaseContext, t.Generic[C]):
         else:
             verbosity = 1
 
-        try:
-            if tests:
-                result = run_model_tests(
-                    tests=tests,
-                    models=self._models,
-                    engine_adapter=self._test_engine_adapter,
-                    dialect=self.default_dialect,
-                    verbosity=verbosity,
-                    patterns=match_patterns,
-                    preserve_fixtures=preserve_fixtures,
-                    default_catalog=self.default_catalog,
-                )
-            else:
-                test_meta = []
+        if tests:
+            result = run_model_tests(
+                tests=tests,
+                models=self._models,
+                config=self.config,
+                gateway=self.gateway,
+                dialect=self.default_dialect,
+                verbosity=verbosity,
+                patterns=match_patterns,
+                preserve_fixtures=preserve_fixtures,
+                stream=stream,
+                default_catalog=self.default_catalog,
+                default_catalog_dialect=self.engine_adapter.DIALECT,
+            )
+        else:
+            test_meta = []
 
-                for path, config in self.configs.items():
-                    test_meta.extend(
-                        get_all_model_tests(
-                            path / c.TESTS,
-                            patterns=match_patterns,
-                            ignore_patterns=config.ignore_patterns,
-                        )
+            for path, config in self.configs.items():
+                test_meta.extend(
+                    get_all_model_tests(
+                        path / c.TESTS,
+                        patterns=match_patterns,
+                        ignore_patterns=config.ignore_patterns,
                     )
-
-                result = run_tests(
-                    test_meta,
-                    models=self._models,
-                    engine_adapter=self._test_engine_adapter,
-                    dialect=self.default_dialect,
-                    verbosity=verbosity,
-                    preserve_fixtures=preserve_fixtures,
-                    stream=stream,
-                    default_catalog=self.default_catalog,
                 )
-        finally:
-            self._test_engine_adapter.close()
+
+            result = run_tests(
+                model_test_metadata=test_meta,
+                models=self._models,
+                config=self.config,
+                gateway=self.gateway,
+                dialect=self.default_dialect,
+                verbosity=verbosity,
+                preserve_fixtures=preserve_fixtures,
+                stream=stream,
+                default_catalog=self.default_catalog,
+                default_catalog_dialect=self.engine_adapter.DIALECT,
+            )
 
         return result
 
+    @python_api_analytics
     def audit(
         self,
         start: TimeLike,
@@ -1470,6 +1549,7 @@ class GenericContext(BaseContext, t.Generic[C]):
 
         self.console.log_status_update("Done.")
 
+    @python_api_analytics
     def rewrite(self, sql: str, dialect: str = "") -> exp.Expression:
         """Rewrite a sql expression with semantic references into an executable query.
 
@@ -1489,6 +1569,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             dialect=dialect or self.default_dialect,
         )
 
+    @python_api_analytics
     def migrate(self) -> None:
         """Migrates SQLMesh to the current running version.
 
@@ -1507,6 +1588,7 @@ class GenericContext(BaseContext, t.Generic[C]):
             raise e
         self.notification_target_manager.notify(NotificationEvent.MIGRATION_END)
 
+    @python_api_analytics
     def rollback(self) -> None:
         """Rolls back SQLMesh to the previous migration.
 
@@ -1514,18 +1596,24 @@ class GenericContext(BaseContext, t.Generic[C]):
         """
         self._new_state_sync().rollback()
 
+    @python_api_analytics
     def create_external_models(self) -> None:
-        """Create a schema file with all external models.
+        """Create a file to document the schema of external models.
 
-        The schema file contains all columns and types of external models, allowing for more robust
-        lineage, validation, and optimizations.
+        The external models file contains all columns and types of external models, allowing for more
+        robust lineage, validation, and optimizations.
         """
         if not self._models:
             self.load(update_schemas=False)
 
         for path, config in self.configs.items():
-            create_schema_file(
-                path=path / c.SCHEMA_YAML,
+            deprecated_yaml = path / c.EXTERNAL_MODELS_DEPRECATED_YAML
+
+            external_models_yaml = (
+                path / c.EXTERNAL_MODELS_YAML if not deprecated_yaml.exists() else deprecated_yaml
+            )
+            create_external_models_file(
+                path=external_models_yaml,
                 models=UniqueKeyDict(
                     "models",
                     {
@@ -1540,18 +1628,20 @@ class GenericContext(BaseContext, t.Generic[C]):
                 max_workers=self.concurrent_tasks,
             )
 
-    def print_info(self) -> None:
+    @python_api_analytics
+    def print_info(self, skip_connection: bool = False) -> None:
         """Prints information about connections, models, macros, etc. to the console."""
         self.console.log_status_update(f"Models: {len(self.models)}")
         self.console.log_status_update(f"Macros: {len(self._macros) - len(macro.get_registry())}")
 
-        self._try_connection("data warehouse", self._engine_adapter)
+        if skip_connection:
+            return
+
+        self._try_connection("data warehouse", self._engine_adapter.ping)
 
         state_connection = self.config.get_state_connection(self.gateway)
         if state_connection:
-            self._try_connection("state backend", state_connection.create_engine_adapter())
-
-        self._try_connection("test", self._test_engine_adapter)
+            self._try_connection("state backend", state_connection.connection_validator())
 
     def close(self) -> None:
         """Releases all resources allocated by this context."""
@@ -1632,6 +1722,7 @@ class GenericContext(BaseContext, t.Generic[C]):
     def _apply(self, plan: Plan, circuit_breaker: t.Optional[t.Callable[[], bool]]) -> None:
         self._scheduler.create_plan_evaluator(self).evaluate(plan, circuit_breaker=circuit_breaker)
 
+    @python_api_analytics
     def table_name(self, model_name: str, dev: bool) -> str:
         """Returns the name of the pysical table for the given model name.
 
@@ -1664,11 +1755,11 @@ class GenericContext(BaseContext, t.Generic[C]):
     def _run_plan_tests(
         self, skip_tests: bool = False
     ) -> t.Tuple[t.Optional[unittest.result.TestResult], t.Optional[str]]:
-        if self._test_engine_adapter and not skip_tests:
+        if not skip_tests:
             result, test_output = self._run_tests()
             if result.testsRun > 0:
                 self.console.log_test_results(
-                    result, test_output, self._test_engine_adapter.dialect
+                    result, test_output, self._test_connection_config._engine_adapter.DIALECT
                 )
             if not result.wasSuccessful():
                 raise PlanError(
@@ -1803,10 +1894,10 @@ class GenericContext(BaseContext, t.Generic[C]):
         expired_environments = self.state_sync.delete_expired_environments()
         cleanup_expired_views(self.engine_adapter, expired_environments, console=self.console)
 
-    def _try_connection(self, connection_name: str, engine_adapter: EngineAdapter) -> None:
+    def _try_connection(self, connection_name: str, validator: t.Callable[[], None]) -> None:
         connection_name = connection_name.capitalize()
         try:
-            engine_adapter.fetchall("SELECT 1")
+            validator()
             self.console.log_status_update(f"{connection_name} connection [green]succeeded[/green]")
         except Exception as ex:
             self.console.log_error(f"{connection_name} connection failed. {ex}")
