@@ -53,9 +53,9 @@ if t.TYPE_CHECKING:
         PySparkSession,
         Query,
         QueryOrDF,
+        SnowparkSession,
     )
     from sqlmesh.core.node import IntervalUnit
-    from sqlmesh.utils.pandas import PandasNamedTuple
 
 logger = logging.getLogger(__name__)
 
@@ -110,26 +110,28 @@ class EngineAdapter:
         default_catalog: t.Optional[str] = None,
         execute_log_level: int = logging.DEBUG,
         register_comments: bool = True,
+        pre_ping: bool = False,
         **kwargs: t.Any,
     ):
         self.dialect = dialect.lower() or self.DIALECT
         self._connection_pool = create_connection_pool(
             connection_factory, multithreaded, cursor_kwargs=cursor_kwargs, cursor_init=cursor_init
         )
-        self.sql_gen_kwargs = sql_gen_kwargs or {}
+        self._sql_gen_kwargs = sql_gen_kwargs or {}
         self._default_catalog = default_catalog
         self._execute_log_level = execute_log_level
         self._extra_config = kwargs
-        self.register_comments = register_comments
+        self._register_comments = register_comments
+        self._pre_ping = pre_ping
 
     def with_log_level(self, level: int) -> EngineAdapter:
         adapter = self.__class__(
             lambda: None,
             dialect=self.dialect,
-            sql_gen_kwargs=self.sql_gen_kwargs,
+            sql_gen_kwargs=self._sql_gen_kwargs,
             default_catalog=self._default_catalog,
             execute_log_level=level,
-            register_comments=self.register_comments,
+            register_comments=self._register_comments,
             **self._extra_config,
         )
 
@@ -146,12 +148,12 @@ class EngineAdapter:
         return None
 
     @property
-    def comments_enabled(self) -> bool:
-        return self.register_comments and self.COMMENT_CREATION_TABLE.is_supported
+    def snowpark(self) -> t.Optional[SnowparkSession]:
+        return None
 
-    @classmethod
-    def is_pandas_df(cls, value: t.Any) -> bool:
-        return isinstance(value, pd.DataFrame)
+    @property
+    def comments_enabled(self) -> bool:
+        return self._register_comments and self.COMMENT_CREATION_TABLE.is_supported
 
     @classmethod
     def _casted_columns(cls, columns_to_types: t.Dict[str, exp.DataType]) -> t.List[exp.Alias]:
@@ -243,7 +245,7 @@ class EngineAdapter:
     ) -> t.Optional[t.Dict[str, exp.DataType]]:
         if columns_to_types:
             return columns_to_types
-        if self.is_pandas_df(query_or_df):
+        if isinstance(query_or_df, pd.DataFrame):
             return columns_to_types_from_df(t.cast(pd.DataFrame, query_or_df))
         return columns_to_types
 
@@ -779,20 +781,29 @@ class EngineAdapter:
         drop_expression = exp.Drop(this=exp.to_table(table_name), kind="TABLE", exists=exists)
         self.execute(drop_expression)
 
-    def alter_table(
+    def get_alter_expressions(
         self,
         current_table_name: TableName,
         target_table_name: TableName,
+    ) -> t.List[exp.AlterTable]:
+        """
+        Determines the alter statements needed to change the current table into the structure of the target table.
+        """
+        return self.SCHEMA_DIFFER.compare_columns(
+            current_table_name,
+            self.columns(current_table_name),
+            self.columns(target_table_name),
+        )
+
+    def alter_table(
+        self,
+        alter_expressions: t.List[exp.AlterTable],
     ) -> None:
         """
-        Performs the required alter statements to change the current table into the structure of the target table.
+        Performs the alter statements to change the current table into the structure of the target table.
         """
         with self.transaction():
-            for alter_expression in self.SCHEMA_DIFFER.compare_columns(
-                current_table_name,
-                self.columns(current_table_name),
-                self.columns(target_table_name),
-            ):
+            for alter_expression in alter_expressions:
                 self.execute(alter_expression)
 
     def create_view(
@@ -804,6 +815,7 @@ class EngineAdapter:
         materialized: bool = False,
         table_description: t.Optional[str] = None,
         column_descriptions: t.Optional[t.Dict[str, str]] = None,
+        view_properties: t.Optional[t.Dict[str, exp.Expression]] = None,
         **create_kwargs: t.Any,
     ) -> None:
         """Create a view with a query or dataframe.
@@ -819,10 +831,13 @@ class EngineAdapter:
             materialized: Whether to create a a materialized view. Only used for engines that support this feature.
             table_description: Optional table description from MODEL DDL.
             column_descriptions: Optional column descriptions from model query.
+            view_properties: Optional view properties to add to the view.
             create_kwargs: Additional kwargs to pass into the Create expression
         """
-        if self.is_pandas_df(query_or_df):
-            values = list(t.cast(pd.DataFrame, query_or_df).itertuples(index=False, name=None))
+        if isinstance(query_or_df, pd.DataFrame):
+            values: t.List[t.Tuple[t.Any, ...]] = list(
+                query_or_df.itertuples(index=False, name=None)
+            )
             columns_to_types = columns_to_types or self._columns_to_types(query_or_df)
             if not columns_to_types:
                 raise SQLMeshError("columns_to_types must be provided for dataframes")
@@ -856,7 +871,7 @@ class EngineAdapter:
                 schema = schema.this
 
         create_view_properties = self._build_view_properties_exp(
-            create_kwargs.pop("table_properties", None),
+            view_properties,
             (
                 table_description
                 if self.COMMENT_CREATION_VIEW.supports_schema_def and self.comments_enabled
@@ -1026,9 +1041,18 @@ class EngineAdapter:
         partitioned_by: t.List[exp.Expression],
         columns_to_types: t.Optional[t.Dict[str, exp.DataType]] = None,
     ) -> None:
-        raise NotImplementedError(
-            "Insert Overwrite by Partition (not time) is not supported by this engine"
-        )
+        if self.INSERT_OVERWRITE_STRATEGY.is_insert_overwrite:
+            target_table = exp.to_table(table_name)
+            source_queries, columns_to_types = self._get_source_queries_and_columns_to_types(
+                query_or_df, columns_to_types, target_table=target_table
+            )
+            self._insert_overwrite_by_condition(
+                table_name, source_queries, columns_to_types=columns_to_types
+            )
+        else:
+            self._replace_by_key(
+                table_name, query_or_df, columns_to_types, partitioned_by, is_unique_key=False
+            )
 
     def insert_overwrite_by_time_partition(
         self,
@@ -1059,7 +1083,7 @@ class EngineAdapter:
 
     def _values_to_sql(
         self,
-        values: t.List[PandasNamedTuple],
+        values: t.List[t.Tuple[t.Any, ...]],
         columns_to_types: t.Dict[str, exp.DataType],
         batch_start: int,
         batch_end: int,
@@ -1105,7 +1129,6 @@ class EngineAdapter:
                         insert_exp = exp.insert(
                             query,
                             table,
-                            # Change once Databricks supports REPLACE WHERE with columns
                             columns=(
                                 list(columns_to_types)
                                 if not insert_overwrite_strategy.is_replace_where
@@ -1114,7 +1137,7 @@ class EngineAdapter:
                             overwrite=insert_overwrite_strategy.is_insert_overwrite,
                         )
                         if insert_overwrite_strategy.is_replace_where:
-                            insert_exp.set("where", where)
+                            insert_exp.set("where", where or exp.true())
                         self.execute(insert_exp)
 
     def update_table(
@@ -1741,6 +1764,15 @@ class EngineAdapter:
         ):
             yield
             return
+
+        if self._pre_ping:
+            try:
+                logger.debug("Pinging the database to check the connection")
+                self.ping()
+            except Exception:
+                logger.info("Connection to the database was lost. Reconnecting...")
+                self._connection_pool.close()
+
         self._connection_pool.begin()
         try:
             yield
@@ -1847,13 +1879,15 @@ class EngineAdapter:
             finally:
                 self.drop_table(table)
 
-    def _table_properties_to_expressions(
-        self, table_properties: t.Optional[t.Dict[str, exp.Expression]] = None
+    def _table_or_view_properties_to_expressions(
+        self, table_or_view_properties: t.Optional[t.Dict[str, exp.Expression]] = None
     ) -> t.List[exp.Property]:
-        if not table_properties:
+        """Converts model properties (either physical or virtual) to a list of property expressions."""
+        if not table_or_view_properties:
             return []
         return [
-            exp.Property(this=key, value=value.copy()) for key, value in table_properties.items()
+            exp.Property(this=key, value=value.copy())
+            for key, value in table_or_view_properties.items()
         ]
 
     def _build_table_properties_exp(
@@ -1883,7 +1917,7 @@ class EngineAdapter:
 
     def _build_view_properties_exp(
         self,
-        table_properties: t.Optional[t.Dict[str, exp.Expression]] = None,
+        view_properties: t.Optional[t.Dict[str, exp.Expression]] = None,
         table_description: t.Optional[str] = None,
     ) -> t.Optional[exp.Properties]:
         """Creates a SQLGlot table properties expression for view"""
@@ -1919,7 +1953,7 @@ class EngineAdapter:
             "dialect": self.dialect,
             "pretty": False,
             "comments": False,
-            **self.sql_gen_kwargs,
+            **self._sql_gen_kwargs,
             **kwargs,
         }
 
@@ -1981,6 +2015,49 @@ class EngineAdapter:
         table = exp.to_table(table_name)
         self.execute(f"TRUNCATE TABLE {table.sql(dialect=self.dialect, identify=True)}")
 
+    def _replace_by_key(
+        self,
+        target_table: TableName,
+        source_table: QueryOrDF,
+        columns_to_types: t.Optional[t.Dict[str, exp.DataType]],
+        key: t.Sequence[exp.Expression],
+        is_unique_key: bool,
+    ) -> None:
+        if columns_to_types is None:
+            columns_to_types = self.columns(target_table)
+
+        temp_table = self._get_temp_table(target_table)
+        key_exp = exp.func("CONCAT_WS", "'__SQLMESH_DELIM__'", *key) if len(key) > 1 else key[0]
+        column_names = list(columns_to_types or [])
+
+        with self.transaction():
+            self.ctas(temp_table, source_table, columns_to_types=columns_to_types, exists=False)
+
+            try:
+                delete_query = exp.select(key_exp).from_(temp_table)
+                insert_query = self._select_columns(columns_to_types).from_(temp_table)
+                if not is_unique_key:
+                    delete_query = delete_query.distinct()
+                else:
+                    insert_query = insert_query.distinct(*key)
+
+                insert_statement = exp.insert(
+                    insert_query,
+                    target_table,
+                    columns=column_names,
+                )
+                delete_filter = key_exp.isin(query=delete_query)
+
+                if not self.INSERT_OVERWRITE_STRATEGY.is_replace_where:
+                    self.execute(exp.delete(target_table).where(delete_filter))
+                else:
+                    insert_statement.set("where", delete_filter)
+                    insert_statement.set("this", exp.to_table(target_table))
+
+                self.execute(insert_statement)
+            finally:
+                self.drop_table(temp_table)
+
     def _build_create_comment_table_exp(
         self, table: exp.Table, table_comment: str, table_kind: str
     ) -> exp.Comment | str:
@@ -2036,6 +2113,12 @@ class EngineAdapter:
     ) -> None:
         self.execute(exp.rename_table(old_table_name, new_table_name))
 
+    def ping(self) -> None:
+        try:
+            self._execute(exp.select("1").sql(dialect=self.dialect))
+        finally:
+            self._connection_pool.close_cursor()
+
     @classmethod
     def _select_columns(cls, columns: t.Iterable[str]) -> exp.Select:
         return exp.select(*(exp.column(c, quoted=True) for c in columns))
