@@ -182,12 +182,15 @@ class Loader(abc.ABC):
     def _load_external_models(self) -> UniqueKeyDict[str, Model]:
         models: UniqueKeyDict[str, Model] = UniqueKeyDict("models")
         for context_path, config in self._context.configs.items():
-            schema_path = Path(context_path / c.SCHEMA_YAML)
+            external_models_yaml = Path(context_path / c.EXTERNAL_MODELS_YAML)
+            deprecated_yaml = Path(context_path / c.EXTERNAL_MODELS_DEPRECATED_YAML)
             external_models_path = context_path / c.EXTERNAL_MODELS
 
             paths_to_load = []
-            if schema_path.exists():
-                paths_to_load.append(schema_path)
+            if external_models_yaml.exists():
+                paths_to_load.append(external_models_yaml)
+            elif deprecated_yaml.exists():
+                paths_to_load.append(deprecated_yaml)
 
             if external_models_path.exists() and external_models_path.is_dir():
                 paths_to_load.extend(external_models_path.glob("*.yaml"))
@@ -200,6 +203,7 @@ class Loader(abc.ABC):
                         model = create_external_model(
                             **row,
                             dialect=config.model_defaults.dialect,
+                            defaults=config.model_defaults.dict(),
                             path=path,
                             project=config.project,
                             default_catalog=self._context.default_catalog,
@@ -222,7 +226,6 @@ class SqlMeshLoader(Loader):
         """Loads all user defined macros."""
         # Store a copy of the macro registry
         standard_macros = macro.get_registry()
-
         jinja_macros = JinjaMacroRegistry()
         extractor = MacroExtractor()
 
@@ -309,6 +312,7 @@ class SqlMeshLoader(Loader):
                         project=config.project,
                         default_catalog=self._context.default_catalog,
                         variables=variables,
+                        infer_names=config.model_naming.infer_names,
                     )
 
                 model = cache.get_or_load_model(path, _load)
@@ -329,27 +333,34 @@ class SqlMeshLoader(Loader):
 
         for context_path, config in self._context.configs.items():
             variables = self._variables(config)
-            for path in self._glob_paths(context_path / c.MODELS, config=config, extension=".py"):
-                if not os.path.getsize(path):
-                    continue
+            model_registry._dialect = config.model_defaults.dialect
+            try:
+                for path in self._glob_paths(
+                    context_path / c.MODELS, config=config, extension=".py"
+                ):
+                    if not os.path.getsize(path):
+                        continue
 
-                self._track_file(path)
-                import_python_file(path, context_path)
-                new = registry.keys() - registered
-                registered |= new
-                for name in new:
-                    model = registry[name].model(
-                        path=path,
-                        module_path=context_path,
-                        defaults=config.model_defaults.dict(),
-                        dialect=config.model_defaults.dialect,
-                        time_column_format=config.time_column_format,
-                        physical_schema_override=config.physical_schema_override,
-                        project=config.project,
-                        default_catalog=self._context.default_catalog,
-                        variables=variables,
-                    )
-                    models[model.fqn] = model
+                    self._track_file(path)
+                    import_python_file(path, context_path)
+                    new = registry.keys() - registered
+                    registered |= new
+                    for name in new:
+                        model = registry[name].model(
+                            path=path,
+                            module_path=context_path,
+                            defaults=config.model_defaults.dict(),
+                            dialect=config.model_defaults.dialect,
+                            time_column_format=config.time_column_format,
+                            physical_schema_override=config.physical_schema_override,
+                            project=config.project,
+                            default_catalog=self._context.default_catalog,
+                            variables=variables,
+                            infer_names=config.model_naming.infer_names,
+                        )
+                        models[model.fqn] = model
+            finally:
+                model_registry._dialect = None
 
         return models
 
