@@ -6,7 +6,6 @@ import logging
 import sys
 import types
 import typing as t
-from difflib import unified_diff
 from functools import cached_property
 from pathlib import Path
 
@@ -24,7 +23,7 @@ from sqlmesh.core import constants as c
 from sqlmesh.core import dialect as d
 from sqlmesh.core.macros import MacroRegistry, MacroStrTemplate, macro
 from sqlmesh.core.model.common import expression_validator
-from sqlmesh.core.model.kind import ModelKindName, SeedKind
+from sqlmesh.core.model.kind import ModelKindName, SeedKind, create_model_kind
 from sqlmesh.core.model.meta import ModelMeta
 from sqlmesh.core.model.seed import CsvSeedReader, Seed, create_seed
 from sqlmesh.core.renderer import ExpressionRenderer, QueryRenderer
@@ -124,7 +123,7 @@ class _Model(ModelMeta, frozen=True):
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
         **kwargs: t.Any,
-    ) -> t.Generator[QueryOrDF, None, None]:
+    ) -> t.Iterator[QueryOrDF]:
         """Renders the content of this model in a form of either a SELECT query, executing which the data for this model can
         be fetched, or a dataframe object which contains the data itself.
 
@@ -150,7 +149,9 @@ class _Model(ModelMeta, frozen=True):
             **kwargs,
         )
 
-    def render_definition(self, include_python: bool = True) -> t.List[exp.Expression]:
+    def render_definition(
+        self, include_python: bool = True, include_defaults: bool = False
+    ) -> t.List[exp.Expression]:
         """Returns the original list of sql expressions comprising the model definition.
 
         Args:
@@ -161,7 +162,7 @@ class _Model(ModelMeta, frozen=True):
         for field_name, field_info in ModelMeta.all_field_infos().items():
             field_value = getattr(self, field_name)
 
-            if field_value != field_info.default:
+            if (include_defaults and field_value) or field_value != field_info.default:
                 if field_name == "description":
                     comment = field_value
                 elif field_name == "kind":
@@ -716,7 +717,7 @@ class _Model(ModelMeta, frozen=True):
             data.append(column_name)
             data.append(column_type.sql())
 
-        for key, value in (self.table_properties or {}).items():
+        for key, value in (self.physical_properties or {}).items():
             data.append(key)
             data.append(gen(value))
 
@@ -778,6 +779,10 @@ class _Model(ModelMeta, frozen=True):
             else:
                 raise SQLMeshError(f"Unexpected audit name '{audit_name}'.")
 
+        for key, value in (self.virtual_properties or {}).items():
+            metadata.append(key)
+            metadata.append(gen(value))
+
         metadata.extend(gen(s) for s in self.signals)
         metadata.extend(self._additional_metadata)
 
@@ -965,11 +970,15 @@ class SqlModel(_SqlBasedModel):
         )
         return query
 
-    def render_definition(self, include_python: bool = True) -> t.List[exp.Expression]:
-        result = super().render_definition(include_python=include_python)
+    def render_definition(
+        self, include_python: bool = True, include_defaults: bool = False
+    ) -> t.List[exp.Expression]:
+        result = super().render_definition(
+            include_python=include_python, include_defaults=include_defaults
+        )
         result.extend(self.pre_statements)
-        result.extend(self.post_statements)
         result.append(self.query)
+        result.extend(self.post_statements)
         return result
 
     @property
@@ -1131,6 +1140,7 @@ class SeedModel(_SqlBasedModel):
     kind: SeedKind
     seed: Seed
     column_hashes_: t.Optional[t.Dict[str, str]] = Field(default=None, alias="column_hashes")
+    derived_columns_to_types: t.Optional[t.Dict[str, exp.DataType]] = None
     is_hydrated: bool = True
     source_type: Literal["seed"] = "seed"
 
@@ -1142,7 +1152,12 @@ class SeedModel(_SqlBasedModel):
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
         **kwargs: t.Any,
-    ) -> t.Generator[QueryOrDF, None, None]:
+    ) -> t.Iterator[QueryOrDF]:
+        if not self.is_hydrated:
+            return
+        yield from self.render_seed()
+
+    def render_seed(self) -> t.Iterator[QueryOrDF]:
         self._ensure_hydrated()
 
         date_columns = []
@@ -1163,38 +1178,27 @@ class SeedModel(_SqlBasedModel):
             # convert all date/time types to native pandas timestamp
             for column in [*date_columns, *datetime_columns]:
                 df[column] = pd.to_datetime(df[column])
+
             # extract datetime.date from pandas timestamp for DATE columns
             for column in date_columns:
                 df[column] = df[column].dt.date
+
             df[bool_columns] = df[bool_columns].apply(lambda i: str_to_bool(str(i)))
             df.loc[:, string_columns] = df[string_columns].mask(
-                cond=lambda x: x.notna(), other=df[string_columns].astype(str)  # type: ignore
+                cond=lambda x: x.notna(),  # type: ignore
+                other=df[string_columns].astype(str),  # type: ignore
             )
             yield df
 
-    def text_diff(self, other: Node) -> str:
-        if not isinstance(other, SeedModel):
-            return super().text_diff(other)
-
-        other._ensure_hydrated()
-        self._ensure_hydrated()
-
-        return "\n".join(
-            (
-                super().text_diff(other),
-                *unified_diff(
-                    self.seed.content.split("\n"),
-                    other.seed.content.split("\n"),
-                ),
-            )
-        ).strip()
-
     @property
     def columns_to_types(self) -> t.Optional[t.Dict[str, exp.DataType]]:
         if self.columns_to_types_ is not None:
             return self.columns_to_types_
-        self._ensure_hydrated()
-        return self._reader.columns_to_types
+        if self.derived_columns_to_types is not None:
+            return self.derived_columns_to_types
+        if self.is_hydrated:
+            return self._reader.columns_to_types
+        return None
 
     @property
     def column_hashes(self) -> t.Dict[str, str]:
@@ -1247,6 +1251,9 @@ class SeedModel(_SqlBasedModel):
                 "seed": Seed(content=""),
                 "is_hydrated": False,
                 "column_hashes_": self.column_hashes,
+                "derived_columns_to_types": self.columns_to_types
+                if self.columns_to_types_ is None
+                else None,
             }
         )
 
@@ -1318,7 +1325,7 @@ class PythonModel(_Model):
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
         **kwargs: t.Any,
-    ) -> t.Generator[QueryOrDF, None, None]:
+    ) -> t.Iterator[QueryOrDF]:
         env = prepare_env(self.python_env)
         start, end = make_inclusive(start or c.EPOCH, end or c.EPOCH)
         execution_time = to_datetime(execution_time or c.EPOCH)
@@ -1346,10 +1353,12 @@ class PythonModel(_Model):
             print_exception(e, self.python_env)
             raise SQLMeshError(f"Error executing Python model '{self.name}'")
 
-    def render_definition(self, include_python: bool = True) -> t.List[exp.Expression]:
+    def render_definition(
+        self, include_python: bool = True, include_defaults: bool = False
+    ) -> t.List[exp.Expression]:
         # Ignore the provided value for the include_python flag, since the Pyhon model's
         # definition without Python code is meaningless.
-        return super().render_definition(include_python=True)
+        return super().render_definition(include_python=True, include_defaults=include_defaults)
 
     @property
     def is_python(self) -> bool:
@@ -1403,6 +1412,7 @@ def load_sql_based_model(
     physical_schema_override: t.Optional[t.Dict[str, str]] = None,
     default_catalog: t.Optional[str] = None,
     variables: t.Optional[t.Dict[str, t.Any]] = None,
+    infer_names: t.Optional[bool] = False,
     **kwargs: t.Any,
 ) -> Model:
     """Load a model from a parsed SQLMesh model SQL file.
@@ -1457,6 +1467,7 @@ def load_sql_based_model(
         python_env=meta_python_env,
         default_catalog=default_catalog,
         quote_identifiers=False,
+        normalize_identifiers=False,
     )
     rendered_meta_exprs = meta_renderer.render()
     if rendered_meta_exprs is None or len(rendered_meta_exprs) != 1:
@@ -1486,7 +1497,14 @@ def load_sql_based_model(
         # Signals must remain unrendered, so that they can be rendered later at evaluation runtime.
         meta_fields["signals"] = unrendered_signals
 
+    if isinstance(meta_fields.get("dialect"), exp.Expression):
+        meta_fields["dialect"] = meta_fields["dialect"].name
+
+    # The name of the model will be inferred from its path relative to `models/`, if it's not explicitly specified
     name = meta_fields.pop("name", "")
+    if not name and infer_names:
+        name = get_model_name(path)
+
     if not name:
         raise_config_error("Model must have a name", path)
     if "default_catalog" in meta_fields:
@@ -1773,6 +1791,7 @@ def create_external_model(
     *,
     dialect: t.Optional[str] = None,
     path: Path = Path(),
+    defaults: t.Optional[t.Dict[str, t.Any]] = None,
     **kwargs: t.Any,
 ) -> Model:
     """Creates an external model.
@@ -1786,6 +1805,7 @@ def create_external_model(
     return _create_model(
         ExternalModel,
         name,
+        defaults=defaults,
         dialect=dialect,
         path=path,
         kind=ModelKindName.EXTERNAL.value,
@@ -1807,11 +1827,21 @@ def _create_model(
     physical_schema_override: t.Optional[t.Dict[str, str]] = None,
     **kwargs: t.Any,
 ) -> Model:
-    _validate_model_fields(klass, {"name", *kwargs} - {"grain"}, path)
+    _validate_model_fields(klass, {"name", *kwargs} - {"grain", "table_properties"}, path)
+
+    kwargs["session_properties"] = _resolve_session_properties(
+        (defaults or {}).get("session_properties"), kwargs.get("session_properties")
+    )
 
     dialect = dialect or ""
     physical_schema_override = physical_schema_override or {}
 
+    raw_kind = kwargs.pop("kind", None)
+    if raw_kind:
+        kwargs["kind"] = create_model_kind(raw_kind, dialect, defaults or {})
+
+    defaults = {k: v for k, v in (defaults or {}).items() if k in klass.all_fields()}
+
     try:
         model = klass(
             name=name,
@@ -1872,6 +1902,29 @@ def _split_sql_model_statements(
     return query, expressions[:pos], expressions[pos + 1 :]
 
 
+def _resolve_session_properties(
+    default: t.Optional[t.Dict[str, t.Any]],
+    provided: t.Optional[exp.Expression | t.Dict[str, t.Any]],
+) -> t.Optional[exp.Expression]:
+    if isinstance(provided, dict):
+        session_properties = {k: exp.Literal.string(k).eq(v) for k, v in provided.items()}
+    elif provided:
+        if isinstance(provided, exp.Paren):
+            provided = exp.Tuple(expressions=[provided.this])
+        session_properties = {expr.this.name: expr for expr in provided}
+    else:
+        session_properties = {}
+
+    for k, v in (default or {}).items():
+        if k not in session_properties:
+            session_properties[k] = exp.Literal.string(k).eq(v)
+
+    if session_properties:
+        return exp.Tuple(expressions=list(session_properties.values()))
+
+    return None
+
+
 def _validate_model_fields(klass: t.Type[_Model], provided_fields: t.Set[str], path: Path) -> None:
     missing_required_fields = klass.missing_required_fields(provided_fields)
     if missing_required_fields:
@@ -1939,11 +1992,11 @@ def _python_env(
         if macro_ref.package is None and macro_ref.name in macros:
             used_macros[macro_ref.name] = macros[macro_ref.name]
 
-    for name, macro in used_macros.items():
-        if isinstance(macro, Executable):
-            serialized_env[name] = macro
-        elif not macro.func.__module__.startswith("sqlmesh."):
-            build_env(macro.func, env=python_env, name=name, path=module_path)
+    for name, used_macro in used_macros.items():
+        if isinstance(used_macro, Executable):
+            serialized_env[name] = used_macro
+        elif not hasattr(used_macro, c.SQLMESH_BUILTIN):
+            build_env(used_macro.func, env=python_env, name=name, path=module_path)
 
     serialized_env.update(serialize_env(python_env, path=module_path))
 
@@ -1983,9 +2036,9 @@ def _parse_dependencies(
 
                 def get_first_arg(keyword_arg_name: str) -> t.Any:
                     if node.args:
-                        table: t.Optional[ast.expr] = node.args[0]
+                        first_arg: t.Optional[ast.expr] = node.args[0]
                     else:
-                        table = next(
+                        first_arg = next(
                             (
                                 keyword.value
                                 for keyword in node.keywords
@@ -1995,11 +2048,11 @@ def _parse_dependencies(
                         )
 
                     try:
-                        expression = to_source(table)
+                        expression = to_source(first_arg)
                         return eval(expression, env)
                     except Exception:
                         raise ConfigError(
-                            f"Error resolving dependencies for '{executable.path}'. References to context / evaluator must be resolvable at parse time.\n\n{expression}"
+                            f"Error resolving dependencies for '{executable.path}'. Argument '{expression.strip()}' must be resolvable at parse time."
                         )
 
                 if func.value.id == "context" and func.attr == "table":
@@ -2078,8 +2131,14 @@ META_FIELD_CONVERTER: t.Dict[str, t.Callable] = {
     "tags": _single_value_or_tuple,
     "grains": _refs_to_sql,
     "references": _refs_to_sql,
-    "table_properties_": lambda value: value,
+    "physical_properties_": lambda value: value,
+    "virtual_properties_": lambda value: value,
     "session_properties_": lambda value: value,
     "allow_partials": exp.convert,
     "signals": lambda values: exp.Tuple(expressions=values),
 }
+
+
+def get_model_name(path: Path) -> str:
+    path_parts = list(path.parts[path.parts.index("models") + 1 : -1]) + [path.stem]
+    return ".".join(path_parts[-3:])
