@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import logging
 import typing as t
 from functools import cached_property
 
@@ -20,11 +21,12 @@ from sqlmesh.core.model.common import (
 from sqlmesh.core.model.kind import (
     IncrementalByUniqueKeyKind,
     ModelKind,
+    OnDestructiveChange,
     SCDType2ByColumnKind,
     SCDType2ByTimeKind,
     TimeColumn,
     ViewKind,
-    _Incremental,
+    _IncrementalBy,
     model_kind_validator,
 )
 from sqlmesh.core.node import _Node, str_or_exp_to_str
@@ -44,6 +46,8 @@ if t.TYPE_CHECKING:
 
 AuditReference = t.Tuple[str, t.Dict[str, exp.Expression]]
 
+logger = logging.getLogger(__name__)
+
 
 class ModelMeta(_Node):
     """Metadata for models which can be defined in SQL."""
@@ -58,12 +62,15 @@ class ModelMeta(_Node):
     default_catalog: t.Optional[str] = None
     depends_on_: t.Optional[t.Set[str]] = Field(default=None, alias="depends_on")
     columns_to_types_: t.Optional[t.Dict[str, exp.DataType]] = Field(default=None, alias="columns")
-    column_descriptions_: t.Optional[t.Dict[str, str]] = None
+    column_descriptions_: t.Optional[t.Dict[str, str]] = Field(
+        default=None, alias="column_descriptions"
+    )
     audits: t.List[AuditReference] = []
     grains: t.List[exp.Expression] = []
     references: t.List[exp.Expression] = []
     physical_schema_override: t.Optional[str] = None
-    table_properties_: t.Optional[exp.Tuple] = Field(default=None, alias="table_properties")
+    physical_properties_: t.Optional[exp.Tuple] = Field(default=None, alias="physical_properties")
+    virtual_properties_: t.Optional[exp.Tuple] = Field(default=None, alias="virtual_properties")
     session_properties_: t.Optional[exp.Tuple] = Field(default=None, alias="session_properties")
     allow_partials: bool = False
     signals: t.List[exp.Tuple] = []
@@ -89,11 +96,11 @@ class ModelMeta(_Node):
                 return v.name.lower(), {}
 
             for arg in args:
-                if not isinstance(arg, exp.EQ):
+                if not isinstance(arg, (exp.PropertyEQ, exp.EQ)):
                     raise ConfigError(
-                        f"Function '{func}' must be called with key-value arguments like {func}(arg=value)."
+                        f"Function '{func}' must be called with key-value arguments like {func}(arg := value)."
                     )
-                kwargs[arg.left.name] = arg.right
+                kwargs[arg.left.name.lower()] = arg.right
             return func.lower(), kwargs
 
         if isinstance(v, (exp.Tuple, exp.Array)):
@@ -103,16 +110,29 @@ class ModelMeta(_Node):
         if isinstance(v, exp.Expression):
             return [extract(v)]
         if isinstance(v, list):
-            return [
-                (
-                    entry[0].lower(),
-                    {
-                        key: d.parse(value)[0] if isinstance(value, str) else value
-                        for key, value in entry[1].items()
-                    },
+            audits = []
+
+            for entry in v:
+                if isinstance(entry, dict):
+                    args = entry
+                    name = entry.pop("name")
+                elif isinstance(entry, (tuple, list)):
+                    name, args = entry
+                else:
+                    raise ConfigError(f"Audit must be a dictionary or named tuple. Got {entry}.")
+
+                audits.append(
+                    (
+                        name.lower(),
+                        {
+                            key: d.parse_one(value) if isinstance(value, str) else value
+                            for key, value in args.items()
+                        },
+                    )
                 )
-                for entry in v
-            ]
+
+            return audits
+
         return v
 
     @field_validator("tags", mode="before")
@@ -130,7 +150,9 @@ class ModelMeta(_Node):
         cls, v: t.Dict[str, t.Any], values: t.Dict[str, t.Any], normalize: bool = False
     ) -> t.Any:
         dialect = values.get("dialect")
-        _normalize = lambda v: normalize_identifiers(v, dialect=dialect) if normalize else v
+
+        def _normalize(value: t.Any) -> t.Any:
+            return normalize_identifiers(value, dialect=dialect) if normalize else value
 
         if isinstance(v, (exp.Tuple, exp.Array)):
             return [_normalize(e).name for e in v.expressions]
@@ -144,10 +166,17 @@ class ModelMeta(_Node):
 
         return v
 
-    @field_validator("dialect", "storage_format", mode="before")
-    def _string_validator(cls, v: t.Any) -> t.Optional[str]:
+    @field_validator("storage_format", mode="before")
+    def _storage_format_validator(cls, v: t.Any) -> t.Optional[str]:
         return str_or_exp_to_str(v)
 
+    @field_validator("dialect", mode="before")
+    def _dialect_validator(cls, v: t.Any) -> t.Optional[str]:
+        # dialects are parsed as identifiers and may get normalized as uppercase,
+        # so this ensures they'll be stored as lowercase
+        dialect = str_or_exp_to_str(v)
+        return dialect and dialect.lower()
+
     @field_validator("partitioned_by_", mode="before")
     @field_validator_v1_args
     def _partition_by_validator(
@@ -169,7 +198,9 @@ class ModelMeta(_Node):
 
         return partitions
 
-    @field_validator("columns_to_types_", mode="before")
+    @field_validator(
+        "columns_to_types_", "derived_columns_to_types", mode="before", check_fields=False
+    )
     @field_validator_v1_args
     def _columns_validator(
         cls, v: t.Any, values: t.Dict[str, t.Any]
@@ -196,6 +227,41 @@ class ModelMeta(_Node):
 
         return v
 
+    @field_validator("column_descriptions_", mode="before")
+    @field_validator_v1_args
+    def _column_descriptions_validator(
+        cls, vs: t.Any, values: t.Dict[str, t.Any]
+    ) -> t.Optional[t.Dict[str, str]]:
+        dialect = values.get("dialect")
+
+        if vs is None:
+            return None
+
+        if isinstance(vs, exp.Paren):
+            vs = vs.flatten()
+
+        if isinstance(vs, (exp.Tuple, exp.Array)):
+            vs = vs.expressions
+
+        raw_col_descriptions = (
+            vs if isinstance(vs, dict) else {v.this.name: v.expression.name for v in vs}
+        )
+
+        col_descriptions = {
+            normalize_identifiers(k, dialect=dialect).name: v
+            for k, v in raw_col_descriptions.items()
+        }
+
+        columns_to_types = values.get("columns_to_types_")
+        if columns_to_types:
+            for column_name in col_descriptions:
+                if column_name not in columns_to_types:
+                    raise ConfigError(
+                        f"In model '{values['name']}', a description is provided for column '{column_name}' but it is not a column in the model."
+                    )
+
+        return col_descriptions
+
     @field_validator("grains", "references", mode="before")
     @field_validator_v1_args
     def _refs_validator(cls, vs: t.Any, values: t.Dict[str, t.Any]) -> t.List[exp.Expression]:
@@ -254,6 +320,21 @@ class ModelMeta(_Node):
                     f"Cannot use argument 'grain' ({grain}) with 'grains' ({grains}), use only grains"
                 )
             values["grains"] = ensure_list(grain)
+
+        table_properties = values.pop("table_properties", None)
+        if table_properties:
+            if not isinstance(table_properties, str):
+                # Do not warn when deserializing from the state.
+                model_name = values["name"]
+                logger.warning(
+                    f"Model '{model_name}' is using the `table_properties` attribute which is deprecated. Please use `physical_properties` instead."
+                )
+            physical_properties = values.get("physical_properties")
+            if physical_properties:
+                raise ConfigError(
+                    f"Cannot use argument 'table_properties' ({table_properties}) with 'physical_properties' ({physical_properties}), use only physical_properties."
+                )
+            values["physical_properties"] = table_properties
         return values
 
     @model_validator(mode="after")
@@ -269,6 +350,8 @@ class ModelMeta(_Node):
             for field in ("partitioned_by_", "clustered_by"):
                 if values.get(field) and not kind.is_materialized:
                     raise ValueError(f"{field} field cannot be set for {kind} models")
+            if kind.is_incremental_by_partition and not values.get("partitioned_by_"):
+                raise ValueError(f"partitioned_by field is required for {kind.name} models")
         return values
 
     @property
@@ -300,7 +383,7 @@ class ModelMeta(_Node):
     @property
     def lookback(self) -> int:
         """The incremental lookback window."""
-        return (self.kind.lookback if isinstance(self.kind, _Incremental) else 0) or 0
+        return (self.kind.lookback if isinstance(self.kind, _IncrementalBy) else 0) or 0
 
     def lookback_start(self, start: TimeLike) -> TimeLike:
         if self.lookback == 0:
@@ -321,13 +404,17 @@ class ModelMeta(_Node):
         return getattr(self.kind, "batch_concurrency", None)
 
     @cached_property
-    def table_properties(self) -> t.Dict[str, exp.Expression]:
-        """A dictionary of table properties."""
-        if self.table_properties_:
-            table_properties = {}
-            for expression in self.table_properties_.expressions:
-                table_properties[expression.this.name] = expression.expression
-            return table_properties
+    def physical_properties(self) -> t.Dict[str, exp.Expression]:
+        """A dictionary of properties that will be applied to the physical layer. It replaces table_properties which is deprecated."""
+        if self.physical_properties_:
+            return {e.this.name: e.expression for e in self.physical_properties_.expressions}
+        return {}
+
+    @cached_property
+    def virtual_properties(self) -> t.Dict[str, exp.Expression]:
+        """A dictionary of properties that will be applied to the virtual layer."""
+        if self.virtual_properties_:
+            return {e.this.name: e.expression for e in self.virtual_properties_.expressions}
         return {}
 
     @property
@@ -386,3 +473,7 @@ class ModelMeta(_Node):
         return normalize_model_name(
             self.name, default_catalog=self.default_catalog, dialect=self.dialect
         )
+
+    @property
+    def on_destructive_change(self) -> OnDestructiveChange:
+        return getattr(self.kind, "on_destructive_change", OnDestructiveChange.ALLOW)
