@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import logging
 import re
 import sys
 import typing as t
@@ -7,6 +8,8 @@ from collections import defaultdict
 from datetime import datetime
 from functools import cached_property
 
+
+from sqlmesh.core.console import Console, get_console
 from sqlmesh.core.config import (
     AutoCategorizationMode,
     CategorizerConfig,
@@ -15,6 +18,7 @@ from sqlmesh.core.config import (
 from sqlmesh.core.context_diff import ContextDiff
 from sqlmesh.core.environment import EnvironmentNamingInfo
 from sqlmesh.core.plan.definition import Plan, SnapshotMapping, earliest_interval_start
+from sqlmesh.core.schema_diff import SchemaDiffer, has_drop_alteration
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Snapshot,
@@ -22,11 +26,13 @@ from sqlmesh.core.snapshot import (
     categorize_change,
 )
 from sqlmesh.core.snapshot.definition import Interval, SnapshotId, start_date
-from sqlmesh.utils import random_id
+from sqlmesh.utils import columns_to_types_all_known, random_id
 from sqlmesh.utils.dag import DAG
 from sqlmesh.utils.date import TimeLike, now, to_datetime, yesterday_ds
 from sqlmesh.utils.errors import NoChangesPlanError, PlanError, SQLMeshError
 
+logger = logging.getLogger(__name__)
+
 
 class PlanBuilder:
     """Plan Builder constructs a Plan based on user choices for how they want to backfill, preview, etc. their changes.
@@ -47,6 +53,7 @@ class PlanBuilder:
         skip_backfill: Whether to skip the backfill step.
         is_dev: Whether this plan is for development purposes.
         forward_only: Whether the purpose of the plan is to make forward only changes.
+        allow_destructive_models: A list of fully qualified model names whose forward-only changes are allowed to be destructive.
         environment_ttl: The period of time that a development environment should exist before being deleted.
         categorizer_config: Auto categorization settings.
         auto_categorization_enabled: Whether to apply auto categorization.
@@ -61,11 +68,13 @@ class PlanBuilder:
         ensure_finalized_snapshots: Whether to compare against snapshots from the latest finalized
             environment state, or to use whatever snapshots are in the current environment state even if
             the environment is not finalized.
+        engine_schema_differ: Schema differ from the context engine adapter.
     """
 
     def __init__(
         self,
         context_diff: ContextDiff,
+        engine_schema_differ: SchemaDiffer,
         start: t.Optional[TimeLike] = None,
         end: t.Optional[TimeLike] = None,
         execution_time: t.Optional[TimeLike] = None,
@@ -76,6 +85,7 @@ class PlanBuilder:
         skip_backfill: bool = False,
         is_dev: bool = False,
         forward_only: bool = False,
+        allow_destructive_models: t.Optional[t.Iterable[str]] = None,
         environment_ttl: t.Optional[str] = None,
         environment_suffix_target: EnvironmentSuffixTarget = EnvironmentSuffixTarget.default,
         environment_catalog_mapping: t.Optional[t.Dict[re.Pattern, str]] = None,
@@ -88,12 +98,16 @@ class PlanBuilder:
         enable_preview: bool = False,
         end_bounded: bool = False,
         ensure_finalized_snapshots: bool = False,
+        console: t.Optional[Console] = None,
     ):
         self._context_diff = context_diff
         self._no_gaps = no_gaps
         self._skip_backfill = skip_backfill
         self._is_dev = is_dev
         self._forward_only = forward_only
+        self._allow_destructive_models = set(
+            allow_destructive_models if allow_destructive_models is not None else []
+        )
         self._enable_preview = enable_preview
         self._end_bounded = end_bounded
         self._ensure_finalized_snapshots = ensure_finalized_snapshots
@@ -101,12 +115,14 @@ class PlanBuilder:
         self._categorizer_config = categorizer_config or CategorizerConfig()
         self._auto_categorization_enabled = auto_categorization_enabled
         self._include_unmodified = include_unmodified
-        self._restate_models = set(restate_models or [])
+        self._restate_models = set(restate_models) if restate_models is not None else None
         self._effective_from = effective_from
         self._execution_time = execution_time
         self._backfill_models = backfill_models
         self._end = end or default_end
         self._apply = apply
+        self._engine_schema_differ = engine_schema_differ
+        self._console = console or get_console()
 
         self._start = start
         if not self._start and self._forward_only_preview_needed:
@@ -171,7 +187,7 @@ class PlanBuilder:
     def apply(self) -> None:
         """Builds and applies the plan."""
         if not self._apply:
-            raise SQLMeshError(f"Plan was not initialized with an applier.")
+            raise SQLMeshError("Plan was not initialized with an applier.")
         self._apply(self.build())
 
     def build(self) -> Plan:
@@ -190,6 +206,7 @@ class PlanBuilder:
         dag = self._build_dag()
         directly_modified, indirectly_modified = self._build_directly_and_indirectly_modified(dag)
 
+        self._check_destructive_changes(directly_modified)
         self._categorize_snapshots(dag, directly_modified, indirectly_modified)
         self._adjust_new_snapshot_intervals()
 
@@ -231,6 +248,7 @@ class PlanBuilder:
             skip_backfill=self._skip_backfill,
             no_gaps=self._no_gaps,
             forward_only=self._forward_only,
+            allow_destructive_models=t.cast(t.Set, self._allow_destructive_models),
             include_unmodified=self._include_unmodified,
             environment_ttl=self._environment_ttl,
             environment_naming_info=self.environment_naming_info,
@@ -286,10 +304,17 @@ class PlanBuilder:
                 return False
             return not snapshot.is_symbolic and not snapshot.is_seed
 
-        restatements: t.Dict[SnapshotId, Interval] = {}
-        dummy_interval = (sys.maxsize, -sys.maxsize)
         restate_models = self._restate_models
+        if restate_models == set():
+            # This is a warning but we print this as error since the Console is lacking API for warnings.
+            self._console.log_error(
+                "Provided restated models do not match any models. No models will be included in plan."
+            )
+            return {}
+
+        restatements: t.Dict[SnapshotId, Interval] = {}
         forward_only_preview_needed = self._forward_only_preview_needed
+        is_preview = False
         if not restate_models and forward_only_preview_needed:
             # Add model names for new forward-only snapshots to the restatement list
             # in order to compute previews.
@@ -298,18 +323,28 @@ class PlanBuilder:
                 for s in self._context_diff.new_snapshots.values()
                 if s.is_materialized and (self._forward_only or s.model.forward_only)
             }
+            is_preview = True
+
         if not restate_models:
-            return restatements
+            return {}
 
         # Add restate snapshots and their downstream snapshots
+        dummy_interval = (sys.maxsize, -sys.maxsize)
         for model_fqn in restate_models:
             snapshot = self._model_fqn_to_snapshot.get(model_fqn)
-            if not snapshot or (
-                not forward_only_preview_needed and not is_restateable_snapshot(snapshot)
-            ):
-                raise PlanError(
-                    f"Cannot restate from '{model_fqn}'. Either such model doesn't exist, no other materialized model references it, or restatement was disabled for this model."
-                )
+            if not snapshot:
+                raise PlanError(f"Cannot restate model '{model_fqn}'. Model does not exist.")
+            if not forward_only_preview_needed:
+                if not self._is_dev and snapshot.disable_restatement:
+                    # This is a warning but we print this as error since the Console is lacking API for warnings.
+                    self._console.log_error(
+                        f"Cannot restate model '{model_fqn}'. Restatement is disabled for this model."
+                    )
+                    continue
+                elif snapshot.is_symbolic or snapshot.is_seed:
+                    logger.info("Skipping restatement for model '%s'", model_fqn)
+                    continue
+
             restatements[snapshot.snapshot_id] = dummy_interval
             for downstream_s_id in dag.downstream(snapshot.snapshot_id):
                 if is_restateable_snapshot(self._context_diff.snapshots[downstream_s_id]):
@@ -325,6 +360,7 @@ class PlanBuilder:
                 self._end or now(),
                 self._execution_time,
                 strict=False,
+                is_preview=is_preview,
             )
             # Since we are traversing the graph in topological order and the largest interval range is pushed down
             # the graph we just have to check our immediate parents in the graph and not the whole upstream graph.
@@ -403,6 +439,41 @@ class PlanBuilder:
             if new.is_forward_only:
                 new.dev_intervals = new.intervals.copy()
 
+    def _check_destructive_changes(self, directly_modified: t.Set[SnapshotId]) -> None:
+        for s_id in sorted(directly_modified):
+            snapshot = self._context_diff.snapshots[s_id]
+            # should we raise/warn if this snapshot has/inherits a destructive change?
+            should_raise_or_warn = (
+                self._is_forward_only_change(s_id) or self._forward_only
+            ) and snapshot.needs_destructive_check(self._allow_destructive_models)
+
+            if not should_raise_or_warn or not snapshot.is_model:
+                continue
+
+            new, old = self._context_diff.modified_snapshots[snapshot.name]
+
+            # we must know all columns_to_types to determine whether a change is destructive
+            old_columns_to_types = old.model.columns_to_types or {}
+            new_columns_to_types = new.model.columns_to_types or {}
+
+            if columns_to_types_all_known(old_columns_to_types) and columns_to_types_all_known(
+                new_columns_to_types
+            ):
+                schema_diff = self._engine_schema_differ.compare_columns(
+                    new.name,
+                    old_columns_to_types,
+                    new_columns_to_types,
+                )
+
+                if has_drop_alteration(schema_diff):
+                    warning_msg = f"Plan results in a destructive change to forward-only model '{snapshot.name}'s schema"
+                    if snapshot.model.on_destructive_change.is_warn:
+                        logger.warning(warning_msg)
+                    else:
+                        raise PlanError(
+                            f"{warning_msg}. To allow this, change the model's `on_destructive_change` setting to `warn` or `allow` or include it in the plan's `--allow-destructive-model` option."
+                        )
+
     def _categorize_snapshots(
         self,
         dag: DAG[SnapshotId],
@@ -432,7 +503,7 @@ class PlanBuilder:
                             snapshot.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
                         else:
                             snapshot.categorize_as(SnapshotChangeCategory.NON_BREAKING)
-                    elif self._is_forward_only_model(s_id) and is_directly_modified:
+                    elif self._is_forward_only_change(s_id) and is_directly_modified:
                         self._set_choice(
                             snapshot,
                             SnapshotChangeCategory.FORWARD_ONLY,
@@ -487,7 +558,7 @@ class PlanBuilder:
                         # an indirectly modified snapshot to be created because of a new parent
                         snapshot.categorize_as(
                             SnapshotChangeCategory.FORWARD_ONLY
-                            if self._is_forward_only_model(s_id)
+                            if self._is_forward_only_change(s_id)
                             else SnapshotChangeCategory.INDIRECT_BREAKING
                         )
                     else:
@@ -497,7 +568,7 @@ class PlanBuilder:
             elif s_id in self._context_diff.added and self._is_new_snapshot(snapshot):
                 snapshot.categorize_as(
                     SnapshotChangeCategory.FORWARD_ONLY
-                    if self._is_forward_only_model(s_id)
+                    if self._is_forward_only_change(s_id)
                     else SnapshotChangeCategory.BREAKING
                 )
 
@@ -529,7 +600,7 @@ class PlanBuilder:
             if not self._is_new_snapshot(child_snapshot):
                 continue
 
-            is_forward_only_child = self._is_forward_only_model(child_s_id)
+            is_forward_only_child = self._is_forward_only_change(child_s_id)
 
             if is_breaking_choice:
                 child_snapshot.categorize_as(
@@ -543,9 +614,8 @@ class PlanBuilder:
                 child_snapshot.categorize_as(SnapshotChangeCategory.INDIRECT_NON_BREAKING)
 
             for upstream_id in directly_modified:
-                if (
-                    upstream_id == snapshot.snapshot_id
-                    or child_s_id not in indirectly_modified.get(upstream_id, set())
+                if upstream_id == snapshot.snapshot_id or child_s_id not in indirectly_modified.get(
+                    upstream_id, set()
                 ):
                     continue
 
@@ -577,8 +647,13 @@ class PlanBuilder:
             if not snapshot.disable_restatement:
                 snapshot.effective_from = self._effective_from
 
-    def _is_forward_only_model(self, s_id: SnapshotId) -> bool:
+    def _is_forward_only_change(self, s_id: SnapshotId) -> bool:
         snapshot = self._context_diff.snapshots[s_id]
+        if snapshot.name in self._context_diff.modified_snapshots:
+            _, old = self._context_diff.modified_snapshots[snapshot.name]
+            # If the model kind has changed, then we should not consider this to be a forward-only change.
+            if snapshot.is_model and old.model.kind.name != snapshot.model.kind.name:
+                return False
         return (
             snapshot.is_model
             and snapshot.model.forward_only
@@ -628,7 +703,9 @@ class PlanBuilder:
                 )
 
     def _ensure_no_new_snapshots_with_restatements(self) -> None:
-        if self._restate_models and self._context_diff.new_snapshots:
+        if self._restate_models is not None and (
+            self._context_diff.new_snapshots or self._context_diff.modified_snapshots
+        ):
             raise PlanError(
                 "Model changes and restatements can't be a part of the same plan. "
                 "Revert or apply changes before proceeding with restatements."
