@@ -48,6 +48,7 @@ class BaseExpressionRenderer:
         default_catalog: t.Optional[str] = None,
         quote_identifiers: bool = True,
         model_fqn: t.Optional[str] = None,
+        normalize_identifiers: bool = True,
     ):
         self._expression = expression
         self._dialect = dialect
@@ -57,6 +58,7 @@ class BaseExpressionRenderer:
         self._python_env = python_env or {}
         self._only_execution_time = only_execution_time
         self._default_catalog = default_catalog
+        self._normalize_identifiers = normalize_identifiers
         self._quote_identifiers = quote_identifiers
         self.update_schema({} if schema is None else schema)
         self._cache: t.List[t.Optional[exp.Expression]] = []
@@ -100,12 +102,17 @@ class BaseExpressionRenderer:
             return self._cache
 
         if self._model_fqn and "this_model" not in kwargs:
-            if snapshots:
-                kwargs["this_model"] = self._to_table_mapping(
-                    [snapshots[self._model_fqn]], deployability_index
-                )
-            else:
-                kwargs["this_model"] = self._model_fqn
+            kwargs["this_model"] = exp.to_table(
+                self._to_table_mapping(
+                    (
+                        [snapshots[self._model_fqn]]
+                        if snapshots and self._model_fqn in snapshots
+                        else []
+                    ),
+                    deployability_index,
+                ).get(self._model_fqn, self._model_fqn),
+                dialect=self._dialect,
+            ).sql(dialect=self._dialect, identify=True)
 
         expressions = [self._expression]
 
@@ -124,6 +131,7 @@ class BaseExpressionRenderer:
             table_mapping=table_mapping,
             deployability_index=deployability_index,
             default_catalog=self._default_catalog,
+            runtime_stage=runtime_stage.value,
         )
 
         if isinstance(self._expression, d.Jinja):
@@ -160,6 +168,7 @@ class BaseExpressionRenderer:
             ),
             snapshots=snapshots,
             default_catalog=self._default_catalog,
+            path=self._path,
         )
 
         for definition in self._macro_definitions:
@@ -282,9 +291,12 @@ class BaseExpressionRenderer:
 
     @contextmanager
     def _normalize_and_quote(self, query: E) -> t.Iterator[E]:
-        with d.normalize_and_quote(
-            query, self._dialect, self._default_catalog, quote=self._quote_identifiers
-        ) as query:
+        if self._normalize_identifiers:
+            with d.normalize_and_quote(
+                query, self._dialect, self._default_catalog, quote=self._quote_identifiers
+            ) as query:
+                yield query
+        else:
             yield query
 
     def _should_cache(self, runtime_stage: RuntimeStage, *args: t.Any) -> bool:
