@@ -8,7 +8,6 @@ from datetime import datetime
 from sqlmesh.core import constants as c
 from sqlmesh.core.console import Console, get_console
 from sqlmesh.core.environment import EnvironmentNamingInfo
-from sqlmesh.core.model import SeedModel
 from sqlmesh.core.notification_target import (
     NotificationEvent,
     NotificationTargetManager,
@@ -141,6 +140,7 @@ class Scheduler:
         end: TimeLike,
         execution_time: TimeLike,
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         """Evaluate a snapshot and add the processed interval to the state sync.
@@ -151,6 +151,7 @@ class Scheduler:
             end: The end datetime to render.
             execution_time: The date/time time reference to use for execution time. Defaults to now.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            batch_index: If the snapshot is part of a batch of related snapshots; which index in the batch is it
             kwargs: Additional kwargs to pass to the renderer.
         """
         validate_date_range(start, end)
@@ -160,11 +161,6 @@ class Scheduler:
         }
         snapshots[snapshot.name] = snapshot
 
-        if isinstance(snapshot.node, SeedModel) and not snapshot.node.is_hydrated:
-            snapshot = self.state_sync.get_snapshots([snapshot], hydrate_seeds=True)[
-                snapshot.snapshot_id
-            ]
-
         is_deployable = deployability_index.is_deployable(snapshot)
 
         wap_id = self.snapshot_evaluator.evaluate(
@@ -174,6 +170,7 @@ class Scheduler:
             execution_time=execution_time,
             snapshots=snapshots,
             deployability_index=deployability_index,
+            batch_index=batch_index,
             **kwargs,
         )
         try:
@@ -290,7 +287,7 @@ class Scheduler:
             try:
                 assert execution_time  # mypy
                 assert deployability_index  # mypy
-                self.evaluate(snapshot, start, end, execution_time, deployability_index)
+                self.evaluate(snapshot, start, end, execution_time, deployability_index, batch_idx)
                 evaluation_duration_ms = now_timestamp() - execution_start_ts
             finally:
                 self.console.update_snapshot_evaluation_progress(
