@@ -319,6 +319,9 @@ class SnapshotInfoMixin(ModelKindMixin):
             version: The snapshot version.
             is_deployable: Indicates whether to return the table name for deployment to production.
         """
+        if self.is_external:
+            return self.name
+
         is_dev_table = not is_deployable
         if is_dev_table:
             version = self.temp_version_get_or_generate()
@@ -656,14 +659,11 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
         else:
             self.intervals = merged_intervals
 
-    def remove_interval(
-        self, interval: Interval, execution_time: t.Optional[TimeLike] = None
-    ) -> None:
+    def remove_interval(self, interval: Interval) -> None:
         """Remove an interval from the snapshot.
 
         Args:
             interval: The interval to remove.
-            execution_time: The date/time time reference to use for execution time. Defaults to now.
         """
         self.intervals = remove_interval(self.intervals, *interval)
         self.dev_intervals = remove_interval(self.dev_intervals, *interval)
@@ -675,9 +675,21 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
         execution_time: t.Optional[TimeLike] = None,
         *,
         strict: bool = True,
+        is_preview: bool = False,
     ) -> Interval:
+        """Get the interval that should be removed from the snapshot.
+
+        Args:
+            start: The start date/time of the interval to remove.
+            end: The end date/time of the interval to removed.
+            execution_time: The time the interval is being removed.
+            strict: Whether to fail when the inclusive start is the same as the exclusive end.
+            is_preview: Whether the interval needs to be removed for a preview of forward-only changes.
+                When previewing, we are not actually restating a model, but removing an interval to trigger
+                a run.
+        """
         end = execution_time or now() if self.depends_on_past else end
-        if self.full_history_restatement_only and self.intervals:
+        if not is_preview and self.full_history_restatement_only and self.intervals:
             start = self.intervals[0][0]
         return self.inclusive_exclusive(start, end, strict)
 
@@ -740,12 +752,16 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
         if self.identifier == other.identifier or (
             # Indirect Non-Breaking snapshots share the dev table with its previous version.
             # The same applies to migrated snapshots.
-            (self.is_indirect_non_breaking or self.migrated)
-            and other.snapshot_id in previous_ids
+            (self.is_indirect_non_breaking or self.migrated) and other.snapshot_id in previous_ids
         ):
             for start, end in other.dev_intervals:
                 self.add_interval(start, end, is_dev=True)
 
+    @property
+    def evaluatable(self) -> bool:
+        """Whether or not a snapshot should be evaluated and have intervals."""
+        return bool(not self.is_symbolic or self.model.audits)
+
     def missing_intervals(
         self,
         start: TimeLike,
@@ -765,7 +781,6 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
             start: The start date/time of the interval (inclusive)
             end: The end date/time of the interval (inclusive if the type is date, exclusive otherwise)
             execution_time: The date/time time reference to use for execution time. Defaults to now.
-            restatements: A set of snapshot names being restated
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
             ignore_cron: Whether to ignore the node's cron schedule.
             end_bounded: If set to true, the returned intervals will be bounded by the target end date, disregarding lookback,
@@ -799,7 +814,7 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
             self.intervals if deployability_index.is_representative(self) else self.dev_intervals
         )
 
-        if self.is_symbolic or (self.is_seed and intervals):
+        if not self.evaluatable or (self.is_seed and intervals):
             return []
 
         allow_partials = not end_bounded and self.is_model and self.model.allow_partials
@@ -931,6 +946,16 @@ class Snapshot(PydanticModel, SnapshotInfoMixin):
             else default
         )
 
+    def needs_destructive_check(
+        self,
+        allow_destructive_snapshots: t.Set[str],
+    ) -> bool:
+        return (
+            self.is_model
+            and not self.model.on_destructive_change.is_allow
+            and self.name not in allow_destructive_snapshots
+        )
+
     @property
     def physical_schema(self) -> str:
         if self.physical_schema_ is not None:
@@ -1211,7 +1236,7 @@ class DeployabilityIndex(PydanticModel, frozen=True):
             ):
                 return
 
-            if deployable:
+            if deployable and node in snapshots:
                 snapshot = snapshots[node]
                 # Capture uncategorized snapshot which represents a forward-only model.
                 is_forward_only_model = (
@@ -1496,7 +1521,7 @@ def missing_intervals(
     deployability_index = deployability_index or DeployabilityIndex.all_deployable()
 
     for snapshot in snapshots:
-        if snapshot.is_symbolic:
+        if not snapshot.evaluatable:
             continue
         interval = restatements.get(snapshot.snapshot_id)
         snapshot_start_date = start_dt
@@ -1505,7 +1530,7 @@ def missing_intervals(
             snapshot_start_date, snapshot_end_date = (to_datetime(i) for i in interval)
             snapshot = snapshot.copy()
             snapshot.intervals = snapshot.intervals.copy()
-            snapshot.remove_interval(interval, execution_time)
+            snapshot.remove_interval(interval)
 
         missing_interval_end_date = snapshot_end_date
         node_end_date = snapshot.node.end
