@@ -43,10 +43,12 @@ from sqlmesh.core.macros import RuntimeStage
 from sqlmesh.core.model import (
     IncrementalUnmanagedKind,
     Model,
+    SeedModel,
     SCDType2ByColumnKind,
     SCDType2ByTimeKind,
     ViewKind,
 )
+from sqlmesh.core.schema_diff import has_drop_alteration
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Intervals,
@@ -98,6 +100,7 @@ class SnapshotEvaluator:
         execution_time: TimeLike,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: t.Optional[DeployabilityIndex] = None,
+        batch_index: int = 0,
         **kwargs: t.Any,
     ) -> t.Optional[str]:
         """Renders the snapshot's model, executes it and stores the result in the snapshot's physical table.
@@ -109,6 +112,7 @@ class SnapshotEvaluator:
             execution_time: The date/time time reference to use for execution time.
             snapshots: All upstream snapshots (by name) to use for expansion and mapping of physical locations.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            batch_index: If the snapshot is part of a batch of related snapshots; which index in the batch is it
             kwargs: Additional kwargs to pass to the renderer.
 
         Returns:
@@ -121,6 +125,7 @@ class SnapshotEvaluator:
             execution_time,
             snapshots,
             deployability_index=deployability_index,
+            batch_index=batch_index,
             **kwargs,
         )
         if result is None or isinstance(result, str):
@@ -234,6 +239,7 @@ class SnapshotEvaluator:
         snapshots: t.Dict[SnapshotId, Snapshot],
         deployability_index: t.Optional[DeployabilityIndex] = None,
         on_complete: t.Optional[t.Callable[[SnapshotInfoLike], None]] = None,
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         """Creates a physical snapshot schema and table for the given collection of snapshots.
 
@@ -242,6 +248,7 @@ class SnapshotEvaluator:
             snapshots: Mapping of snapshot ID to snapshot.
             deployability_index: Determines snapshots that are deployable in the context of this creation.
             on_complete: A callback to call on each successfully created snapshot.
+            allow_destructive_snapshots: Set of snapshots that are allowed to have destructive schema changes.
         """
         snapshots_with_table_names = defaultdict(set)
         tables_by_schema = defaultdict(set)
@@ -283,23 +290,29 @@ class SnapshotEvaluator:
         with self.concurrent_context():
             concurrent_apply_to_snapshots(
                 snapshots_to_create,
-                lambda s: self._create_snapshot(s, snapshots, deployability_index, on_complete),
+                lambda s: self._create_snapshot(
+                    s, snapshots, deployability_index, on_complete, allow_destructive_snapshots
+                ),
                 self.ddl_concurrent_tasks,
             )
 
     def migrate(
-        self, target_snapshots: t.Iterable[Snapshot], snapshots: t.Dict[SnapshotId, Snapshot]
+        self,
+        target_snapshots: t.Iterable[Snapshot],
+        snapshots: t.Dict[SnapshotId, Snapshot],
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         """Alters a physical snapshot table to match its snapshot's schema for the given collection of snapshots.
 
         Args:
             target_snapshots: Target snapshots.
             snapshots: Mapping of snapshot ID to snapshot.
+            allow_destructive_snapshots: Set of snapshots that are allowed to have destructive schema changes.
         """
         with self.concurrent_context():
             concurrent_apply_to_snapshots(
                 target_snapshots,
-                lambda s: self._migrate_snapshot(s, snapshots),
+                lambda s: self._migrate_snapshot(s, snapshots, allow_destructive_snapshots),
                 self.ddl_concurrent_tasks,
             )
 
@@ -443,6 +456,7 @@ class SnapshotEvaluator:
         snapshots: t.Dict[str, Snapshot],
         limit: t.Optional[int] = None,
         deployability_index: t.Optional[DeployabilityIndex] = None,
+        batch_index: int = 0,
         **kwargs: t.Any,
     ) -> DF | str | None:
         """Renders the snapshot's model and executes it. The return value depends on whether the limit was specified.
@@ -455,9 +469,10 @@ class SnapshotEvaluator:
             snapshots: All upstream snapshots to use for expansion and mapping of physical locations.
             limit: If limit is not None, the query will not be persisted but evaluated and returned as a dataframe.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            batch_index: If the snapshot is part of a batch of related snapshots; which index in the batch is it
             kwargs: Additional kwargs to pass to the renderer.
         """
-        if not snapshot.is_model:
+        if not snapshot.is_model or snapshot.is_seed:
             return None
 
         model = snapshot.model
@@ -481,6 +496,7 @@ class SnapshotEvaluator:
                     query_or_df,
                     snapshots,
                     deployability_index,
+                    batch_index=batch_index,
                     start=start,
                     end=end,
                     execution_time=execution_time,
@@ -493,6 +509,7 @@ class SnapshotEvaluator:
                     query_or_df,
                     snapshots,
                     deployability_index,
+                    batch_index=batch_index,
                     start=start,
                     end=end,
                     execution_time=execution_time,
@@ -543,14 +560,23 @@ class SnapshotEvaluator:
 
             if limit is not None:
                 query_or_df = next(queries_or_dfs)
-                if isinstance(query_or_df, exp.Select):
-                    existing_limit = query_or_df.args.get("limit")
-                    if existing_limit:
-                        limit = min(
-                            limit,
-                            execute(exp.select(existing_limit.expression)).rows[0][0],
-                        )
-                return query_or_df.head(limit) if hasattr(query_or_df, "head") else self.adapter._fetch_native_df(query_or_df.limit(limit))  # type: ignore
+                if isinstance(query_or_df, pd.DataFrame):
+                    return query_or_df.head(limit)
+                if not isinstance(query_or_df, exp.Expression):
+                    # We assume that if this branch is reached, `query_or_df` is a pyspark / snowpark dataframe,
+                    # so we use `limit` instead of `head` to get back a dataframe instead of List[Row]
+                    # https://spark.apache.org/docs/3.1.1/api/python/reference/api/pyspark.sql.DataFrame.head.html#pyspark.sql.DataFrame.head
+                    return query_or_df.limit(limit)
+
+                assert isinstance(query_or_df, exp.Query)
+
+                existing_limit = query_or_df.args.get("limit")
+                if existing_limit:
+                    limit = min(limit, execute(exp.select(existing_limit.expression)).rows[0][0])
+                    assert limit is not None
+
+                return self.adapter._fetch_native_df(query_or_df.limit(limit))
+
             # DataFrames, unlike SQL expressions, can provide partial results by yielding dataframes. As a result,
             # if the engine supports INSERT OVERWRITE or REPLACE WHERE and the snapshot is incremental by time range, we risk
             # having a partial result since each dataframe write can re-truncate partitions. To avoid this, we
@@ -566,7 +592,7 @@ class SnapshotEvaluator:
                 query_or_df = reduce(
                     lambda a, b: (
                         pd.concat([a, b], ignore_index=True)  # type: ignore
-                        if self.adapter.is_pandas_df(a)
+                        if isinstance(a, pd.DataFrame)
                         else a.union_all(b)  # type: ignore
                     ),  # type: ignore
                     queries_or_dfs,
@@ -587,6 +613,7 @@ class SnapshotEvaluator:
         snapshots: t.Dict[SnapshotId, Snapshot],
         deployability_index: t.Optional[DeployabilityIndex],
         on_complete: t.Optional[t.Callable[[SnapshotInfoLike], None]],
+        allow_destructive_snapshots: t.Set[str],
     ) -> None:
         if not snapshot.is_model:
             return
@@ -636,7 +663,13 @@ class SnapshotEvaluator:
                 )
                 try:
                     self.adapter.clone_table(target_table_name, snapshot.table_name(), replace=True)
-                    self.adapter.alter_table(target_table_name, tmp_table_name)
+                    alter_expressions = self.adapter.get_alter_expressions(
+                        target_table_name, tmp_table_name
+                    )
+                    _check_destructive_schema_change(
+                        snapshot, alter_expressions, allow_destructive_snapshots
+                    )
+                    self.adapter.alter_table(alter_expressions)
                 finally:
                     self.adapter.drop_table(tmp_table_name)
             else:
@@ -658,7 +691,10 @@ class SnapshotEvaluator:
             on_complete(snapshot)
 
     def _migrate_snapshot(
-        self, snapshot: Snapshot, snapshots: t.Dict[SnapshotId, Snapshot]
+        self,
+        snapshot: Snapshot,
+        snapshots: t.Dict[SnapshotId, Snapshot],
+        allow_destructive_snapshots: t.Set[str],
     ) -> None:
         if (
             not snapshot.is_paused
@@ -679,7 +715,11 @@ class SnapshotEvaluator:
         tmp_table_name = snapshot.table_name(is_deployable=False)
         target_table_name = snapshot.table_name()
         _evaluation_strategy(snapshot, self.adapter).migrate(
-            snapshot, parent_snapshots_by_name, target_table_name, tmp_table_name
+            snapshot,
+            parent_snapshots_by_name,
+            target_table_name,
+            tmp_table_name,
+            allow_destructive_snapshots,
         )
 
     def _promote_snapshot(
@@ -817,12 +857,16 @@ def _evaluation_strategy(snapshot: SnapshotInfoLike, adapter: EngineAdapter) ->
         klass = EmbeddedStrategy
     elif snapshot.is_symbolic or snapshot.is_audit:
         klass = SymbolicStrategy
-    elif snapshot.is_full or snapshot.is_seed:
+    elif snapshot.is_full:
         klass = FullRefreshStrategy
+    elif snapshot.is_seed:
+        klass = SeedStrategy
     elif snapshot.is_incremental_by_time_range:
         klass = IncrementalByTimeRangeStrategy
     elif snapshot.is_incremental_by_unique_key:
         klass = IncrementalByUniqueKeyStrategy
+    elif snapshot.is_incremental_by_partition:
+        klass = IncrementalByPartitionStrategy
     elif snapshot.is_incremental_unmanaged:
         klass = IncrementalUnmanagedStrategy
     elif snapshot.is_view:
@@ -847,6 +891,7 @@ class EvaluationStrategy(abc.ABC):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         """Inserts the given query or a DataFrame into the target table or replaces a view.
@@ -857,6 +902,7 @@ class EvaluationStrategy(abc.ABC):
             query_or_df: The query or DataFrame to insert.
             snapshots: Parent snapshots.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            batch_index: Snapshot position in the current batch
         """
 
     @abc.abstractmethod
@@ -867,6 +913,7 @@ class EvaluationStrategy(abc.ABC):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         """Appends the given query or a DataFrame to the existing table.
@@ -877,6 +924,7 @@ class EvaluationStrategy(abc.ABC):
             query_or_df: The query or DataFrame to insert.
             snapshots: Parent snapshots.
             deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+            batch_index: Snapshot position in the current batch
         """
 
     @abc.abstractmethod
@@ -905,6 +953,7 @@ class EvaluationStrategy(abc.ABC):
         snapshots: t.Dict[str, Snapshot],
         target_table_name: str,
         source_table_name: str,
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         """Migrates the target table schema so that it corresponds to the source table schema.
 
@@ -913,6 +962,7 @@ class EvaluationStrategy(abc.ABC):
             snapshots: Parent snapshots.
             target_table_name: The target table name.
             source_table_name: The source table name.
+            allow_destructive_snapshots: Set of snapshots that are allowed to have destructive schema changes.
         """
 
     @abc.abstractmethod
@@ -969,7 +1019,7 @@ class EvaluationStrategy(abc.ABC):
             partitioned_by=model.partitioned_by,
             partition_interval_unit=model.interval_unit,
             clustered_by=model.clustered_by,
-            table_properties=model.table_properties,
+            table_properties=model.physical_properties,
             table_description=model.description,
             column_descriptions=model.column_descriptions,
         )
@@ -983,6 +1033,7 @@ class SymbolicStrategy(EvaluationStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         pass
@@ -994,6 +1045,7 @@ class SymbolicStrategy(EvaluationStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         pass
@@ -1014,6 +1066,7 @@ class SymbolicStrategy(EvaluationStrategy):
         snapshots: t.Dict[str, Snapshot],
         target_table_name: str,
         source_table_name: str,
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         pass
 
@@ -1066,6 +1119,7 @@ class PromotableStrategy(EvaluationStrategy):
             exp.select("*").from_(table_name, dialect=self.adapter.dialect),
             table_description=snapshot.model.description if is_prod else None,
             column_descriptions=snapshot.model.column_descriptions if is_prod else None,
+            view_properties=snapshot.model.virtual_properties,
         )
 
     def demote(
@@ -1086,6 +1140,7 @@ class MaterializableStrategy(PromotableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1111,7 +1166,7 @@ class MaterializableStrategy(PromotableStrategy):
                 partitioned_by=model.partitioned_by,
                 partition_interval_unit=model.interval_unit,
                 clustered_by=model.clustered_by,
-                table_properties=model.table_properties,
+                table_properties=model.physical_properties,
                 table_description=model.description if is_table_deployable else None,
                 column_descriptions=model.column_descriptions if is_table_deployable else None,
             )
@@ -1133,7 +1188,7 @@ class MaterializableStrategy(PromotableStrategy):
                 partitioned_by=model.partitioned_by,
                 partition_interval_unit=model.interval_unit,
                 clustered_by=model.clustered_by,
-                table_properties=model.table_properties,
+                table_properties=model.physical_properties,
                 table_description=model.description if is_table_deployable else None,
                 column_descriptions=model.column_descriptions if is_table_deployable else None,
             )
@@ -1144,15 +1199,38 @@ class MaterializableStrategy(PromotableStrategy):
         snapshots: t.Dict[str, Snapshot],
         target_table_name: str,
         source_table_name: str,
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         logger.info(f"Altering table '{target_table_name}'")
-        self.adapter.alter_table(target_table_name, source_table_name)
+        alter_expressions = self.adapter.get_alter_expressions(target_table_name, source_table_name)
+        _check_destructive_schema_change(snapshot, alter_expressions, allow_destructive_snapshots)
+        self.adapter.alter_table(alter_expressions)
 
     def delete(self, table_name: str) -> None:
         self.adapter.drop_table(table_name)
         logger.info("Dropped table '%s'", table_name)
 
 
+class IncrementalByPartitionStrategy(MaterializableStrategy):
+    def insert(
+        self,
+        snapshot: Snapshot,
+        name: str,
+        query_or_df: QueryOrDF,
+        snapshots: t.Dict[str, Snapshot],
+        deployability_index: DeployabilityIndex,
+        batch_index: int,
+        **kwargs: t.Any,
+    ) -> None:
+        model = snapshot.model
+        self.adapter.insert_overwrite_by_partition(
+            name,
+            query_or_df,
+            partitioned_by=model.partitioned_by,
+            columns_to_types=model.columns_to_types,
+        )
+
+
 class IncrementalByTimeRangeStrategy(MaterializableStrategy):
     def insert(
         self,
@@ -1161,6 +1239,7 @@ class IncrementalByTimeRangeStrategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1183,10 +1262,14 @@ class IncrementalByUniqueKeyStrategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
-        if not _intervals(snapshot, deployability_index):
+        # https://github.com/TobikoData/sqlmesh/issues/2609
+        # If there are no existing intervals yet; only clear the table for the first snapshot in the batch
+        # Otherwise, each subsequent snapshot in the batch will keep clearing the table when it runs instead of appending to it
+        if not _intervals(snapshot, deployability_index) and batch_index == 0:
             self._replace_query_for_model(model, name, query_or_df)
         else:
             self.adapter.merge(
@@ -1204,6 +1287,7 @@ class IncrementalByUniqueKeyStrategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: t.Optional[DeployabilityIndex],
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1224,6 +1308,7 @@ class IncrementalUnmanagedStrategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1243,6 +1328,7 @@ class IncrementalUnmanagedStrategy(MaterializableStrategy):
                 query_or_df,
                 snapshots,
                 deployability_index,
+                batch_index,
                 **kwargs,
             )
 
@@ -1255,12 +1341,47 @@ class FullRefreshStrategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
         self._replace_query_for_model(model, name, query_or_df)
 
 
+class SeedStrategy(MaterializableStrategy):
+    def create(
+        self,
+        snapshot: Snapshot,
+        name: str,
+        is_table_deployable: bool,
+        is_snapshot_deployable: bool,
+        **render_kwargs: t.Any,
+    ) -> None:
+        super().create(snapshot, name, is_table_deployable, is_snapshot_deployable, **render_kwargs)
+
+        # For seeds we insert data at the time of table creation.
+        if is_table_deployable:
+            model = t.cast(SeedModel, snapshot.model)
+            for index, df in enumerate(model.render_seed()):
+                if index == 0:
+                    self._replace_query_for_model(model, name, df)
+                else:
+                    self.adapter.insert_append(name, df, columns_to_types=model.columns_to_types)
+
+    def insert(
+        self,
+        snapshot: Snapshot,
+        name: str,
+        query_or_df: QueryOrDF,
+        snapshots: t.Dict[str, Snapshot],
+        deployability_index: DeployabilityIndex,
+        batch_index: int,
+        **kwargs: t.Any,
+    ) -> None:
+        # Data has already been inserted at the time of table creation.
+        pass
+
+
 class SCDType2Strategy(MaterializableStrategy):
     def create(
         self,
@@ -1284,7 +1405,7 @@ class SCDType2Strategy(MaterializableStrategy):
                 partitioned_by=model.partitioned_by,
                 partition_interval_unit=model.interval_unit,
                 clustered_by=model.clustered_by,
-                table_properties=model.table_properties,
+                table_properties=model.physical_properties,
                 table_description=model.description if is_table_deployable else None,
                 column_descriptions=model.column_descriptions if is_table_deployable else None,
             )
@@ -1307,6 +1428,7 @@ class SCDType2Strategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1355,6 +1477,7 @@ class SCDType2Strategy(MaterializableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1402,6 +1525,7 @@ class ViewStrategy(PromotableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         model = snapshot.model
@@ -1433,7 +1557,7 @@ class ViewStrategy(PromotableStrategy):
             query_or_df,
             model.columns_to_types,
             materialized=self._is_materialized_view(model),
-            table_properties=model.table_properties,
+            view_properties=model.physical_properties,
             table_description=model.description,
             column_descriptions=model.column_descriptions,
         )
@@ -1445,6 +1569,7 @@ class ViewStrategy(PromotableStrategy):
         query_or_df: QueryOrDF,
         snapshots: t.Dict[str, Snapshot],
         deployability_index: DeployabilityIndex,
+        batch_index: int,
         **kwargs: t.Any,
     ) -> None:
         raise ConfigError(f"Cannot append to a view '{table_name}'.")
@@ -1464,7 +1589,7 @@ class ViewStrategy(PromotableStrategy):
             name,
             model.render_query_or_raise(**render_kwargs),
             materialized=self._is_materialized_view(model),
-            table_properties=model.table_properties,
+            view_properties=model.physical_properties,
             table_description=model.description if is_table_deployable else None,
             column_descriptions=model.column_descriptions if is_table_deployable else None,
         )
@@ -1475,6 +1600,7 @@ class ViewStrategy(PromotableStrategy):
         snapshots: t.Dict[str, Snapshot],
         target_table_name: str,
         source_table_name: str,
+        allow_destructive_snapshots: t.Set[str] = set(),
     ) -> None:
         logger.info("Migrating view '%s'", target_table_name)
         model = snapshot.model
@@ -1485,7 +1611,7 @@ class ViewStrategy(PromotableStrategy):
             ),
             model.columns_to_types,
             materialized=self._is_materialized_view(model),
-            table_properties=model.table_properties,
+            view_properties=model.physical_properties,
             table_description=model.description,
             column_descriptions=model.column_descriptions,
         )
@@ -1512,3 +1638,22 @@ def _intervals(snapshot: Snapshot, deployability_index: DeployabilityIndex) -> I
         if deployability_index.is_deployable(snapshot)
         else snapshot.dev_intervals
     )
+
+
+def _check_destructive_schema_change(
+    snapshot: Snapshot,
+    alter_expressions: t.List[exp.AlterTable],
+    allow_destructive_snapshots: t.Set[str],
+) -> None:
+    if snapshot.needs_destructive_check(allow_destructive_snapshots) and has_drop_alteration(
+        alter_expressions
+    ):
+        warning_msg = (
+            f"Plan results in a destructive change to forward-only table '{snapshot.name}'s schema."
+        )
+        if snapshot.model.on_destructive_change.is_warn:
+            logger.warning(warning_msg)
+            return
+        raise SQLMeshError(
+            f"{warning_msg} To allow this, change the model's `on_destructive_change` setting to `warn` or `allow` or include it in the plan's `--allow-destructive-model` option."
+        )
