@@ -19,6 +19,7 @@ from __future__ import annotations
 import contextlib
 import json
 import logging
+import time
 import typing as t
 from collections import defaultdict
 from copy import deepcopy
@@ -29,6 +30,7 @@ from sqlglot import __version__ as SQLGLOT_VERSION
 from sqlglot import exp
 from sqlglot.helper import seq_get
 
+from sqlmesh.core import analytics
 from sqlmesh.core import constants as c
 from sqlmesh.core.audit import ModelAudit
 from sqlmesh.core.console import Console, get_console
@@ -39,7 +41,6 @@ from sqlmesh.core.snapshot import (
     Intervals,
     Node,
     Snapshot,
-    SnapshotChangeCategory,
     SnapshotFingerprint,
     SnapshotId,
     SnapshotIdLike,
@@ -96,9 +97,9 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         context_path: The context path, used for caching snapshot models.
     """
 
+    INTERVAL_BATCH_SIZE = 1000
     SNAPSHOT_BATCH_SIZE = 1000
     SNAPSHOT_MIGRATION_BATCH_SIZE = 500
-    SNAPSHOT_SEED_MIGRATION_BATCH_SIZE = 200
 
     def __init__(
         self,
@@ -114,7 +115,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         self.console = console or get_console()
         self.snapshots_table = exp.table_("_snapshots", db=self.schema)
         self.environments_table = exp.table_("_environments", db=self.schema)
-        self.seeds_table = exp.table_("_seeds", db=self.schema)
         self.intervals_table = exp.table_("_intervals", db=self.schema)
         self.plan_dags_table = exp.table_("_plan_dags", db=self.schema)
         self.versions_table = exp.table_("_versions", db=self.schema)
@@ -143,12 +143,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             "previous_finalized_snapshots": exp.DataType.build("text"),
         }
 
-        self._seed_columns_to_types = {
-            "name": exp.DataType.build("text"),
-            "version": exp.DataType.build("text"),
-            "content": exp.DataType.build("text"),
-        }
-
         self._interval_columns_to_types = {
             "id": exp.DataType.build("text"),
             "created_ts": exp.DataType.build("bigint"),
@@ -212,22 +206,11 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             snapshots = tuple(snapshots)
             self.delete_snapshots(snapshots)
 
-        seed_contents = []
         snapshots_to_store = []
 
         for snapshot in snapshots:
-            if isinstance(snapshot.node, SeedModel) and snapshot.change_category in (
-                SnapshotChangeCategory.BREAKING,
-                SnapshotChangeCategory.NON_BREAKING,
-            ):
+            if isinstance(snapshot.node, SeedModel):
                 seed_model = t.cast(SeedModel, snapshot.node)
-                seed_contents.append(
-                    {
-                        "name": snapshot.name,
-                        "version": snapshot.version,
-                        "content": seed_model.seed.content,
-                    }
-                )
                 snapshot = snapshot.copy(update={"node": seed_model.to_dehydrated()})
             snapshots_to_store.append(snapshot)
 
@@ -237,13 +220,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             columns_to_types=self._snapshot_columns_to_types,
         )
 
-        if seed_contents:
-            self.engine_adapter.insert_append(
-                self.seeds_table,
-                pd.DataFrame(seed_contents),
-                columns_to_types=self._seed_columns_to_types,
-            )
-
     def _update_versions(
         self,
         schema_version: int = SCHEMA_VERSION,
@@ -310,7 +286,7 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             )
 
         unique_expired_versions = unique(expired_candidates.values())
-        version_batches = self._snapshot_batches(unique_expired_versions)
+        version_batches = self._batches(unique_expired_versions)
         cleanup_targets = []
         for versions_batch in version_batches:
             snapshots = self._get_snapshots_with_same_version(versions_batch)
@@ -345,10 +321,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
                         )
                     )
 
-        seed_deletion_candidates = [t.snapshot for t in cleanup_targets if not t.dev_table_only]
-        if seed_deletion_candidates:
-            self._delete_seeds(seed_deletion_candidates)
-
         return cleanup_targets
 
     def delete_expired_environments(self) -> t.List[Environment]:
@@ -394,7 +366,7 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         )
         if exclude_external:
             query = query.where(exp.column("kind_name").neq(ModelKindName.EXTERNAL.value))
-        return {name for name, in self._fetchall(query)}
+        return {name for (name,) in self._fetchall(query)}
 
     def reset(self, default_catalog: t.Optional[str]) -> None:
         """Resets the state store to the state when it was first initialized."""
@@ -454,11 +426,35 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             return query.lock(copy=False)
         return query
 
+    def _get_snapshots_expressions(
+        self,
+        snapshot_ids: t.Optional[t.Iterable[SnapshotIdLike]] = None,
+        lock_for_update: bool = False,
+        batch_size: t.Optional[int] = None,
+    ) -> t.Iterator[exp.Expression]:
+        for where in (
+            [None]
+            if snapshot_ids is None
+            else self._snapshot_id_filter(snapshot_ids, alias="snapshots", batch_size=batch_size)
+        ):
+            query = (
+                exp.select(
+                    "snapshots.snapshot",
+                    "snapshots.name",
+                    "snapshots.identifier",
+                    "snapshots.version",
+                )
+                .from_(exp.to_table(self.snapshots_table).as_("snapshots"))
+                .where(where)
+            )
+            if lock_for_update:
+                query = query.lock(copy=False)
+            yield query
+
     def _get_snapshots(
         self,
         snapshot_ids: t.Optional[t.Iterable[SnapshotIdLike]] = None,
         lock_for_update: bool = False,
-        hydrate_seeds: bool = False,
         hydrate_intervals: bool = True,
     ) -> t.Dict[SnapshotId, Snapshot]:
         """Fetches specified snapshots or all snapshots.
@@ -466,7 +462,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         Args:
             snapshot_ids: The collection of snapshot like objects to fetch.
             lock_for_update: Lock the snapshot rows for future update
-            hydrate_seeds: Whether to hydrate seed snapshots with the content.
             hydrate_intervals: Whether to hydrate result snapshots with intervals.
 
         Returns:
@@ -476,40 +471,14 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         duplicates: t.Dict[SnapshotId, Snapshot] = {}
         model_cache = ModelCache(self._context_path / c.CACHE)
 
-        for where in (
-            [None] if snapshot_ids is None else self._snapshot_id_filter(snapshot_ids, "snapshots")
-        ):
-            query = (
-                exp.select(
-                    "snapshots.snapshot",
-                    "snapshots.name",
-                    "snapshots.identifier",
+        for query in self._get_snapshots_expressions(snapshot_ids, lock_for_update):
+            for serialized_snapshot, name, identifier, _ in self._fetchall(query):
+                snapshot = parse_snapshot(
+                    model_cache,
+                    serialized_snapshot=serialized_snapshot,
+                    name=name,
+                    identifier=identifier,
                 )
-                .from_(exp.to_table(self.snapshots_table).as_("snapshots"))
-                .where(where)
-            )
-            if hydrate_seeds:
-                query = query.select(exp.column("content", table="seeds")).join(
-                    exp.to_table(self.seeds_table).as_("seeds"),
-                    on=exp.and_(
-                        exp.column("name", table="snapshots").eq(exp.column("name", table="seeds")),
-                        exp.column("version", table="snapshots").eq(
-                            exp.column("version", table="seeds")
-                        ),
-                    ),
-                    join_type="left",
-                )
-            elif lock_for_update:
-                query = query.lock(copy=False)
-
-            for row in self._fetchall(query):
-                payload = json.loads(row[0])
-
-                def loader() -> Node:
-                    return parse_obj_as(Node, payload["node"])  # type: ignore
-
-                payload["node"] = model_cache.get_or_load(f"{row[1]}_{row[2]}", loader=loader)  # type: ignore
-                snapshot = Snapshot(**payload)
                 snapshot_id = snapshot.snapshot_id
                 if snapshot_id in snapshots:
                     other = duplicates.get(snapshot_id, snapshots[snapshot_id])
@@ -520,9 +489,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
                 else:
                     snapshots[snapshot_id] = snapshot
 
-                if hydrate_seeds and isinstance(snapshot.node, SeedModel) and row[-1]:
-                    snapshot.node = t.cast(SeedModel, snapshot.node).to_hydrated(row[-1])
-
         if snapshots and hydrate_intervals:
             _, intervals = self._get_snapshot_intervals(snapshots.values())
             Snapshot.hydrate_with_intervals_by_version(snapshots.values(), intervals)
@@ -621,27 +587,48 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         end: TimeLike,
         is_dev: bool = False,
     ) -> None:
-        start_ts, end_ts = snapshot.inclusive_exclusive(start, end, strict=False)
-        if start_ts >= end_ts:
-            logger.info(
-                "Skipping partial interval (%s, %s) for snapshot %s",
-                start,
-                end,
-                snapshot.snapshot_id,
-            )
-            return
+        super().add_interval(snapshot, start, end, is_dev)
 
-        logger.info(
-            "Adding %s (%s, %s) for snapshot %s",
-            "dev interval" if is_dev else "interval",
-            start_ts,
-            end_ts,
-            snapshot.snapshot_id,
+    @transactional()
+    def _add_snapshot_intervals(self, snapshot_intervals: SnapshotIntervals) -> None:
+        def remove_partial_intervals(
+            intervals: t.List[Interval], snapshot_id: SnapshotId, *, is_dev: bool
+        ) -> t.List[Interval]:
+            results = []
+            for start_ts, end_ts in intervals:
+                if start_ts < end_ts:
+                    logger.info(
+                        "Adding %s (%s, %s) for snapshot %s",
+                        "dev interval" if is_dev else "interval",
+                        start_ts,
+                        end_ts,
+                        snapshot_id,
+                    )
+                    results.append((start_ts, end_ts))
+                else:
+                    logger.info(
+                        "Skipping partial interval (%s, %s) for snapshot %s",
+                        start_ts,
+                        end_ts,
+                        snapshot_intervals.snapshot_id,
+                    )
+            return results
+
+        snapshot_intervals = snapshot_intervals.copy(
+            update={
+                "intervals": remove_partial_intervals(
+                    snapshot_intervals.intervals, snapshot_intervals.snapshot_id, is_dev=False
+                ),
+                "dev_intervals": remove_partial_intervals(
+                    snapshot_intervals.dev_intervals, snapshot_intervals.snapshot_id, is_dev=True
+                ),
+            }
         )
-
+        if not snapshot_intervals.intervals and not snapshot_intervals.dev_intervals:
+            return
         self.engine_adapter.insert_append(
             self.intervals_table,
-            _intervals_to_df([(snapshot, (start_ts, end_ts))], is_dev, False),
+            _snapshot_interval_to_df(snapshot_intervals, is_removed=False),
             columns_to_types=self._interval_columns_to_types,
         )
 
@@ -649,7 +636,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
     def remove_interval(
         self,
         snapshot_intervals: t.Sequence[t.Tuple[SnapshotInfoLike, Interval]],
-        execution_time: t.Optional[TimeLike] = None,
         remove_shared_versions: bool = False,
     ) -> None:
         intervals_to_remove: t.Sequence[
@@ -698,9 +684,12 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         self._push_snapshot_intervals(snapshot_intervals)
 
         if interval_ids:
-            self.engine_adapter.delete_from(
-                self.intervals_table, exp.column("id").isin(*interval_ids)
-            )
+            for interval_id_batch in self._batches(
+                list(interval_ids), batch_size=self.INTERVAL_BATCH_SIZE
+            ):
+                self.engine_adapter.delete_from(
+                    self.intervals_table, exp.column("id").isin(*interval_id_batch)
+                )
 
     def refresh_snapshot_intervals(self, snapshots: t.Collection[Snapshot]) -> t.List[Snapshot]:
         if not snapshots:
@@ -917,34 +906,39 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
     ) -> None:
         """Migrate the state sync to the latest SQLMesh / SQLGlot version."""
         versions = self.get_versions(validate=False)
-        migrations = MIGRATIONS[versions.schema_version :]
 
-        if (
-            not migrations
-            and major_minor(SQLGLOT_VERSION) == versions.minor_sqlglot_version
-            and major_minor(SQLMESH_VERSION) == versions.minor_sqlmesh_version
-        ):
-            return
-
-        migrate_rows = migrations or major_minor(SQLGLOT_VERSION) != versions.minor_sqlglot_version
-        if not skip_backup and migrate_rows:
-            self._backup_state()
+        migration_start_ts = time.perf_counter()
 
         try:
-            for migration in migrations:
-                logger.info(f"Applying migration {migration}")
-                migration.migrate(self, default_catalog=default_catalog)
+            migrate_rows = self._apply_migrations(default_catalog, skip_backup)
+
+            if not migrate_rows and major_minor(SQLMESH_VERSION) == versions.minor_sqlmesh_version:
+                return
+
             if migrate_rows:
                 self._migrate_rows(promoted_snapshots_only)
                 # Cleanup plan DAGs since we currently don't migrate snapshot records that are in there.
                 self.engine_adapter.delete_from(self.plan_dags_table, "TRUE")
             self._update_versions()
+
+            analytics.collector.on_migration_end(
+                from_sqlmesh_version=versions.sqlmesh_version,
+                state_sync_type=self.state_type(),
+                migration_time_sec=time.perf_counter() - migration_start_ts,
+            )
         except Exception as e:
             if skip_backup:
                 logger.error("Backup was skipped so no rollback was attempted.")
             else:
                 self.rollback()
 
+            analytics.collector.on_migration_end(
+                from_sqlmesh_version=versions.sqlmesh_version,
+                state_sync_type=self.state_type(),
+                migration_time_sec=time.perf_counter() - migration_start_ts,
+                error=e,
+            )
+
             self.console.stop_migration_progress(success=False)
             raise SQLMeshError("SQLMesh migration failed.") from e
 
@@ -955,7 +949,7 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         """Rollback to the previous migration."""
         logger.info("Starting migration rollback.")
         tables = (self.snapshots_table, self.environments_table, self.versions_table)
-        optional_tables = (self.seeds_table, self.intervals_table, self.plan_dags_table)
+        optional_tables = (self.intervals_table, self.plan_dags_table)
         versions = self.get_versions(validate=False)
         if versions.schema_version == 0:
             # Clean up state tables
@@ -975,12 +969,14 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
 
         logger.info("Migration rollback successful.")
 
+    def state_type(self) -> str:
+        return self.engine_adapter.dialect
+
     def _backup_state(self) -> None:
         for table in (
             self.snapshots_table,
             self.environments_table,
             self.versions_table,
-            self.seeds_table,
             self.intervals_table,
             self.plan_dags_table,
         ):
@@ -992,6 +988,24 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
                         backup_name, exp.select("*").from_(table), exists=False
                     )
 
+    def _apply_migrations(
+        self,
+        default_catalog: t.Optional[str],
+        skip_backup: bool,
+    ) -> bool:
+        versions = self.get_versions(validate=False)
+        migrations = MIGRATIONS[versions.schema_version :]
+
+        migrate_rows = migrations or major_minor(SQLGLOT_VERSION) != versions.minor_sqlglot_version
+        if not skip_backup and migrate_rows:
+            self._backup_state()
+
+        for migration in migrations:
+            logger.info(f"Applying migration {migration}")
+            migration.migrate(self, default_catalog=default_catalog)
+
+        return bool(migrate_rows)
+
     def _migrate_rows(self, promoted_snapshots_only: bool) -> None:
         logger.info("Fetching environments")
         environments = self.get_environments()
@@ -1187,10 +1201,6 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
             except Exception:
                 logger.warning("Failed to unpause migrated snapshots", exc_info=True)
 
-    def _delete_seeds(self, snapshots: t.Iterable[SnapshotNameVersionLike]) -> None:
-        for where in self._snapshot_name_version_filter(snapshots, alias=None):
-            self.engine_adapter.delete_from(self.seeds_table, where=where)
-
     def _snapshot_ids_exist(
         self, snapshot_ids: t.Iterable[SnapshotIdLike], table_name: exp.Table
     ) -> t.Set[SnapshotId]:
@@ -1211,7 +1221,7 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         name_identifiers = sorted(
             {(snapshot_id.name, snapshot_id.identifier) for snapshot_id in snapshot_ids}
         )
-        batches = self._snapshot_batches(name_identifiers, batch_size=batch_size)
+        batches = self._batches(name_identifiers, batch_size=batch_size)
 
         if not name_identifiers:
             yield exp.false()
@@ -1244,10 +1254,10 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
         alias: t.Optional[str] = "snapshots",
     ) -> t.Iterator[exp.Condition]:
         name_versions = sorted({(s.name, s.version) for s in snapshot_name_versions})
-        batches = self._snapshot_batches(name_versions)
+        batches = self._batches(name_versions)
 
         if not name_versions:
-            return exp.false()
+            yield exp.false()
         elif self.engine_adapter.SUPPORTS_TUPLE_IN:
             for versions in batches:
                 yield t.cast(
@@ -1271,9 +1281,7 @@ class EngineAdapterStateSync(CommonStateSyncMixin, StateSync):
                     ]
                 )
 
-    def _snapshot_batches(
-        self, l: t.List[T], batch_size: t.Optional[int] = None
-    ) -> t.List[t.List[T]]:
+    def _batches(self, l: t.List[T], batch_size: t.Optional[int] = None) -> t.List[t.List[T]]:
         batch_size = batch_size or self.SNAPSHOT_BATCH_SIZE
         return [l[i : i + batch_size] for i in range(0, len(l), batch_size)]
 
@@ -1301,6 +1309,27 @@ def _intervals_to_df(
     )
 
 
+def _snapshot_interval_to_df(
+    snapshot_intervals: SnapshotIntervals,
+    is_removed: bool = False,
+) -> pd.DataFrame:
+    return pd.DataFrame(
+        [
+            _interval_to_df(
+                snapshot_intervals,
+                start_ts,
+                end_ts,
+                is_dev=is_dev,
+                is_removed=is_removed,
+            )
+            for is_dev in (False, True)
+            for start_ts, end_ts in getattr(
+                snapshot_intervals, "dev_intervals" if is_dev else "intervals"
+            )
+        ]
+    )
+
+
 def _interval_to_df(
     snapshot: t.Union[SnapshotInfoLike, SnapshotIntervals],
     start_ts: int,
@@ -1380,6 +1409,23 @@ def _snapshot_to_json(snapshot: Snapshot) -> str:
     return snapshot.json(exclude={"intervals", "dev_intervals"})
 
 
+def parse_snapshot(
+    model_cache: ModelCache,
+    serialized_snapshot: str,
+    name: str,
+    identifier: str,
+) -> Snapshot:
+    payload = json.loads(serialized_snapshot)
+
+    def loader() -> Node:
+        return parse_obj_as(Node, payload["node"])  # type: ignore
+
+    payload["node"] = model_cache.get_or_load(f"{name}_{identifier}", loader=loader)  # type: ignore
+    snapshot = Snapshot(**payload)
+
+    return snapshot
+
+
 class LazilyParsedSnapshots:
     def __init__(self, raw_snapshots: t.Dict[SnapshotId, t.Dict[str, t.Any]]):
         self._raw_snapshots = raw_snapshots
