@@ -10,6 +10,7 @@ from unittest.mock import patch
 
 import numpy as np
 import pandas as pd
+from io import StringIO
 from freezegun import freeze_time
 from pandas.api.types import is_object_dtype
 from sqlglot import Dialect, exp
@@ -19,12 +20,17 @@ from sqlglot.optimizer.normalize_identifiers import normalize_identifiers
 from sqlmesh.core import constants as c
 from sqlmesh.core.dialect import normalize_model_name, schema_
 from sqlmesh.core.engine_adapter import EngineAdapter
+from sqlmesh.core.macros import RuntimeStage
 from sqlmesh.core.model import Model, PythonModel, SqlModel
 from sqlmesh.utils import UniqueKeyDict, random_id, type_is_known, yaml
 from sqlmesh.utils.date import pandas_timestamp_to_pydatetime
 from sqlmesh.utils.errors import ConfigError, TestError
+from sqlmesh.utils.yaml import load as yaml_load
 
-Row = t.Dict[str, t.Any]
+if t.TYPE_CHECKING:
+    from sqlglot.dialects.dialect import DialectType
+
+    Row = t.Dict[str, t.Any]
 
 
 class ModelTest(unittest.TestCase):
@@ -68,16 +74,25 @@ class ModelTest(unittest.TestCase):
         self._normalized_column_name_cache: t.Dict[str, str] = {}
         self._normalized_model_name_cache: t.Dict[t.Tuple[str, bool], str] = {}
 
+        self._test_adapter_dialect = Dialect.get_or_raise(self.engine_adapter.dialect)
+
         self._validate_and_normalize_test()
 
-        # The test schema name is randomized to avoid concurrency issues
-        self._test_schema = f"sqlmesh_test_{random_id(short=True)}"
+        if self.engine_adapter.default_catalog:
+            self._fixture_catalog: t.Optional[exp.Identifier] = exp.parse_identifier(
+                self.engine_adapter.default_catalog, dialect=self._test_adapter_dialect
+            )
+        else:
+            self._fixture_catalog = None
 
-        self._engine_adapter_dialect = Dialect.get_or_raise(self.engine_adapter.dialect)
-        self._transforms = self._engine_adapter_dialect.generator_class.TRANSFORMS
+        # The test schema name is randomized to avoid concurrency issues
+        self._fixture_schema = exp.to_identifier(f"sqlmesh_test_{random_id(short=True)}")
+        self._qualified_fixture_schema = schema_(self._fixture_schema, self._fixture_catalog)
+
+        self._transforms = self._test_adapter_dialect.generator_class.TRANSFORMS
+        self._execution_time = str(self.body.get("vars", {}).get("execution_time") or "")
 
         # When execution_time is set, we mock the CURRENT_* SQL expressions so they always return it
-        self._execution_time = str(self.body.get("vars", {}).get("execution_time") or "")
         if self._execution_time:
             exec_time = exp.Literal.string(self._execution_time)
             self._transforms = {
@@ -95,6 +110,8 @@ class ModelTest(unittest.TestCase):
 
     def setUp(self) -> None:
         """Load all input tables"""
+        self.engine_adapter.create_schema(self._qualified_fixture_schema)
+
         for name, values in self.body.get("inputs", {}).items():
             all_types_are_known = False
             known_columns_to_types: t.Dict[str, exp.DataType] = {}
@@ -109,39 +126,44 @@ class ModelTest(unittest.TestCase):
                     len(known_columns_to_types) == len(inferred_columns_to_types)
                 )
 
-            # Types specified in the test take precedence over the corresponding inferred ones
+            # Types specified in the test will override the corresponding inferred ones
             known_columns_to_types.update(values.get("columns", {}))
 
-            rows = values["rows"]
+            rows = values.get("rows")
             if not all_types_are_known and rows:
                 for col, value in rows[0].items():
                     if col not in known_columns_to_types:
                         v_type = annotate_types(exp.convert(value)).type or type(value).__name__
-                        v_type = exp.maybe_parse(v_type, into=exp.DataType, dialect=self.dialect)
+                        v_type = exp.maybe_parse(
+                            v_type, into=exp.DataType, dialect=self._test_adapter_dialect
+                        )
 
                         if not type_is_known(v_type):
                             _raise_error(
                                 f"Failed to infer the data type of column '{col}' for '{name}'. This issue can be "
                                 "mitigated by casting the column in the model definition, setting its type in "
-                                "schema.yaml if it's an external model, setting the model's 'columns' property, "
+                                "external_models.yaml if it's an external model, setting the model's 'columns' property, "
                                 "or setting its 'columns' mapping in the test itself",
                                 self.path,
                             )
 
                         known_columns_to_types[col] = v_type
 
-            test_fixture_table = self._test_fixture_table(name)
-            self.engine_adapter.create_schema(
-                schema_(test_fixture_table.args["db"], test_fixture_table.args.get("catalog"))
-            )
+            if rows is None:
+                query_or_df: exp.Query | pd.DataFrame = self._add_missing_columns(
+                    values["query"], known_columns_to_types
+                )
+            else:
+                query_or_df = self._create_df(values, columns=known_columns_to_types)
 
-            df = _create_df(rows, columns=known_columns_to_types)
-            self.engine_adapter.create_view(test_fixture_table, df, known_columns_to_types)
+            self.engine_adapter.create_view(
+                self._test_fixture_table(name), query_or_df, known_columns_to_types
+            )
 
     def tearDown(self) -> None:
         """Drop all fixture tables."""
         if not self.preserve_fixtures:
-            self.engine_adapter.drop_schema(self._test_schema, cascade=True)
+            self.engine_adapter.drop_schema(self._qualified_fixture_schema, cascade=True)
 
     def assert_equal(
         self,
@@ -175,7 +197,9 @@ class ModelTest(unittest.TestCase):
         # coming from a database query, it is safe to assume that the column contains only
         # a single type.
         object_sentinel_values = {
-            col: actual[col][0] for col in actual_types if is_object_dtype(actual_types[col])
+            col: actual[col][0]
+            for col in actual_types
+            if is_object_dtype(actual_types[col]) and len(actual[col]) != 0
         }
         for col, value in object_sentinel_values.items():
             # can't use `isinstance()` here - https://stackoverflow.com/a/68743663/1707525
@@ -297,17 +321,50 @@ class ModelTest(unittest.TestCase):
         partial = outputs.pop("partial", None)
 
         def _normalize_rows(
-            values: t.List[Row] | t.Dict, name: str, partial: bool = False
+            values: t.List[Row] | t.Dict,
+            name: str,
+            partial: bool = False,
+            dialect: DialectType = None,
         ) -> t.Dict:
             if not isinstance(values, dict):
                 values = {"rows": values}
 
-            if "rows" not in values:
+            rows = values.get("rows")
+            query = values.get("query")
+
+            fmt = values.get("format")
+            path = values.get("path")
+            if fmt == "csv":
+                csv_settings = values.get("csv_settings") or {}
+                rows = pd.read_csv(path or StringIO(rows), **csv_settings).to_dict(orient="records")
+            elif fmt in (None, "yaml"):
+                if path:
+                    input_rows = yaml_load(Path(path))
+                    rows = input_rows.get("rows") if isinstance(input_rows, dict) else input_rows
+            else:
+                _raise_error(f"Unsupported data format '{fmt}' for '{name}'", self.path)
+
+            if query is not None:
+                if rows is not None:
+                    _raise_error(
+                        f"Invalid test, cannot set both 'query' and 'rows' for '{name}'", self.path
+                    )
+
+                # We parse the user-supplied query using the testing adapter dialect, but we
+                # normalize its identifiers according to the model's dialect, so that, e.g.,
+                # the projection names match those in its `columns_to_types` field
+                values["query"] = normalize_identifiers(
+                    exp.maybe_parse(query, dialect=self._test_adapter_dialect), dialect=dialect
+                )
+                return values
+
+            if rows is None:
                 _raise_error(f"Incomplete test, missing row data for '{name}'", self.path)
 
+            assert isinstance(rows, list)
             values["rows"] = [
                 {self._normalize_column_name(column): value for column, value in row.items()}
-                for row in values["rows"]
+                for row in rows
             ]
             if partial:
                 values["partial"] = True
@@ -317,12 +374,19 @@ class ModelTest(unittest.TestCase):
         def _normalize_sources(
             sources: t.Dict, partial: bool = False, with_default_catalog: bool = True
         ) -> t.Dict:
-            return {
-                self._normalize_model_name(
+            normalized_sources = {}
+            for name, values in sources.items():
+                normalized_name = self._normalize_model_name(
                     name, with_default_catalog=with_default_catalog
-                ): _normalize_rows(values, name, partial=partial)
-                for name, values in sources.items()
-            }
+                )
+                model = self.models.get(normalized_name)
+                dialect = model.dialect if model else self.dialect
+
+                normalized_sources[normalized_name] = _normalize_rows(
+                    values, name, partial=partial, dialect=dialect
+                )
+
+            return normalized_sources
 
         normalized_model_name = self._normalize_model_name(self.body["model"])
         self.body["model"] = normalized_model_name
@@ -341,7 +405,9 @@ class ModelTest(unittest.TestCase):
                     )
 
                 values["columns"] = {
-                    self._normalize_column_name(c): exp.DataType.build(t, dialect=self.dialect)
+                    self._normalize_column_name(c): exp.DataType.build(
+                        t, dialect=self._test_adapter_dialect
+                    )
                     for c, t in columns.items()
                 }
 
@@ -358,17 +424,21 @@ class ModelTest(unittest.TestCase):
             outputs["ctes"] = _normalize_sources(ctes, partial=partial, with_default_catalog=False)
 
         if query or query == []:
-            outputs["query"] = _normalize_rows(query, self.model.name, partial=partial)
+            outputs["query"] = _normalize_rows(
+                query, self.model.name, partial=partial, dialect=self.model.dialect
+            )
 
     def _test_fixture_table(self, name: str) -> exp.Table:
         table = self._fixture_table_cache.get(name)
         if not table:
-            table = exp.to_table(name, dialect=self.dialect)
+            table = exp.to_table(name, dialect=self._test_adapter_dialect)
 
-            # We change both the schema and the catalog, so we need to ensure there are no name clashes
+            # We change the table path below, so this ensures there are no name clashes
             table.this.set("this", "__".join(part.name for part in table.parts))
-            table.set("db", exp.to_identifier(self._test_schema))
-            table.set("catalog", exp.to_identifier(self.default_catalog))
+
+            table.set("db", self._fixture_schema.copy())
+            if self._fixture_catalog:
+                table.set("catalog", self._fixture_catalog.copy())
 
             self._fixture_table_cache[name] = table
 
@@ -393,6 +463,44 @@ class ModelTest(unittest.TestCase):
 
         return normalized_name
 
+    def _execute(self, query: exp.Query) -> pd.DataFrame:
+        """Executes the given query using the testing engine adapter and returns a DataFrame."""
+        with patch.dict(self._test_adapter_dialect.generator_class.TRANSFORMS, self._transforms):
+            return self.engine_adapter.fetchdf(query)
+
+    def _create_df(
+        self,
+        values: t.Dict[str, t.Any],
+        columns: t.Optional[t.Collection] = None,
+        partial: t.Optional[bool] = False,
+    ) -> pd.DataFrame:
+        query = values.get("query")
+        if query:
+            return self._execute(self._add_missing_columns(query, columns))
+
+        rows = values["rows"]
+        if columns:
+            referenced_columns = list(dict.fromkeys(col for row in rows for col in row))
+            _raise_if_unexpected_columns(columns, referenced_columns)
+
+            if partial:
+                columns = referenced_columns
+
+        return pd.DataFrame.from_records(rows, columns=columns)
+
+    def _add_missing_columns(
+        self, query: exp.Query, all_columns: t.Optional[t.Collection[str]] = None
+    ) -> exp.Query:
+        if not all_columns or query.is_star:
+            return query
+
+        query_columns = set(query.named_selects)
+        missing_columns = [col for col in all_columns if col not in query_columns]
+        if missing_columns:
+            query.select(*[exp.null().as_(col) for col in missing_columns], copy=False)
+
+        return query
+
 
 class SqlModelTest(ModelTest):
     def test_ctes(self, ctes: t.Dict[str, exp.Expression]) -> None:
@@ -408,29 +516,16 @@ class SqlModelTest(ModelTest):
                 for alias, cte in ctes.items():
                     cte_query = cte_query.with_(alias, cte.this)
 
-                rows = values["rows"]
                 partial = values.get("partial")
                 sort = cte_query.args.get("order") is None
 
                 actual = self._execute(cte_query)
-                expected = _create_df(rows, columns=cte_query.named_selects, partial=partial)
+                expected = self._create_df(values, columns=cte_query.named_selects, partial=partial)
 
                 self.assert_equal(expected, actual, sort=sort, partial=partial)
 
     def runTest(self) -> None:
-        # For tests we just use the model name for the table reference and we don't want to expand
-        mapping = {
-            name: self._test_fixture_table(name).sql()
-            for name in [
-                self._normalize_model_name(name)
-                for name in self.models.keys() | self.body.get("inputs", {}).keys()
-            ]
-        }
-        query = self.model.render_query_or_raise(
-            **self.body.get("vars", {}),
-            engine_adapter=self.engine_adapter,
-            table_mapping=mapping,
-        )
+        query = self._render_model_query()
 
         self.test_ctes(
             {
@@ -441,19 +536,23 @@ class SqlModelTest(ModelTest):
 
         values = self.body["outputs"].get("query")
         if values is not None:
-            rows = values["rows"]
             partial = values.get("partial")
             sort = query.args.get("order") is None
 
             actual = self._execute(query)
-            expected = _create_df(rows, columns=self.model.columns_to_types, partial=partial)
+            expected = self._create_df(values, columns=self.model.columns_to_types, partial=partial)
 
             self.assert_equal(expected, actual, sort=sort, partial=partial)
 
-    def _execute(self, query: exp.Expression) -> pd.DataFrame:
-        """Executes the query with the engine adapter and returns a DataFrame."""
-        with patch.dict(self._engine_adapter_dialect.generator_class.TRANSFORMS, self._transforms):
-            return self.engine_adapter.fetchdf(query)
+    def _render_model_query(self) -> exp.Query:
+        return self.model.render_query_or_raise(
+            **self.body.get("vars", {}),
+            engine_adapter=self.engine_adapter,
+            table_mapping={
+                name: self._test_fixture_table(name).sql() for name in self.body.get("inputs", {})
+            },
+            runtime_stage=RuntimeStage.TESTING,
+        )
 
 
 class PythonModelTest(ModelTest):
@@ -506,24 +605,22 @@ class PythonModelTest(ModelTest):
     def runTest(self) -> None:
         values = self.body["outputs"].get("query")
         if values is not None:
-            rows = values["rows"]
             partial = values.get("partial")
 
             actual_df = self._execute_model()
             actual_df.reset_index(drop=True, inplace=True)
-            expected = _create_df(rows, columns=self.model.columns_to_types, partial=partial)
+            expected = self._create_df(values, columns=self.model.columns_to_types, partial=partial)
 
             self.assert_equal(expected, actual_df, sort=False, partial=partial)
 
     def _execute_model(self) -> pd.DataFrame:
         """Executes the python model and returns a DataFrame."""
         time_ctx = freeze_time(self._execution_time) if self._execution_time else nullcontext()
-        with patch.dict(self._engine_adapter_dialect.generator_class.TRANSFORMS, self._transforms):
+        with patch.dict(self._test_adapter_dialect.generator_class.TRANSFORMS, self._transforms):
             with t.cast(AbstractContextManager, time_ctx):
-                return t.cast(
-                    pd.DataFrame,
-                    next(self.model.render(context=self.context, **self.body.get("vars", {}))),
-                )
+                df = next(self.model.render(context=self.context, **self.body.get("vars", {})))
+                assert not isinstance(df, exp.Expression)
+                return df if isinstance(df, pd.DataFrame) else df.toPandas()
 
 
 def generate_test(
@@ -574,8 +671,7 @@ def generate_test(
     # ruamel.yaml does not support pandas Timestamps, so we must convert them to python
     # datetime or datetime.date objects based on column type
     inputs = {
-        models[dep]
-        .name: pandas_timestamp_to_pydatetime(
+        models[dep].name: pandas_timestamp_to_pydatetime(
             engine_adapter.fetchdf(query).apply(lambda col: col.map(_normalize_df_value)),
             models[dep].columns_to_types,
         )
@@ -603,14 +699,8 @@ def generate_test(
     test.setUp()
 
     if isinstance(model, SqlModel):
-        mapping = {
-            name: test._test_fixture_table(name).sql() for name in models.keys() | inputs.keys()
-        }
-        model_query = model.render_query_or_raise(
-            **t.cast(t.Dict[str, t.Any], variables),
-            engine_adapter=test_engine_adapter,
-            table_mapping=mapping,
-        )
+        assert isinstance(test, SqlModelTest)
+        model_query = test._render_model_query()
 
         if include_ctes:
             ctes = {}
@@ -620,7 +710,7 @@ def generate_test(
                 for prev in previous_ctes:
                     cte_query = cte_query.with_(prev.alias, prev.this)
 
-                cte_output = t.cast(SqlModelTest, test)._execute(cte_query)
+                cte_output = test._execute(cte_query)
                 ctes[cte.alias] = (
                     pandas_timestamp_to_pydatetime(
                         cte_output.apply(lambda col: col.map(_normalize_df_value)),
@@ -635,7 +725,7 @@ def generate_test(
             if ctes:
                 outputs["ctes"] = ctes
 
-        output = t.cast(SqlModelTest, test)._execute(model_query)
+        output = test._execute(model_query)
     else:
         output = t.cast(PythonModelTest, test)._execute_model()
 
@@ -654,19 +744,6 @@ def generate_test(
         yaml.dump({test_name: test_body}, file)
 
 
-def _create_df(
-    rows: t.List[Row], columns: t.Optional[t.Collection] = None, partial: t.Optional[bool] = False
-) -> pd.DataFrame:
-    if columns:
-        referenced_columns = list(dict.fromkeys(col for row in rows for col in row))
-        _raise_if_unexpected_columns(columns, referenced_columns)
-
-        if partial:
-            columns = referenced_columns
-
-    return pd.DataFrame.from_records(rows, columns=columns)
-
-
 def _raise_if_unexpected_columns(
     expected_cols: t.Collection[str], actual_cols: t.Collection[str]
 ) -> None:
