@@ -5,7 +5,6 @@ from abc import abstractmethod
 from enum import Enum
 from pathlib import Path
 
-from dbt.contracts.relation import Policy, RelationType
 from pydantic import Field
 from sqlglot.helper import ensure_list
 
@@ -24,6 +23,7 @@ from sqlmesh.dbt.common import (
     SqlStr,
     sql_str_validator,
 )
+from sqlmesh.dbt.relation import Policy, RelationType
 from sqlmesh.dbt.test import TestConfig
 from sqlmesh.utils import AttributeDict
 from sqlmesh.utils.conversions import ensure_bool
@@ -297,7 +297,7 @@ class BaseModelConfig(GeneralConfig):
         return {
             "audits": [(test.name, {}) for test in self.tests],
             "columns": column_types_to_sqlmesh(self.columns, context.dialect) or None,
-            "column_descriptions_": column_descriptions_to_sqlmesh(self.columns) or None,
+            "column_descriptions": column_descriptions_to_sqlmesh(self.columns) or None,
             "depends_on": {
                 model.canonical_name(context) for model in model_context.refs.values()
             }.union({source.canonical_name(context) for source in model_context.sources.values()}),
