@@ -125,9 +125,9 @@ class ManifestHelper:
                 **_config(source),
                 **source.to_dict(),
             )
-            self._sources_per_package[source.package_name][
-                source_config.config_name
-            ] = source_config
+            self._sources_per_package[source.package_name][source_config.config_name] = (
+                source_config
+            )
 
     def _load_macros(self) -> None:
         for macro in self._manifest.macros.values():
@@ -256,6 +256,7 @@ class ManifestHelper:
             profiles_dir=str(self.profiles_path),
             target=self.target.name,
             macro_debugging=False,
+            REQUIRE_RESOURCE_NAMES_WITHOUT_SPACES=True,
         )
         flags.set_from_args(args, None)
 
@@ -271,17 +272,25 @@ class ManifestHelper:
 
         self._project_name = project.project_name
 
-        register_adapter(runtime_config)
+        if DBT_VERSION >= (1, 8):
+            from dbt.mp_context import get_mp_context  # type: ignore
+            from dbt_common.context import set_invocation_context  # type: ignore
+
+            register_adapter(runtime_config, get_mp_context())  # type: ignore
+            set_invocation_context({})
+        else:
+            register_adapter(runtime_config)  # type: ignore
+
         manifest = ManifestLoader.get_full_manifest(runtime_config)
         reset_adapters()
         return manifest
 
     def _load_project(self, profile: Profile) -> Project:
-        project_renderer = DbtProjectYamlRenderer(profile)
+        project_renderer = DbtProjectYamlRenderer(profile, cli_vars=self.variable_overrides)
         return Project.from_project_root(str(self.project_path), project_renderer)
 
     def _load_profile(self) -> Profile:
-        profile_renderer = ProfileRenderer({})
+        profile_renderer = ProfileRenderer(cli_vars=self.variable_overrides)
         raw_profiles = read_profile(str(self.profiles_path))
         return Profile.from_raw_profiles(
             raw_profiles=raw_profiles,
@@ -370,16 +379,19 @@ class ManifestHelper:
                     source = ".".join(args)
                     if not self._is_disabled_source(source):
                         dependencies.sources.add(source)
+                dependencies.macros.append(MacroReference(name="source"))
             elif call_name[0] == "ref":
                 args = [jinja_call_arg_name(arg) for arg in node.args]
                 if args and all(arg for arg in args):
                     ref = ".".join(args)
                     if not self._is_disabled_ref(ref):
                         dependencies.refs.add(ref)
+                dependencies.macros.append(MacroReference(name="ref"))
             elif call_name[0] == "var":
                 args = [jinja_call_arg_name(arg) for arg in node.args]
                 if args and args[0]:
                     dependencies.variables.add(args[0])
+                dependencies.macros.append(MacroReference(name="var"))
             elif len(call_name) == 1:
                 macro_name = call_name[0]
                 if macro_name in BUILTIN_CALLS:
@@ -471,8 +483,8 @@ def _node_base_config(node: ManifestNode) -> t.Dict[str, t.Any]:
 
 
 def _convert_jinja_test_to_macro(test_jinja: str) -> str:
-    TEST_TAG_REGEX = "\s*{%\s*test\s+"
-    ENDTEST_REGEX = "{%\s*endtest\s*%}"
+    TEST_TAG_REGEX = r"\s*{%\s*test\s+"
+    ENDTEST_REGEX = r"{%\s*endtest\s*%}"
     match = re.match(TEST_TAG_REGEX, test_jinja)
     if not match:
         # already a macro
