@@ -4,6 +4,7 @@ import logging
 import typing as t
 
 from sqlglot import exp
+from sqlglot.errors import SqlglotError
 from sqlglot.helper import ensure_list
 
 from sqlmesh.core import dialect as d
@@ -20,7 +21,7 @@ from sqlmesh.core.model import (
     ViewKind,
     create_sql_model,
 )
-from sqlmesh.core.model.kind import SCDType2ByTimeKind
+from sqlmesh.core.model.kind import SCDType2ByTimeKind, OnDestructiveChange
 from sqlmesh.dbt.basemodel import BaseModelConfig, Materialization, SnapshotStrategy
 from sqlmesh.dbt.common import SqlStr, extract_jinja_config, sql_str_validator
 from sqlmesh.utils.errors import ConfigError
@@ -85,6 +86,7 @@ class ModelConfig(BaseModelConfig):
     unique_key: t.Optional[t.List[str]] = None
     partition_by: t.Optional[t.Union[t.List[str], t.Dict[str, t.Any]]] = None
     full_refresh: t.Optional[bool] = None
+    on_schema_change: t.Optional[str] = None
 
     # Snapshot (SCD Type 2) Fields
     updated_at: t.Optional[str] = None
@@ -180,16 +182,30 @@ class ModelConfig(BaseModelConfig):
         """
         target = context.target
         materialization = self.model_materialization
+
+        # args common to all sqlmesh incremental kinds, regardless of materialization
+        incremental_kind_kwargs: t.Dict[str, t.Any] = {}
+        if self.on_schema_change:
+            on_schema_change = self.on_schema_change.lower()
+
+            on_destructive_change = OnDestructiveChange.WARN
+            if on_schema_change == "sync_all_columns":
+                on_destructive_change = OnDestructiveChange.ALLOW
+            elif on_schema_change == "fail":
+                on_destructive_change = OnDestructiveChange.ERROR
+
+            incremental_kind_kwargs["on_destructive_change"] = on_destructive_change
+
         if materialization == Materialization.TABLE:
             return FullKind()
         if materialization == Materialization.VIEW:
             return ViewKind()
         if materialization == Materialization.INCREMENTAL:
-            incremental_kwargs: t.Dict[str, t.Any] = {"dialect": context.dialect}
+            incremental_materialization_kwargs: t.Dict[str, t.Any] = {"dialect": context.dialect}
             for field in ("batch_size", "lookback", "forward_only"):
                 field_val = getattr(self, field, None) or self.meta.get(field, None)
                 if field_val:
-                    incremental_kwargs[field] = field_val
+                    incremental_materialization_kwargs[field] = field_val
 
             if self.time_column:
                 strategy = self.incremental_strategy or target.default_incremental_strategy(
@@ -209,7 +225,8 @@ class ModelConfig(BaseModelConfig):
                     disable_restatement=(
                         self.disable_restatement if self.disable_restatement is not None else False
                     ),
-                    **incremental_kwargs,
+                    **incremental_kind_kwargs,
+                    **incremental_materialization_kwargs,
                 )
 
             disable_restatement = self.disable_restatement
@@ -233,7 +250,8 @@ class ModelConfig(BaseModelConfig):
                 return IncrementalByUniqueKeyKind(
                     unique_key=self.unique_key,
                     disable_restatement=disable_restatement,
-                    **incremental_kwargs,
+                    **incremental_kind_kwargs,
+                    **incremental_materialization_kwargs,
                 )
 
             logger.warning(
@@ -247,8 +265,9 @@ class ModelConfig(BaseModelConfig):
             )
             return IncrementalUnmanagedKind(
                 insert_overwrite=strategy in INCREMENTAL_BY_TIME_STRATEGIES,
-                forward_only=incremental_kwargs.get("forward_only", True),
+                forward_only=incremental_materialization_kwargs.get("forward_only", True),
                 disable_restatement=disable_restatement,
+                **incremental_kind_kwargs,
             )
         if materialization == Materialization.EPHEMERAL:
             return EmbeddedKind()
@@ -268,6 +287,7 @@ class ModelConfig(BaseModelConfig):
                     if target.dialect == "bigquery"
                     else exp.DataType.build("TIMESTAMP")
                 ),
+                **incremental_kind_kwargs,
             }
             if self.snapshot_strategy.is_check:
                 return SCDType2ByColumnKind(
@@ -300,7 +320,12 @@ class ModelConfig(BaseModelConfig):
     def _big_query_partition_by_expr(self, context: DbtContext) -> exp.Expression:
         assert isinstance(self.partition_by, dict)
         data_type = self.partition_by["data_type"].lower()
-        field = d.parse_one(self.partition_by["field"], dialect="bigquery")
+        raw_field = self.partition_by["field"]
+        try:
+            field = d.parse_one(raw_field, dialect="bigquery")
+        except SqlglotError as e:
+            raise ConfigError(f"Failed to parse partition_by field '{raw_field}': {e}") from e
+
         if data_type == "date" and self.partition_by["granularity"].lower() == "day":
             return field
 
@@ -346,9 +371,13 @@ class ModelConfig(BaseModelConfig):
             )
 
         if self.cluster_by:
-            optional_kwargs["clustered_by"] = [
-                d.parse_one(c, dialect=dialect).name for c in self.cluster_by
-            ]
+            clustered_by = []
+            for c in self.cluster_by:
+                try:
+                    clustered_by.append(d.parse_one(c, dialect=dialect).name)
+                except SqlglotError as e:
+                    raise ConfigError(f"Failed to parse cluster_by field '{c}': {e}") from e
+            optional_kwargs["clustered_by"] = clustered_by
 
         model_kwargs = self.sqlmesh_model_kwargs(context)
         if self.sql_header:
@@ -359,14 +388,14 @@ class ModelConfig(BaseModelConfig):
             if dbt_max_partition_blob:
                 model_kwargs["pre_statements"].append(d.jinja_statement(dbt_max_partition_blob))
 
-            table_properties = {}
+            physical_properties = {}
             if self.partition_expiration_days is not None:
-                table_properties["partition_expiration_days"] = self.partition_expiration_days
+                physical_properties["partition_expiration_days"] = self.partition_expiration_days
             if self.require_partition_filter is not None:
-                table_properties["require_partition_filter"] = self.require_partition_filter
+                physical_properties["require_partition_filter"] = self.require_partition_filter
 
-            if table_properties:
-                model_kwargs["table_properties"] = table_properties
+            if physical_properties:
+                model_kwargs["physical_properties"] = physical_properties
 
         if context.target.dialect == "snowflake" and self.snowflake_warehouse is not None:
             model_kwargs["session_properties"] = {"warehouse": self.snowflake_warehouse}
