@@ -10,7 +10,6 @@ from sqlalchemy.orm import Session
 
 from sqlmesh.core.engine_adapter import create_engine_adapter
 from sqlmesh.core.environment import EnvironmentNamingInfo
-from sqlmesh.core.model import SeedModel
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     Snapshot,
@@ -116,6 +115,7 @@ class SnapshotEvaluationTarget(BaseTarget[commands.EvaluateCommandPayload], Pyda
         end: The end of the interval to evaluate.
         execution_time: The date/time time reference to use for execution time. Defaults to now.
         deployability_index: Determines snapshots that are deployable in the context of this evaluation.
+        batch_index: For snapshots that are part of a batch, this is their position in the batch
     """
 
     command_type: commands.CommandType = commands.CommandType.EVALUATE
@@ -131,6 +131,7 @@ class SnapshotEvaluationTarget(BaseTarget[commands.EvaluateCommandPayload], Pyda
     execution_time: t.Optional[TimeLike] = None
     deployability_index: DeployabilityIndex
     plan_id: t.Optional[str] = None
+    batch_index: int = 0
 
     def post_hook(
         self,
@@ -146,27 +147,21 @@ class SnapshotEvaluationTarget(BaseTarget[commands.EvaluateCommandPayload], Pyda
             )
 
     def _get_command_payload(self, context: Context) -> t.Optional[commands.EvaluateCommandPayload]:
-        snapshot = self.snapshot
-        if isinstance(snapshot.node, SeedModel) and not snapshot.node.is_hydrated:
-            with util.scoped_state_sync() as state_sync:
-                snapshot = state_sync.get_snapshots([snapshot], hydrate_seeds=True)[
-                    snapshot.snapshot_id
-                ]
-
         return commands.EvaluateCommandPayload(
-            snapshot=snapshot,
+            snapshot=self.snapshot,
             parent_snapshots=self.parent_snapshots,
             start=self._get_start(context),
             end=self._get_end(context),
             execution_time=self._get_execution_time(context),
             deployability_index=self.deployability_index,
+            batch_index=self.batch_index,
         )
 
     def _get_start(self, context: Context) -> TimeLike:
         if self.start:
             return self.start
 
-        start = self.snapshot.node.interval_unit.cron_floor(context["dag_run"].data_interval_start)
+        start = self.snapshot.node.interval_unit.cron_floor(context["dag_run"].data_interval_start)  # type: ignore
         if not self.snapshot.is_model:
             return start
 
@@ -174,7 +169,7 @@ class SnapshotEvaluationTarget(BaseTarget[commands.EvaluateCommandPayload], Pyda
 
     def _get_end(self, context: Context) -> TimeLike:
         return self.end or self.snapshot.node.interval_unit.cron_floor(
-            context["dag_run"].data_interval_end
+            context["dag_run"].data_interval_end  # type: ignore
         )
 
     def _get_execution_time(self, context: Context) -> TimeLike:
@@ -282,6 +277,7 @@ class SnapshotCreateTablesTarget(BaseTarget[commands.CreateTablesCommandPayload]
     new_snapshots: t.List[Snapshot]
     ddl_concurrent_tasks: int
     deployability_index: DeployabilityIndex
+    allow_destructive_snapshots: t.Set[str]
 
     def _get_command_payload(
         self, context: Context
@@ -293,6 +289,7 @@ class SnapshotCreateTablesTarget(BaseTarget[commands.CreateTablesCommandPayload]
             target_snapshot_ids=[s.snapshot_id for s in self.new_snapshots],
             snapshots=_get_snapshots_with_parents(self.new_snapshots),
             deployability_index=self.deployability_index,
+            allow_destructive_snapshots=self.allow_destructive_snapshots,
         )
 
 
@@ -308,6 +305,7 @@ class SnapshotMigrateTablesTarget(BaseTarget[commands.MigrateTablesCommandPayloa
 
     snapshots: t.List[Snapshot]
     ddl_concurrent_tasks: int
+    allow_destructive_snapshots: t.Set[str]
 
     def _get_command_payload(
         self, context: Context
@@ -318,6 +316,7 @@ class SnapshotMigrateTablesTarget(BaseTarget[commands.MigrateTablesCommandPayloa
         return commands.MigrateTablesCommandPayload(
             target_snapshot_ids=[s.snapshot_id for s in self.snapshots],
             snapshots=_get_snapshots_with_parents(self.snapshots),
+            allow_destructive_snapshots=self.allow_destructive_snapshots,
         )
 
 
