@@ -130,7 +130,7 @@ def create_plan_dag_spec(
             [s for s in all_snapshots.values() if request.is_selected_for_backfill(s.name)],
             start=request.environment.start_at,
             end=end,
-            execution_time=now(),
+            execution_time=request.execution_time or now(),
             deployability_index=deployability_index_for_evaluation,
             restatements=restatements,
             end_bounded=request.end_bounded,
@@ -154,7 +154,9 @@ def create_plan_dag_spec(
             if deployability_index_for_creation.is_representative(s)
         }
         if request.no_gaps and not request.is_dev
-        else None if request.no_gaps else set()
+        else None
+        if request.no_gaps
+        else set()
     )
 
     return common.PlanDagSpec(
@@ -170,6 +172,7 @@ def create_plan_dag_spec(
         ddl_concurrent_tasks=request.ddl_concurrent_tasks,
         users=request.users,
         is_dev=request.is_dev,
+        allow_destructive_snapshots=request.allow_destructive_snapshots,
         forward_only=request.forward_only,
         dag_start_ts=to_timestamp(now_dt),
         deployability_index=deployability_index_for_evaluation,
@@ -180,6 +183,7 @@ def create_plan_dag_spec(
         directly_modified_snapshots=request.directly_modified_snapshots,
         indirectly_modified_snapshots=request.indirectly_modified_snapshots,
         removed_snapshots=request.removed_snapshots,
+        execution_time=request.execution_time,
     )
 
 
