@@ -16,6 +16,7 @@ from pathlib import Path
 
 from astor import to_source
 
+from sqlmesh.core import constants as c
 from sqlmesh.utils import format_exception, unique
 from sqlmesh.utils.errors import SQLMeshError
 from sqlmesh.utils.pydantic import PydanticModel
@@ -215,8 +216,6 @@ def normalize_source(obj: t.Any) -> str:
             # remove function return type annotation
             if isinstance(node, ast.FunctionDef):
                 node.returns = None
-        elif isinstance(node, ast.arg):
-            node.annotation = None
 
     return to_source(root_node).strip()
 
@@ -278,7 +277,7 @@ def build_env(
     if name not in env:
         # We only need to add the undecorated code of @macro() functions in env, which
         # is accessible through the `__wrapped__` attribute added by functools.wraps
-        env[name] = obj.__wrapped__ if getattr(obj, "__sqlmesh_macro__", None) else obj
+        env[name] = obj.__wrapped__ if hasattr(obj, c.SQLMESH_MACRO) else obj
 
         if (
             obj_module
@@ -348,7 +347,10 @@ def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable
         if callable(v):
             name = v.__name__
             name = k if name == "<lambda>" else name
-            file_path = Path(inspect.getfile(v))
+
+            # We can't call getfile on built-in callables
+            # https://docs.python.org/3/library/inspect.html#inspect.getfile
+            file_path = Path(inspect.getfile(v)) if not inspect.isbuiltin(v) else None
 
             if _is_relative_to(file_path, path):
                 serialized[k] = Executable(
@@ -356,7 +358,7 @@ def serialize_env(env: t.Dict[str, t.Any], path: Path) -> t.Dict[str, Executable
                     payload=normalize_source(v),
                     kind=ExecutableKind.DEFINITION,
                     # Do `as_posix` to serialize windows path back to POSIX
-                    path=file_path.relative_to(path.absolute()).as_posix(),
+                    path=t.cast(Path, file_path).relative_to(path.absolute()).as_posix(),
                     alias=k if name != k else None,
                 )
             else:
