@@ -75,6 +75,21 @@ def parse_obj_as(type_: T, obj: t.Any) -> T:
     return pydantic.tools.parse_obj_as(type_, obj)  # type: ignore
 
 
+def get_dialect(values: t.Any) -> str:
+    """Extracts dialect from a dict or pydantic obj, defaulting to the globally set dialect.
+
+    Python models allow users to instantiate pydantic models by hand. This is problematic
+    because the validators kick in with the SQLGLot dialect. To instantiate Pydantic Models used
+    in python models using the project default dialect, we set a class variable on the model
+    registry and use that here.
+    """
+
+    from sqlmesh.core.model import model
+
+    dialect = (values if isinstance(values, dict) else values.data).get("dialect")
+    return model._dialect if dialect is None else dialect
+
+
 def _expression_encoder(e: exp.Expression) -> str:
     return e.meta.get("sql") or e.sql(dialect=e.meta.get("dialect"))
 
@@ -124,6 +139,11 @@ class PydanticModel(pydantic.BaseModel):
         if include is None and self.__config__.extra != "allow":  # type: ignore
             # Workaround to support @cached_property in Pydantic v1.
             include = {f.name for f in self.all_field_infos().values()}  # type: ignore
+
+        mode = kwargs.pop("mode", None)
+        if mode == "json":
+            # Pydantic v1 doesn't support the 'json' mode for dict().
+            return json.loads(super().json(include=include, **kwargs))
         return super().dict(include=include, **kwargs)  # type: ignore
 
     def json(
@@ -162,7 +182,9 @@ class PydanticModel(pydantic.BaseModel):
     @classmethod
     def parse_raw(cls: t.Type["Model"], b: t.Union[str, bytes], **kwargs: t.Any) -> "Model":
         return (
-            super().model_validate_json(b, **kwargs) if PYDANTIC_MAJOR_VERSION >= 2 else super().parse_raw(b, **kwargs)  # type: ignore
+            super().model_validate_json(b, **kwargs)  # type: ignore
+            if PYDANTIC_MAJOR_VERSION >= 2
+            else super().parse_raw(b, **kwargs)
         )
 
     @classmethod
@@ -185,7 +207,9 @@ class PydanticModel(pydantic.BaseModel):
 
     @classmethod
     def required_fields(cls: t.Type["PydanticModel"]) -> t.Set[str]:
-        return cls._fields(lambda field: field.is_required() if PYDANTIC_MAJOR_VERSION >= 2 else field.required)  # type: ignore
+        return cls._fields(
+            lambda field: field.is_required() if PYDANTIC_MAJOR_VERSION >= 2 else field.required
+        )  # type: ignore
 
     @classmethod
     def _fields(
@@ -294,8 +318,7 @@ def _get_field(
     v: t.Any,
     values: t.Any,
 ) -> exp.Expression:
-    values = values if isinstance(values, dict) else values.data
-    dialect = values.get("dialect")
+    dialect = get_dialect(values)
 
     if isinstance(v, exp.Expression):
         expression = v
@@ -315,8 +338,7 @@ def _get_fields(
     v: t.Any,
     values: t.Any,
 ) -> t.List[exp.Expression]:
-    values = values if isinstance(values, dict) else values.data
-    dialect = values.get("dialect")
+    dialect = get_dialect(values)
 
     if isinstance(v, (exp.Tuple, exp.Array)):
         expressions: t.List[exp.Expression] = v.expressions
