@@ -23,9 +23,16 @@ from sqlmesh.core.context import Context
 from sqlmesh.core.engine_adapter import SparkEngineAdapter
 from sqlmesh.core.engine_adapter.base import EngineAdapter
 from sqlmesh.core.macros import macro
-from sqlmesh.core.model import model
+from sqlmesh.core.model import IncrementalByTimeRangeKind, SqlModel, model
+from sqlmesh.core.model.kind import OnDestructiveChange
 from sqlmesh.core.plan import BuiltInPlanEvaluator, Plan
-from sqlmesh.core.snapshot import Node, Snapshot
+from sqlmesh.core.snapshot import (
+    Node,
+    Snapshot,
+    SnapshotChangeCategory,
+    SnapshotDataVersion,
+    SnapshotFingerprint,
+)
 from sqlmesh.utils import random_id
 from sqlmesh.utils.date import TimeLike, to_date
 
@@ -217,7 +224,7 @@ def sushi_context_pre_scheduling(init_and_plan_context: t.Callable) -> Context:
 def sushi_context_fixed_date(init_and_plan_context: t.Callable) -> Context:
     context, plan = init_and_plan_context("examples/sushi")
 
-    for model in context.models.values():
+    for model in context.models.values():  # noqa: F402
         if model.start:
             context.upsert_model(model.name, start="2022-01-01")
 
@@ -318,6 +325,51 @@ def make_snapshot() -> t.Callable:
     return _make_function
 
 
+@pytest.fixture
+def make_snapshot_on_destructive_change(make_snapshot: t.Callable) -> t.Callable:
+    def _make_function(
+        name: str = "a",
+        old_query: str = "select '1' as one, '2022-01-01' ds",
+        new_query: str = "select 1 as one, '2022-01-01' ds",
+        on_destructive_change: OnDestructiveChange = OnDestructiveChange.ERROR,
+    ) -> t.Tuple[Snapshot, Snapshot]:
+        snapshot_old = make_snapshot(
+            SqlModel(
+                name=name,
+                dialect="duckdb",
+                query=parse_one(old_query),
+                kind=IncrementalByTimeRangeKind(
+                    time_column="ds", forward_only=True, on_destructive_change=on_destructive_change
+                ),
+            )
+        )
+
+        snapshot = make_snapshot(
+            SqlModel(
+                name=name,
+                dialect="duckdb",
+                query=parse_one(new_query),
+                kind=IncrementalByTimeRangeKind(
+                    time_column="ds", forward_only=True, on_destructive_change=on_destructive_change
+                ),
+            )
+        )
+        snapshot.previous_versions = (
+            SnapshotDataVersion(
+                fingerprint=SnapshotFingerprint(
+                    data_hash="test_data_hash",
+                    metadata_hash="test_metadata_hash",
+                ),
+                version="test_version",
+                change_category=SnapshotChangeCategory.FORWARD_ONLY,
+            ),
+        )
+
+        return snapshot_old, snapshot
+
+    return _make_function
+
+
 @pytest.fixture
 def random_name() -> t.Callable:
     return lambda: f"generated_{random_id()}"
@@ -365,7 +417,7 @@ def copy_to_temp_path(tmp_path: Path) -> t.Callable:
         return []
 
     def _make_function(
-        paths: t.Union[t.Union[str, Path], t.Collection[t.Union[str, Path]]]
+        paths: t.Union[t.Union[str, Path], t.Collection[t.Union[str, Path]]],
     ) -> t.List[Path]:
         paths = ensure_list(paths)
         temp_dirs = []
