@@ -10,6 +10,7 @@ from sqlglot import expressions as exp
 from sqlglot import parse_one
 from sqlglot.helper import ensure_list
 
+from sqlmesh.core import dialect as d
 from sqlmesh.core.dialect import normalize_model_name
 from sqlmesh.core.engine_adapter import EngineAdapter, EngineAdapterWithIndexSupport
 from sqlmesh.core.engine_adapter.shared import InsertOverwriteStrategy
@@ -26,12 +27,12 @@ def test_create_view(make_mocked_engine_adapter: t.Callable):
     adapter = make_mocked_engine_adapter(EngineAdapter)
     adapter.create_view("test_view", parse_one("SELECT a FROM tbl"))
     adapter.create_view("test_view", parse_one("SELECT a FROM tbl"), replace=False)
-    # Test that `table_properties` are ignored for base engine adapter
+    # Test that `physical_properties` are ignored for base engine adapter
     adapter.create_view(
         "test_view",
         parse_one("SELECT a FROM tbl"),
         replace=True,
-        table_properties={"a": exp.convert(1)},
+        view_properties={"a": exp.convert(1)},
     )
 
     assert to_sql_calls(adapter) == [
@@ -48,7 +49,7 @@ def test_create_view_pandas(make_mocked_engine_adapter: t.Callable):
         "test_view",
         pd.DataFrame({"a": [1, 2, 3]}),
         replace=True,
-        table_properties={"a": exp.convert(1)},
+        view_properties={"a": exp.convert(1)},
     )
 
     assert to_sql_calls(adapter) == [
@@ -825,10 +826,7 @@ def test_alter_table(
 
     adapter.columns = table_columns
 
-    adapter.alter_table(
-        current_table_name,
-        target_table_name,
-    )
+    adapter.alter_table(adapter.get_alter_expressions(current_table_name, target_table_name))
 
     adapter.cursor.begin.assert_called_once()
     adapter.cursor.commit.assert_called_once()
@@ -2312,7 +2310,7 @@ WITH "source" AS (
     COALESCE("joined"."t_price", "joined"."price") AS "price",
     COALESCE("t_test_valid_from", CAST('1970-01-01 00:00:00' AS TIMESTAMP)) AS "test_valid_from",
     CASE
-      WHEN 
+      WHEN
         (
           NOT "t_id" IS NULL AND NOT "id" IS NULL
         )
@@ -2661,3 +2659,96 @@ def test_get_data_objects_batching(mocker: MockerFixture, make_mocked_engine_ada
 
     assert len(calls[0][0][1]) == adapter.DATA_OBJECT_FILTER_BATCH_SIZE
     assert len(calls[1][0][1]) == 1
+
+
+def test_pre_ping(mocker: MockerFixture, make_mocked_engine_adapter: t.Callable):
+    adapter = make_mocked_engine_adapter(EngineAdapter)
+    adapter._pre_ping = True
+    adapter.cursor.execute.side_effect = RuntimeError("error")
+
+    with pytest.raises(RuntimeError):
+        adapter.execute("SELECT 'test'")
+
+    assert to_sql_calls(adapter) == [
+        "SELECT 1",  # ping
+        "SELECT 'test'",
+    ]
+
+    adapter._connection_pool.get().close.assert_called_once()
+
+
+@pytest.mark.parametrize(
+    "partitioned_by",
+    [
+        ["DATETIME_TRUNC(ds, MONTH)", "b"],
+        ["b"],
+    ],
+)
+def test_insert_overwrite_by_partition_query(
+    make_mocked_engine_adapter: t.Callable,
+    mocker: MockerFixture,
+    make_temp_table_name: t.Callable,
+    partitioned_by: t.List[str],
+):
+    adapter = make_mocked_engine_adapter(EngineAdapter)
+
+    temp_table_mock = mocker.patch("sqlmesh.core.engine_adapter.EngineAdapter._get_temp_table")
+    table_name = "test_schema.test_table"
+    temp_table_id = "abcdefgh"
+    temp_table_mock.return_value = make_temp_table_name(table_name, temp_table_id)
+
+    adapter.insert_overwrite_by_partition(
+        table_name,
+        parse_one("SELECT a, ds, b FROM tbl"),
+        partitioned_by=[d.parse_one(k) for k in partitioned_by],
+        columns_to_types={
+            "a": exp.DataType.build("int"),
+            "ds": exp.DataType.build("DATETIME"),
+            "b": exp.DataType.build("boolean"),
+        },
+    )
+
+    expected_delete_stmt = (
+        'DELETE FROM "test_schema"."test_table" WHERE CONCAT_WS(\'__SQLMESH_DELIM__\', DATETIME_TRUNC("ds", MONTH), "b") IN (SELECT DISTINCT CONCAT_WS(\'__SQLMESH_DELIM__\', DATETIME_TRUNC("ds", MONTH), "b") FROM "test_schema"."__temp_test_table_abcdefgh")'
+        if len(partitioned_by) > 1
+        else 'DELETE FROM "test_schema"."test_table" WHERE "b" IN (SELECT DISTINCT "b" FROM "test_schema"."__temp_test_table_abcdefgh")'
+    )
+
+    sql_calls = to_sql_calls(adapter)
+    assert sql_calls == [
+        'CREATE TABLE "test_schema"."__temp_test_table_abcdefgh" AS SELECT "a", "ds", "b" FROM "tbl"',
+        expected_delete_stmt,
+        'INSERT INTO "test_schema"."test_table" ("a", "ds", "b") SELECT "a", "ds", "b" FROM "test_schema"."__temp_test_table_abcdefgh"',
+        'DROP TABLE IF EXISTS "test_schema"."__temp_test_table_abcdefgh"',
+    ]
+
+
+def test_insert_overwrite_by_partition_query_insert_overwrite_strategy(
+    make_mocked_engine_adapter: t.Callable, mocker: MockerFixture, make_temp_table_name: t.Callable
+):
+    adapter = make_mocked_engine_adapter(EngineAdapter)
+    adapter.INSERT_OVERWRITE_STRATEGY = InsertOverwriteStrategy.INSERT_OVERWRITE
+
+    temp_table_mock = mocker.patch("sqlmesh.core.engine_adapter.EngineAdapter._get_temp_table")
+    table_name = "test_schema.test_table"
+    temp_table_id = "abcdefgh"
+    temp_table_mock.return_value = make_temp_table_name(table_name, temp_table_id)
+
+    adapter.insert_overwrite_by_partition(
+        table_name,
+        parse_one("SELECT a, ds, b FROM tbl"),
+        partitioned_by=[
+            d.parse_one("DATETIME_TRUNC(ds, MONTH)"),
+            d.parse_one("b"),
+        ],
+        columns_to_types={
+            "a": exp.DataType.build("int"),
+            "ds": exp.DataType.build("DATETIME"),
+            "b": exp.DataType.build("boolean"),
+        },
+    )
+
+    sql_calls = to_sql_calls(adapter)
+    assert sql_calls == [
+        'INSERT OVERWRITE TABLE "test_schema"."test_table" ("a", "ds", "b") SELECT "a", "ds", "b" FROM "tbl"'
+    ]
