@@ -201,7 +201,7 @@ def test_insert_overwrite_by_time_partition_pandas(
     ]
     assert load_temp_table.kwargs["df"].equals(df)
     assert load_temp_table.kwargs["table"] == get_temp_bq_table.return_value
-    assert load_temp_table.kwargs["job_config"].write_disposition == None
+    assert load_temp_table.kwargs["job_config"].write_disposition is None
     assert (
         merge_sql.sql(dialect="bigquery")
         == "MERGE INTO test_table AS __MERGE_TARGET__ USING (SELECT `a`, `ds` FROM (SELECT `a`, `ds` FROM project.dataset.temp_table) AS _subquery WHERE ds BETWEEN '2022-01-01' AND '2022-01-05') AS __MERGE_SOURCE__ ON FALSE WHEN NOT MATCHED BY SOURCE AND ds BETWEEN '2022-01-01' AND '2022-01-05' THEN DELETE WHEN NOT MATCHED THEN INSERT (a, ds) VALUES (a, ds)"
@@ -415,7 +415,7 @@ def test_ctas_time_partition(
 
     sql_calls = _to_sql_calls(execute_mock)
     assert sql_calls == [
-        f"CREATE TABLE IF NOT EXISTS `test_table` PARTITION BY `ds` AS SELECT * FROM `a`",
+        "CREATE TABLE IF NOT EXISTS `test_table` PARTITION BY `ds` AS SELECT * FROM `a`",
     ]
 
 
@@ -719,3 +719,37 @@ def test_drop_schema(
     sql_calls = _to_sql_calls(execute_mock)
 
     assert sql_calls == ensure_list(expected)
+
+
+def test_view_properties(make_mocked_engine_adapter: t.Callable, mocker: MockerFixture):
+    adapter = make_mocked_engine_adapter(BigQueryEngineAdapter)
+    execute_mock = mocker.patch(
+        "sqlmesh.core.engine_adapter.bigquery.BigQueryEngineAdapter.execute"
+    )
+
+    adapter.create_view(
+        "test_table",
+        parse_one("SELECT 1"),
+        table_description="some description",
+        view_properties={
+            "labels": exp.array("('test-label', 'label-value')"),
+        },
+    )
+
+    adapter.create_view(
+        "test_table",
+        parse_one("SELECT 1"),
+        table_description="some description",
+        view_properties={
+            "labels": exp.array("('test-view-label', 'label-view-value')"),
+        },
+    )
+
+    adapter.create_view("test_table", parse_one("SELECT 1"), view_properties={})
+
+    sql_calls = _to_sql_calls(execute_mock)
+    assert sql_calls == [
+        "CREATE OR REPLACE VIEW `test_table` OPTIONS (description='some description', labels=[('test-label', 'label-value')]) AS SELECT 1",
+        "CREATE OR REPLACE VIEW `test_table` OPTIONS (description='some description', labels=[('test-view-label', 'label-view-value')]) AS SELECT 1",
+        "CREATE OR REPLACE VIEW `test_table` AS SELECT 1",
+    ]
