@@ -16,7 +16,11 @@ from sqlmesh import Config, Context, EngineAdapter
 from sqlmesh.cli.example_project import init_example_project
 from sqlmesh.core.config import load_config_from_paths
 from sqlmesh.core.dialect import normalize_model_name
+import sqlmesh.core.dialect as d
+from sqlmesh.core.engine_adapter import SparkEngineAdapter, TrinoEngineAdapter
+from sqlmesh.core.model import load_sql_based_model
 from sqlmesh.core.engine_adapter.shared import DataObject, DataObjectType
+from sqlmesh.core.model.definition import create_sql_model
 from sqlmesh.utils import random_id
 from sqlmesh.utils.date import now, to_date, to_ds, to_time_column, yesterday
 from sqlmesh.utils.pydantic import PydanticModel
@@ -35,10 +39,12 @@ class TestContext:
         self,
         test_type: str,
         engine_adapter: EngineAdapter,
+        gateway: str,
         columns_to_types: t.Optional[t.Dict[str, t.Union[str, exp.DataType]]] = None,
     ):
         self.test_type = test_type
         self.engine_adapter = engine_adapter
+        self.gateway = gateway
         self._columns_to_types = columns_to_types
         self.test_id = random_id(short=True)
 
@@ -96,6 +102,20 @@ class TestContext:
     def current_catalog_type(self) -> str:
         return self.engine_adapter.current_catalog_type
 
+    @property
+    def supports_merge(self) -> bool:
+        if self.dialect == "spark":
+            engine_adapter: SparkEngineAdapter = self.engine_adapter
+            # Spark supports MERGE on the Iceberg catalog (which is configured under "testing" in these integration tests)
+            return engine_adapter.default_catalog == "testing"
+
+        if self.dialect == "trino":
+            engine_adapter: TrinoEngineAdapter = self.engine_adapter
+            # Trino supports MERGE on Delta and Iceberg but not Hive
+            return engine_adapter.get_catalog_type(engine_adapter.default_catalog) != "hive"
+
+        return True
+
     def add_test_suffix(self, value: str) -> str:
         return f"{value}_{self.test_id}"
 
@@ -110,11 +130,11 @@ class TestContext:
 
     def _format_df(self, data: pd.DataFrame, to_datetime: bool = True) -> pd.DataFrame:
         for timestamp_column in self.timestamp_columns:
-            if timestamp_column in data.columns and to_datetime:
-                value = pd.to_datetime(data[timestamp_column])
-                if self.dialect == "duckdb":
-                    value = value.astype("datetime64[us]")
-                data[timestamp_column] = value
+            if timestamp_column in data.columns:
+                value = data[timestamp_column]
+                if to_datetime:
+                    value = pd.to_datetime(value)
+                data[timestamp_column] = value.astype("datetime64[ns]")
         return data
 
     def init(self):
@@ -173,7 +193,7 @@ class TestContext:
         df = self.engine_adapter.fetchdf(exp.select("*").from_(table), quote_identifiers=True)
         if self.dialect == "snowflake" and "id" in df.columns:
             df["id"] = df["id"].apply(lambda x: x if pd.isna(x) else int(x))
-        return df
+        return self._format_df(df)
 
     def compare_with_current(self, table: exp.Table, expected: pd.DataFrame) -> None:
         compare_dataframes(
@@ -249,6 +269,17 @@ class TestContext:
                     schema_name = '{schema_name}'
                     AND table_name = '{table_name}'
             """
+        elif self.dialect == "duckdb":
+            kind = "table" if table_kind == "BASE TABLE" else "view"
+            query = f"""
+                SELECT
+                    {kind}_name,
+                    comment
+                FROM duckdb_{kind}s()
+                WHERE
+                    schema_name = '{schema_name}'
+                    AND {kind}_name = '{table_name}'
+            """
 
         result = self.engine_adapter.fetchall(query)
 
@@ -333,6 +364,16 @@ class TestContext:
         elif self.dialect == "trino":
             query = f"SHOW COLUMNS FROM {schema_name}.{table_name}"
             comment_index = 3
+        elif self.dialect == "duckdb":
+            query = f"""
+                SELECT
+                    column_name,
+                    comment
+                FROM duckdb_columns()
+                WHERE
+                    schema_name = '{schema_name}'
+                    AND table_name = '{table_name}'
+            """
 
         result = self.engine_adapter.fetchall(query)
 
@@ -349,6 +390,34 @@ class TestContext:
 
         return comments
 
+    def create_context(
+        self, config_mutator: t.Optional[t.Callable[[str, Config], None]] = None
+    ) -> Context:
+        private_sqlmesh_dir = pathlib.Path(pathlib.Path().home(), ".sqlmesh")
+        config = load_config_from_paths(
+            Config,
+            project_paths=[
+                pathlib.Path(os.path.join(os.path.dirname(__file__), "config.yaml")),
+                private_sqlmesh_dir / "config.yml",
+                private_sqlmesh_dir / "config.yaml",
+            ],
+        )
+        if config_mutator:
+            config_mutator(self.gateway, config)
+        self._context = Context(paths=".", config=config, gateway=self.gateway)
+        return self._context
+
+    def cleanup(self, ctx: Context):
+        schemas = []
+        for _, model in ctx.models.items():
+            schemas.append(model.schema_name)
+            schemas.append(model.physical_schema)
+
+        for schema_name in set(schemas):
+            self.engine_adapter.drop_schema(
+                schema_name=schema_name, ignore_if_not_exists=True, cascade=True
+            )
+
 
 class MetadataResults(PydanticModel):
     tables: t.List[str] = []
@@ -537,8 +606,9 @@ def default_columns_to_types():
 
 
 @pytest.fixture
-def ctx(engine_adapter, test_type):
-    return TestContext(test_type, engine_adapter)
+def ctx(engine_adapter, test_type, mark_gateway):
+    _, gateway = mark_gateway
+    return TestContext(test_type, engine_adapter, gateway)
 
 
 def test_catalog_operations(ctx: TestContext):
@@ -1102,7 +1172,7 @@ def test_insert_overwrite_by_time_partition(ctx: TestContext):
 
 
 def test_merge(ctx: TestContext):
-    if ctx.dialect in ("trino", "spark"):
+    if not ctx.supports_merge:
         pytest.skip(f"{ctx.dialect} doesn't support merge")
 
     ctx.init()
@@ -1163,7 +1233,7 @@ def test_merge(ctx: TestContext):
 
 
 def test_scd_type_2_by_time(ctx: TestContext):
-    time_type = "timestamp"
+    time_type = exp.DataType.build("timestamp")
 
     ctx.columns_to_types = {
         "id": "int",
@@ -1306,7 +1376,7 @@ def test_scd_type_2_by_time(ctx: TestContext):
 
 
 def test_scd_type_2_by_column(ctx: TestContext):
-    time_type = "datetime" if ctx.dialect == "bigquery" else "timestamp"
+    time_type = exp.DataType.build("timestamp")
 
     ctx.columns_to_types = {
         "id": "int",
@@ -1631,7 +1701,7 @@ def test_sushi(mark_gateway: t.Tuple[str, str], ctx: TestContext):
 
         cust_rev_by_day_model_tbl_props = context._models[cust_rev_by_day_key].copy(
             update={
-                "table_properties": {
+                "physical_properties": {
                     "delta.columnMapping.mode": exp.Literal(this="name", is_string=True)
                 }
             }
@@ -1691,6 +1761,7 @@ def test_sushi(mark_gateway: t.Tuple[str, str], ctx: TestContext):
             },
             "raw_marketing": {
                 "table": "Table of marketing status.",
+                "column": {"customer_id": "Unique identifier of the customer"},
             },
             "top_waiters": {
                 "table": "View of top waiters.",
@@ -1873,8 +1944,7 @@ def test_sushi(mark_gateway: t.Tuple[str, str], ctx: TestContext):
 
     # confirm view layer comments are registered in PROD
     if ctx.engine_adapter.COMMENT_CREATION_VIEW.is_supported:
-        prod_plan = context.plan(skip_tests=True, no_prompts=True, auto_apply=True)
-
+        context.plan(skip_tests=True, no_prompts=True, auto_apply=True)
         validate_comments("sushi", is_physical_layer=False)
 
 
@@ -2086,3 +2156,89 @@ def test_to_time_column(
         assert df[col_name][0] is expected
     else:
         assert df[col_name][0] == expected
+
+
+def test_batch_size_on_incremental_by_unique_key_model(
+    ctx: TestContext, mark_gateway: t.Tuple[str, str]
+):
+    if ctx.test_type != "query":
+        pytest.skip("This only needs to run once so we skip anything not query")
+
+    if not ctx.supports_merge:
+        _, gateway = mark_gateway
+        pytest.skip(f"{ctx.dialect} on {gateway} doesnt support merge")
+
+    def _mutate_config(current_gateway_name: str, config: Config):
+        # make stepping through in the debugger easier
+        connection = config.gateways[current_gateway_name].connection
+        connection.concurrent_tasks = 1
+
+    context = ctx.create_context(_mutate_config)
+
+    schema = ctx.schema(TEST_SCHEMA)
+    seed_query = ctx.input_data(
+        pd.DataFrame(
+            [
+                [2, "2020-01-01"],
+                [1, "2020-01-01"],
+                [3, "2020-01-03"],
+                [1, "2020-01-04"],
+                [1, "2020-01-05"],
+                [1, "2020-01-06"],
+                [1, "2020-01-07"],
+            ],
+            columns=["item_id", "event_date"],
+        ),
+        columns_to_types={
+            "item_id": exp.DataType.build("integer"),
+            "event_date": exp.DataType.build("date"),
+        },
+    )
+    context.upsert_model(create_sql_model(name=f"{schema}.seed_model", query=seed_query))
+    context.upsert_model(
+        load_sql_based_model(
+            d.parse(
+                f"""MODEL (
+                    name {schema}.test_model,
+                    kind INCREMENTAL_BY_UNIQUE_KEY (
+                        unique_key item_id,
+                        batch_size 1
+                    ),
+                    start '2020-01-01',
+                    end '2020-01-07',
+                    cron '@daily'
+                );
+
+                select * from {schema}.seed_model
+                where event_date between @start_date and @end_date""",
+            )
+        )
+    )
+
+    try:
+        context.plan(auto_apply=True, no_prompts=True)
+
+        results = ctx.get_metadata_results(schema)
+        assert "test_model" in results.views
+
+        actual_df = (
+            ctx.get_current_data(f"{schema}.test_model")
+            .sort_values(by="event_date")
+            .reset_index(drop=True)
+        )
+        actual_df["event_date"] = actual_df["event_date"].astype(str)
+        assert actual_df.count()[0] == 3
+
+        expected_df = pd.DataFrame(
+            [[2, "2020-01-01"], [3, "2020-01-03"], [1, "2020-01-07"]],
+            columns=actual_df.columns,
+        ).sort_values(by="event_date")
+
+        pd.testing.assert_frame_equal(
+            actual_df,
+            expected_df,
+            check_dtype=False,
+        )
+
+    finally:
+        ctx.cleanup(context)
