@@ -118,7 +118,9 @@ def test_replace_query_table_properties_exists(
 def test_create_view_properties(make_mocked_engine_adapter: t.Callable):
     adapter = make_mocked_engine_adapter(SparkEngineAdapter)
 
-    adapter.create_view("test_view", parse_one("SELECT a FROM tbl"), table_properties={"a": exp.convert(1)})  # type: ignore
+    adapter.create_view(
+        "test_view", parse_one("SELECT a FROM tbl"), view_properties={"a": exp.convert(1)}
+    )  # type: ignore
     adapter.cursor.execute.assert_called_once_with(
         "CREATE OR REPLACE VIEW test_view TBLPROPERTIES ('a'=1) AS SELECT a FROM tbl"
     )
@@ -148,7 +150,7 @@ def test_alter_table(make_mocked_engine_adapter: t.Callable):
 
     adapter.columns = table_columns
 
-    adapter.alter_table(current_table_name, target_table_name)
+    adapter.alter_table(adapter.get_alter_expressions(current_table_name, target_table_name))
 
     adapter.cursor.execute.assert_has_calls(
         [
@@ -323,7 +325,7 @@ def test_replace_query_self_ref_exists(
         # the schema for the table already exists
         "CREATE SCHEMA IF NOT EXISTS `db`",
         f"CREATE TABLE IF NOT EXISTS `db`.`temp_table_{temp_table_id}` AS SELECT `col` FROM `db`.`table`",
-        f"INSERT OVERWRITE TABLE `db`.`table` (`col`) SELECT `col` + 1 AS `col` FROM `db`.`temp_table_abcdefgh`",
+        "INSERT OVERWRITE TABLE `db`.`table` (`col`) SELECT `col` + 1 AS `col` FROM `db`.`temp_table_abcdefgh`",
         f"DROP TABLE IF EXISTS `db`.`temp_table_{temp_table_id}`",
     ]
 
