@@ -107,7 +107,7 @@ def test_render_sql_model(sushi_context, assert_exp_eq, copy_to_temp_path: t.Cal
             end=date(2021, 1, 1),
             expand=True,
         ),
-        f"""
+        """
         SELECT
           CAST("o"."waiter_id" AS INT) AS "waiter_id", /* Waiter id */
           CAST(SUM("oi"."quantity" * "i"."price") AS DOUBLE) AS "revenue", /* Revenue from orders taken by this waiter */
@@ -157,7 +157,7 @@ def test_render_sql_model(sushi_context, assert_exp_eq, copy_to_temp_path: t.Cal
     unpushed = Context(paths=copy_to_temp_path("examples/sushi"))
     assert_exp_eq(
         unpushed.render("sushi.waiter_revenue_by_day"),
-        f"""
+        """
         SELECT
           CAST("o"."waiter_id" AS INT) AS "waiter_id", /* Waiter id */
           CAST(SUM("oi"."quantity" * "i"."price") AS DOUBLE) AS "revenue", /* Revenue from orders taken by this waiter */
@@ -210,6 +210,7 @@ def test_diff(sushi_context: Context, mocker: MockerFixture):
     )
     plan = PlanBuilder(
         context_diff=sushi_context._context_diff("prod"),
+        engine_schema_differ=sushi_context.engine_adapter.SCHEMA_DIFFER,
     ).build()
 
     promotion_result = plan_evaluator._promote(plan)
@@ -254,6 +255,36 @@ def test_evaluate_limit():
     assert context.evaluate("without_limit", "2020-01-01", "2020-01-02", "2020-01-02", 2).size == 2
 
 
+def test_plan_execution_time():
+    context = Context(config=Config())
+    context.upsert_model(
+        load_sql_based_model(
+            parse(
+                """
+                MODEL(
+                    name db.x,
+                    start '2024-01-01',
+                    kind FULL
+                );
+
+                SELECT @execution_date AS execution_date
+                """
+            )
+        )
+    )
+
+    context.plan(
+        "dev",
+        execution_time="2024-01-02",
+        auto_apply=True,
+        no_prompts=True,
+    )
+    assert (
+        str(list(context.fetchdf("select * from db__dev.x")["execution_date"])[0])
+        == "2024-01-02 00:00:00"
+    )
+
+
 def test_clear_caches(tmp_path: pathlib.Path):
     models_dir = tmp_path / "models"
 
@@ -282,12 +313,12 @@ def test_ignore_files(mocker: MockerFixture, tmp_path: pathlib.Path):
     models_dir = pathlib.Path("models")
     macros_dir = pathlib.Path("macros")
 
-    ignore_model_file = create_temp_file(
+    create_temp_file(
         tmp_path,
         pathlib.Path(models_dir, "ignore", "ignore_model.sql"),
         "MODEL(name ignore.ignore_model); SELECT 1 AS cola",
     )
-    ignore_macro_file = create_temp_file(
+    create_temp_file(
         tmp_path,
         pathlib.Path(macros_dir, "macro_ignore.py"),
         """
@@ -298,7 +329,7 @@ def test():
     return "test"
 """,
     )
-    constant_ignore_model_file = create_temp_file(
+    create_temp_file(
         tmp_path,
         pathlib.Path(models_dir, ".ipynb_checkpoints", "ignore_model2.sql"),
         "MODEL(name ignore_model2); SELECT cola::bigint AS cola FROM db.other_table",
@@ -563,6 +594,7 @@ def test_unrestorable_snapshot(sushi_context: Context) -> None:
         """
         ),
         default_catalog=sushi_context.default_catalog,
+        dialect=sushi_context.default_dialect,
     )
     model_v2 = load_sql_based_model(
         parse(
@@ -572,6 +604,7 @@ def test_unrestorable_snapshot(sushi_context: Context) -> None:
         """
         ),
         default_catalog=sushi_context.default_catalog,
+        dialect=sushi_context.default_dialect,
     )
 
     sushi_context.upsert_model(model_v1)
@@ -581,10 +614,20 @@ def test_unrestorable_snapshot(sushi_context: Context) -> None:
     )
 
     sushi_context.upsert_model(model_v2)
-    sushi_context.plan(auto_apply=True, no_prompts=True, forward_only=True)
+    sushi_context.plan(
+        auto_apply=True,
+        no_prompts=True,
+        forward_only=True,
+        allow_destructive_models=["memory.sushi.test_unrestorable"],
+    )
 
     sushi_context.upsert_model(model_v1)
-    sushi_context.plan(auto_apply=True, no_prompts=True, forward_only=True)
+    sushi_context.plan(
+        auto_apply=True,
+        no_prompts=True,
+        forward_only=True,
+        allow_destructive_models=["memory.sushi.test_unrestorable"],
+    )
     model_v1_new_snapshot = sushi_context.get_snapshot(
         "memory.sushi.test_unrestorable", raise_if_missing=True
     )
@@ -625,7 +668,7 @@ def test_load_external_models(copy_to_temp_path):
 
     assert len(external_model_names) > 0
 
-    # from schema.yaml in root dir
+    # from default external_models.yaml in root dir
     assert "raw.demographics" in external_model_names
 
     # from external_models/model1.yaml
