@@ -20,12 +20,12 @@ def test_format_model_expressions():
         parse(
             """
     MODEL(
-    name a.b,
-    kind full,
+    name a.b, -- a
+    kind full, -- b
     references (
      a,
      (b, c) as d,
-     ),
+     ),  -- c
      audits [
     not_null(columns=[
       foo_id,
@@ -83,9 +83,9 @@ def test_format_model_expressions():
     assert (
         x
         == """MODEL (
-  name a.b,
-  kind FULL,
-  references (a, (b, c) AS d),
+  name a.b, /* a */
+  kind FULL, /* b */
+  references (a, (b, c) AS d), /* c */
   audits ARRAY(
     NOT_NULL(
       columns = ARRAY(
@@ -173,6 +173,27 @@ JINJA_QUERY_BEGIN;
 JINJA_END;"""
     )
 
+    x = format_model_expressions(
+        parse(
+            """
+            MODEL(name a.b, kind FULL, dialect bigquery);
+            SELECT SAFE_CAST('bla' AS INT64) AS FOO
+            """
+        ),
+        dialect="bigquery",
+    )
+    assert (
+        x
+        == """MODEL (
+  name a.b,
+  kind FULL,
+  dialect bigquery
+);
+
+SELECT
+  SAFE_CAST('bla' AS INT64) AS FOO"""
+    )
+
 
 def test_macro_format():
     assert parse_one("@EACH(ARRAY(1,2), x -> x)").sql() == "@EACH(ARRAY(1, 2), x -> x)"
@@ -216,9 +237,7 @@ def test_text_diff():
 -  *
 -FROM x
 +  1
-+FROM y""" in text_diff(
-        parse("SELECT * FROM x"), parse("SELECT 1 FROM y")
-    )
++FROM y""" in text_diff(parse("SELECT * FROM x"), parse("SELECT 1 FROM y"))
 
 
 def test_parse():
@@ -318,13 +337,21 @@ def test_seed():
         """
         MODEL (
             kind SEED (
-                path '..\..\..\data\data.csv',
+                path '..\\..\\..\\data\\data.csv', -- c
             ),
         );
     """
     )
     assert len(expressions) == 1
     assert "../../../data/data.csv" in expressions[0].sql()
+    assert (
+        format_model_expressions(expressions)
+        == """MODEL (
+  kind SEED (
+    path '../../../data/data.csv' /* c */
+  )
+)"""
+    )
 
 
 def test_select_from_values_for_batch_range_json():
@@ -532,3 +559,36 @@ def test_model_normalization_quote_flexibility():
     # It doesn't work the other way which is what we currently expect
     with pytest.raises(ParseError):
         normalize_model_name("`catalog`.`db`.`table`", default_catalog=None, dialect=None)
+
+
+def test_macro_parse():
+    q = parse_one(
+        """select * from table(@get(x) OVER (PARTITION BY y ORDER BY z)) AS results""",
+        read="snowflake",
+    )
+    assert (
+        q.sql()
+        == "SELECT * FROM TABLE(@get(x) OVER (PARTITION BY y ORDER BY z NULLS LAST)) AS results"
+    )
+
+
+def test_conditional_statement():
+    q = parse_one(
+        """
+        @IF(
+          TRUE,
+          COPY INTO 's3://example/data.csv'
+            FROM EXTRA.EXAMPLE.TABLE
+            STORAGE_INTEGRATION = S3_INTEGRATION
+            FILE_FORMAT = (TYPE = CSV COMPRESSION = NONE NULL_IF = ('') FIELD_OPTIONALLY_ENCLOSED_BY = '"')
+            HEADER = TRUE
+            OVERWRITE = TRUE
+            SINGLE = TRUE
+        ) -- this is a comment
+        """,
+        read="snowflake",
+    )
+    assert (
+        q.sql("snowflake")
+        == "@IF(TRUE, COPY INTO 's3://example/data.csv' FROM EXTRA.EXAMPLE.TABLE STORAGE_INTEGRATION = S3_INTEGRATION FILE_FORMAT = (TYPE=CSV COMPRESSION=NONE NULL_IF=('') FIELD_OPTIONALLY_ENCLOSED_BY='\"') HEADER = TRUE OVERWRITE = TRUE SINGLE = TRUE /* this is a comment */)"
+    )
