@@ -23,6 +23,7 @@ from sqlmesh.core.model import (
     ModelKindName,
     SqlModel,
     TimeColumn,
+    load_sql_based_model,
 )
 from sqlmesh.core.model.kind import model_kind_type_from_name
 from sqlmesh.core.plan import Plan, PlanBuilder, SnapshotIntervals
@@ -358,7 +359,7 @@ def test_hourly_model_with_lookback_no_backfill_in_dev(init_and_plan_context: t.
     top_waiters_model = add_projection_to_model(t.cast(SqlModel, top_waiters_model), literal=True)
     context.upsert_model(top_waiters_model)
 
-    snapshot = context.get_snapshot(model, raise_if_missing=True)
+    context.get_snapshot(model, raise_if_missing=True)
     top_waiters_snapshot = context.get_snapshot("sushi.top_waiters", raise_if_missing=True)
 
     with freeze_time(now() + timedelta(hours=2)):
@@ -1107,6 +1108,60 @@ def test_dbt_select_star_is_directly_modified(sushi_test_dbt_context: Context):
     assert plan.snapshots[snapshot_b_id].change_category == SnapshotChangeCategory.NON_BREAKING
 
 
+@freeze_time("2023-01-08 15:00:00")
+def test_incremental_by_partition(init_and_plan_context: t.Callable):
+    context, plan = init_and_plan_context("examples/sushi")
+    context.apply(plan)
+
+    source_name = "raw.test_incremental_by_partition"
+    model_name = "memory.sushi.test_incremental_by_partition"
+
+    expressions = d.parse(
+        f"""
+        MODEL (
+            name {model_name},
+            kind INCREMENTAL_BY_PARTITION,
+            partitioned_by [key],
+            allow_partials true,
+        );
+
+        SELECT key, value FROM {source_name};
+        """
+    )
+    model = load_sql_based_model(expressions)
+    context.upsert_model(model)
+
+    context.engine_adapter.ctas(
+        source_name,
+        d.parse_one("SELECT 'key_a' AS key, 1 AS value"),
+    )
+
+    context.plan(auto_apply=True, no_prompts=True)
+    assert context.engine_adapter.fetchall(f"SELECT * FROM {model_name}") == [
+        ("key_a", 1),
+    ]
+
+    context.engine_adapter.replace_query(
+        source_name,
+        d.parse_one("SELECT 'key_b' AS key, 1 AS value"),
+    )
+    context.run(ignore_cron=True)
+    assert context.engine_adapter.fetchall(f"SELECT * FROM {model_name}") == [
+        ("key_a", 1),
+        ("key_b", 1),
+    ]
+
+    context.engine_adapter.replace_query(
+        source_name,
+        d.parse_one("SELECT 'key_a' AS key, 2 AS value"),
+    )
+    context.run(ignore_cron=True)
+    assert context.engine_adapter.fetchall(f"SELECT * FROM {model_name}") == [
+        ("key_b", 1),
+        ("key_a", 2),
+    ]
+
+
 @pytest.mark.parametrize(
     "context_fixture",
     ["sushi_context", "sushi_dbt_context", "sushi_test_dbt_context", "sushi_no_default_catalog"],
@@ -1396,7 +1451,12 @@ def test_environment_promotion(sushi_context: Context):
         DataType.Type.DOUBLE,
         DataType.Type.FLOAT,
     )
-    apply_to_environment(sushi_context, "dev", SnapshotChangeCategory.FORWARD_ONLY)
+    apply_to_environment(
+        sushi_context,
+        "dev",
+        SnapshotChangeCategory.FORWARD_ONLY,
+        allow_destructive_models=['"memory"."sushi"."customer_revenue_by_day"'],
+    )
 
     # Promote to prod
     def _validate_plan(context, plan):
@@ -1428,6 +1488,7 @@ def test_environment_promotion(sushi_context: Context):
         "prod",
         SnapshotChangeCategory.NON_BREAKING,
         plan_validators=[_validate_plan],
+        allow_destructive_models=['"memory"."sushi"."customer_revenue_by_day"'],
     )
 
 
@@ -1721,17 +1782,17 @@ def test_environment_suffix_target_table(init_and_plan_context: t.Callable):
     context, plan = init_and_plan_context("examples/sushi", config="environment_suffix_config")
     context.apply(plan)
     metadata = DuckDBMetadata.from_context(context)
-    environments_schemas = {"raw", "sushi"}
+    environments_schemas = {"sushi"}
     internal_schemas = {"sqlmesh", "sqlmesh__sushi"}
     starting_schemas = environments_schemas | internal_schemas
     # Make sure no new schemas are created
-    assert set(metadata.schemas) - starting_schemas == set()
+    assert set(metadata.schemas) - starting_schemas == {"raw"}
     prod_views = {x for x in metadata.qualified_views if x.db in environments_schemas}
     # Make sure that all models are present
     assert len(prod_views) == 12
     apply_to_environment(context, "dev")
     # Make sure no new schemas are created
-    assert set(metadata.schemas) - starting_schemas == set()
+    assert set(metadata.schemas) - starting_schemas == {"raw"}
     dev_views = {
         x for x in metadata.qualified_views if x.db in environments_schemas and "__dev" in x.name
     }
@@ -1788,14 +1849,13 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
     ) = get_default_catalog_and_non_tables(metadata, context.default_catalog)
     assert len(prod_views) == 12
     assert len(dev_views) == 0
-    assert len(user_default_tables) == 23
+    assert len(user_default_tables) == 22
     assert state_metadata.schemas == ["sqlmesh"]
     assert {x.sql() for x in state_metadata.qualified_tables}.issuperset(
         {
             "physical.sqlmesh._environments",
             "physical.sqlmesh._intervals",
             "physical.sqlmesh._plan_dags",
-            "physical.sqlmesh._seeds",
             "physical.sqlmesh._snapshots",
             "physical.sqlmesh._versions",
         }
@@ -1808,7 +1868,7 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
     ) = get_default_catalog_and_non_tables(metadata, context.default_catalog)
     assert len(prod_views) == 12
     assert len(dev_views) == 12
-    assert len(user_default_tables) == 23
+    assert len(user_default_tables) == 22
     assert len(non_default_tables) == 0
     assert state_metadata.schemas == ["sqlmesh"]
     assert {x.sql() for x in state_metadata.qualified_tables}.issuperset(
@@ -1816,7 +1876,6 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
             "physical.sqlmesh._environments",
             "physical.sqlmesh._intervals",
             "physical.sqlmesh._plan_dags",
-            "physical.sqlmesh._seeds",
             "physical.sqlmesh._snapshots",
             "physical.sqlmesh._versions",
         }
@@ -1829,7 +1888,7 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
     ) = get_default_catalog_and_non_tables(metadata, context.default_catalog)
     assert len(prod_views) == 12
     assert len(dev_views) == 24
-    assert len(user_default_tables) == 23
+    assert len(user_default_tables) == 22
     assert len(non_default_tables) == 0
     assert state_metadata.schemas == ["sqlmesh"]
     assert {x.sql() for x in state_metadata.qualified_tables}.issuperset(
@@ -1837,7 +1896,6 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
             "physical.sqlmesh._environments",
             "physical.sqlmesh._intervals",
             "physical.sqlmesh._plan_dags",
-            "physical.sqlmesh._seeds",
             "physical.sqlmesh._snapshots",
             "physical.sqlmesh._versions",
         }
@@ -1851,7 +1909,7 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
     ) = get_default_catalog_and_non_tables(metadata, context.default_catalog)
     assert len(prod_views) == 12
     assert len(dev_views) == 12
-    assert len(user_default_tables) == 23
+    assert len(user_default_tables) == 22
     assert len(non_default_tables) == 0
     assert state_metadata.schemas == ["sqlmesh"]
     assert {x.sql() for x in state_metadata.qualified_tables}.issuperset(
@@ -1859,7 +1917,6 @@ def test_environment_catalog_mapping(init_and_plan_context: t.Callable):
             "physical.sqlmesh._environments",
             "physical.sqlmesh._intervals",
             "physical.sqlmesh._plan_dags",
-            "physical.sqlmesh._seeds",
             "physical.sqlmesh._snapshots",
             "physical.sqlmesh._versions",
         }
@@ -1916,6 +1973,7 @@ def apply_to_environment(
     plan_validators: t.Optional[t.Iterable[t.Callable]] = None,
     apply_validators: t.Optional[t.Iterable[t.Callable]] = None,
     plan_start: t.Optional[TimeLike] = None,
+    allow_destructive_models: t.Optional[t.List[str]] = None,
 ):
     plan_validators = plan_validators or []
     apply_validators = apply_validators or []
@@ -1925,6 +1983,7 @@ def apply_to_environment(
         start=plan_start or start(context) if environment != c.PROD else None,
         forward_only=choice == SnapshotChangeCategory.FORWARD_ONLY,
         include_unmodified=True,
+        allow_destructive_models=allow_destructive_models if allow_destructive_models else [],
     )
     if environment != c.PROD:
         plan_builder.set_start(plan_start or start(context))
@@ -2025,9 +2084,9 @@ def validate_state_sync_environment(
 def validate_tables(snapshots: t.Iterable[Snapshot], context: Context) -> None:
     adapter = context.engine_adapter
     for snapshot in snapshots:
-        if not snapshot.is_model:
+        if not snapshot.is_model or snapshot.is_external:
             continue
-        table_should_exist = not snapshot.is_symbolic
+        table_should_exist = not snapshot.is_embedded
         assert adapter.table_exists(snapshot.table_name()) == table_should_exist
         if table_should_exist:
             assert select_all(snapshot.table_name(), adapter)
