@@ -30,7 +30,7 @@ from sqlmesh.core.model import (
     create_seed_model,
     load_sql_based_model,
 )
-from sqlmesh.core.model.kind import TimeColumn
+from sqlmesh.core.model.kind import TimeColumn, ModelKindName
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
     QualifiedViewName,
@@ -54,7 +54,7 @@ from sqlmesh.utils.jinja import JinjaMacroRegistry, MacroInfo
 def parent_model():
     return SqlModel(
         name="parent.tbl",
-        kind=IncrementalByTimeRangeKind(time_column="ds"),
+        kind=dict(time_column="ds", name=ModelKindName.INCREMENTAL_BY_TIME_RANGE),
         dialect="spark",
         query=parse_one("SELECT 1, ds"),
     )
@@ -64,7 +64,7 @@ def parent_model():
 def model():
     return SqlModel(
         name="name",
-        kind=IncrementalByTimeRangeKind(time_column="ds", batch_size=30),
+        kind=dict(time_column="ds", batch_size=30, name=ModelKindName.INCREMENTAL_BY_TIME_RANGE),
         owner="owner",
         dialect="spark",
         cron="1 0 * * *",
@@ -105,11 +105,12 @@ def test_json(snapshot: Snapshot):
             "cron": "1 0 * * *",
             "kind": {
                 "name": "INCREMENTAL_BY_TIME_RANGE",
-                "time_column": {"column": '"ds"'},
+                "time_column": {"column": "`ds`"},
                 "batch_size": 30,
                 "forward_only": False,
+                "on_destructive_change": "ERROR",
                 "disable_restatement": False,
-                "dialect": "",
+                "dialect": "spark",
             },
             "mapping_schema": {},
             "start": "2020-01-01",
@@ -623,7 +624,7 @@ def test_get_removal_intervals_full_history_restatement_model(make_snapshot):
     assert interval == (to_timestamp("2023-01-01"), execution_time)
 
 
-each_macro = lambda: "test"
+each_macro = lambda: "test"  # noqa: E731
 
 
 def test_fingerprint(model: Model, parent_model: Model):
@@ -631,8 +632,8 @@ def test_fingerprint(model: Model, parent_model: Model):
     fingerprint = fingerprint_from_node(model, nodes={})
 
     original_fingerprint = SnapshotFingerprint(
-        data_hash="3163676913",
-        metadata_hash="892368116",
+        data_hash="3582214120",
+        metadata_hash="2793463216",
     )
 
     assert fingerprint == original_fingerprint
@@ -730,8 +731,8 @@ def test_fingerprint_jinja_macros(model: Model):
         }
     )
     original_fingerprint = SnapshotFingerprint(
-        data_hash="2664333317",
-        metadata_hash="892368116",
+        data_hash="2973224250",
+        metadata_hash="2793463216",
     )
 
     fingerprint = fingerprint_from_node(model, nodes={})
@@ -789,6 +790,22 @@ def test_fingerprint_standalone_audits(parent_model: Model):
     assert new_fingerprint.metadata_hash != fingerprint.metadata_hash
 
 
+def test_fingerprint_virtual_properties(model: Model, parent_model: Model):
+    original_model = deepcopy(model)
+    fingerprint = fingerprint_from_node(model, nodes={})
+
+    updated_model = SqlModel(
+        **original_model.dict(),
+        virtual_properties=parse_one("(labels = [('test-virtual-label', 'label-virtual-value')],)"),
+    )
+    assert "labels" in updated_model.virtual_properties
+    updated_fingerprint = fingerprint_from_node(updated_model, nodes={})
+
+    assert updated_fingerprint != fingerprint
+    assert updated_fingerprint.metadata_hash != fingerprint.metadata_hash
+    assert updated_fingerprint.data_hash == fingerprint.data_hash
+
+
 def test_stamp(model: Model):
     original_fingerprint = fingerprint_from_node(model, nodes={})
 
@@ -1601,6 +1618,20 @@ def test_deployability_index_uncategorized_forward_only_model(make_snapshot):
     assert not deplyability_index.is_representative(snapshot_b)
 
 
+def test_deployability_index_missing_parent(make_snapshot):
+    snapshot_a = make_snapshot(SqlModel(name="a", query=parse_one("SELECT 1")))
+    snapshot_a.categorize_as(SnapshotChangeCategory.BREAKING)
+
+    snapshot_b = make_snapshot(SqlModel(name="b", query=parse_one("SELECT 1")))
+    snapshot_b.categorize_as(SnapshotChangeCategory.FORWARD_ONLY)
+    snapshot_b.parents = (snapshot_a.snapshot_id,)
+
+    deplyability_index = DeployabilityIndex.create({snapshot_b.snapshot_id: snapshot_b})
+
+    assert not deplyability_index.is_deployable(snapshot_b)
+    assert not deplyability_index.is_deployable(snapshot_a)
+
+
 @pytest.mark.parametrize(
     "model_name, environment_naming_info, default_catalog, expected",
     (
@@ -1715,3 +1746,10 @@ def test_missing_intervals_node_start_end(make_snapshot):
     assert missing_intervals([snapshot], start="2024-03-01", end=to_datetime("2024-03-10")) == {}
     assert missing_intervals([snapshot], start="2024-03-13", end="2024-03-14") == {}
     assert missing_intervals([snapshot], start="2024-03-14", end="2024-03-30") == {}
+
+
+def test_external_model_audits(sushi_context):
+    snapshot = sushi_context.get_snapshot("raw.demographics")
+    assert snapshot.evaluatable
+    assert len(snapshot.model.audits) == 2
+    assert snapshot.intervals
