@@ -1,12 +1,15 @@
 import typing as t
-from unittest.mock import call
+from unittest.mock import call, patch
 
+import logging
 import pytest
+from pathlib import Path
 from pytest_mock.plugin import MockerFixture
 from sqlglot import expressions as exp
 from sqlglot import parse, parse_one, select
 
 from sqlmesh.core.audit import ModelAudit, StandaloneAudit
+from sqlmesh.core import dialect as d
 from sqlmesh.core.dialect import schema_, to_schema
 from sqlmesh.core.engine_adapter import EngineAdapter, create_engine_adapter
 from sqlmesh.core.engine_adapter.base import MERGE_SOURCE_ALIAS, MERGE_TARGET_ALIAS
@@ -21,12 +24,14 @@ from sqlmesh.core.model import (
     FullKind,
     IncrementalByTimeRangeKind,
     IncrementalUnmanagedKind,
+    IncrementalByPartitionKind,
     PythonModel,
     SqlModel,
     TimeColumn,
     ViewKind,
     load_sql_based_model,
 )
+from sqlmesh.core.model.kind import OnDestructiveChange
 from sqlmesh.core.node import IntervalUnit
 from sqlmesh.core.snapshot import (
     DeployabilityIndex,
@@ -36,6 +41,7 @@ from sqlmesh.core.snapshot import (
     SnapshotEvaluator,
     SnapshotTableCleanupTask,
 )
+from sqlmesh.utils.concurrency import NodeExecutionFailedError
 from sqlmesh.utils.date import to_timestamp
 from sqlmesh.utils.errors import ConfigError
 from sqlmesh.utils.metaprogramming import Executable
@@ -258,6 +264,7 @@ def test_promote(mocker: MockerFixture, adapter_mock, make_snapshot):
         ),
         table_description=None,
         column_descriptions=None,
+        view_properties={},
     )
 
 
@@ -307,6 +314,7 @@ def test_promote_default_catalog(adapter_mock, make_snapshot):
         ),
         table_description=None,
         column_descriptions=None,
+        view_properties={},
     )
 
 
@@ -351,6 +359,7 @@ def test_promote_forward_only(mocker: MockerFixture, adapter_mock, make_snapshot
                 ),
                 table_description=None,
                 column_descriptions=None,
+                view_properties={},
             ),
             call(
                 "test_schema__test_env.test_model",
@@ -359,6 +368,7 @@ def test_promote_forward_only(mocker: MockerFixture, adapter_mock, make_snapshot
                 ),
                 table_description=None,
                 column_descriptions=None,
+                view_properties={},
             ),
         ]
     )
@@ -459,7 +469,7 @@ def test_evaluate_materialized_view(
             model.render_query(),
             model.columns_to_types,
             materialized=True,
-            table_properties={},
+            view_properties={},
             table_description=None,
             column_descriptions={},
         )
@@ -501,7 +511,7 @@ def test_evaluate_materialized_view_with_execution_time_macro(
         model.render_query(execution_time="2020-01-02"),
         model.columns_to_types,
         materialized=True,
-        table_properties={},
+        view_properties={},
         table_description=None,
         column_descriptions={},
     )
@@ -653,7 +663,7 @@ def test_create_only_dev_table_exists(mocker: MockerFixture, adapter_mock, make_
 
     common_kwargs = dict(
         materialized=False,
-        table_properties={},
+        view_properties={},
         table_description=None,
     )
     rendered_query = model.render_query()
@@ -703,7 +713,7 @@ def test_create_materialized_view(mocker: MockerFixture, adapter_mock, make_snap
 
     common_kwargs = dict(
         materialized=True,
-        table_properties={},
+        view_properties={},
         table_description=None,
     )
 
@@ -733,7 +743,7 @@ def test_create_view_with_properties(mocker: MockerFixture, adapter_mock, make_s
                 kind VIEW (
                     materialized true
                 ),
-                table_properties (
+                physical_properties (
                     "key" = 'value'
                 )
             );
@@ -750,7 +760,7 @@ def test_create_view_with_properties(mocker: MockerFixture, adapter_mock, make_s
 
     common_kwargs = dict(
         materialized=True,
-        table_properties={
+        view_properties={
             "key": exp.convert("value"),
         },
         table_description=None,
@@ -795,6 +805,7 @@ def test_promote_model_info(mocker: MockerFixture, make_snapshot):
         parse_one(f"SELECT * FROM physical_schema.test_schema__test_model__{snapshot.version}"),
         table_description=None,
         column_descriptions=None,
+        view_properties={},
     )
 
 
@@ -824,7 +835,9 @@ def test_migrate(mocker: MockerFixture, make_snapshot):
 
     model = SqlModel(
         name="test_schema.test_model",
-        kind=IncrementalByTimeRangeKind(time_column="a"),
+        kind=IncrementalByTimeRangeKind(
+            time_column="a", on_destructive_change=OnDestructiveChange.ALLOW
+        ),
         storage_format="parquet",
         query=parse_one("SELECT c, a FROM tbl WHERE ds BETWEEN @start_ds and @end_ds"),
     )
@@ -1060,6 +1073,7 @@ def python_func(**kwargs):
 
 def test_create_clone_in_dev(mocker: MockerFixture, adapter_mock, make_snapshot):
     adapter_mock.SUPPORTS_CLONING = True
+    adapter_mock.get_alter_expressions.return_value = []
     evaluator = SnapshotEvaluator(adapter_mock)
 
     model = load_sql_based_model(
@@ -1101,16 +1115,89 @@ def test_create_clone_in_dev(mocker: MockerFixture, adapter_mock, make_snapshot)
         replace=True,
     )
 
-    adapter_mock.alter_table.assert_called_once_with(
+    adapter_mock.get_alter_expressions.assert_called_once_with(
         f"sqlmesh__test_schema.test_schema__test_model__{snapshot.version}__temp",
         f"sqlmesh__test_schema.test_schema__test_model__{snapshot.version}__temp__schema_migration_source",
     )
 
+    adapter_mock.alter_table.assert_called_once_with([])
+
     adapter_mock.drop_table.assert_called_once_with(
         f"sqlmesh__test_schema.test_schema__test_model__{snapshot.version}__temp__schema_migration_source"
     )
 
 
+def test_on_destructive_change_runtime_check(
+    mocker: MockerFixture,
+    make_snapshot,
+):
+    connection_mock = mocker.NonCallableMock()
+    cursor_mock = mocker.Mock()
+    connection_mock.cursor.return_value = cursor_mock
+    adapter = EngineAdapter(lambda: connection_mock, "")
+
+    current_table = "sqlmesh__test_schema.test_schema__test_model__1"
+
+    def columns(table_name):
+        if table_name == current_table:
+            return {
+                "c": exp.DataType.build("int"),
+                "b": exp.DataType.build("int"),
+            }
+        else:
+            return {
+                "c": exp.DataType.build("int"),
+                "a": exp.DataType.build("int"),
+            }
+
+    adapter.columns = columns  # type: ignore
+
+    evaluator = SnapshotEvaluator(adapter)
+
+    # SQLMesh default: ERROR
+    model = SqlModel(
+        name="test_schema.test_model",
+        kind=IncrementalByTimeRangeKind(time_column="a"),
+        query=parse_one("SELECT c, a FROM tbl WHERE ds BETWEEN @start_ds and @end_ds"),
+    )
+    snapshot = make_snapshot(model, version="1")
+    snapshot.change_category = SnapshotChangeCategory.FORWARD_ONLY
+
+    with pytest.raises(
+        NodeExecutionFailedError,
+        match="""Execution failed for node SnapshotId<"test_schema"."test_model""",
+    ):
+        with pytest.raises(
+            RuntimeError,
+            match="""Plan results in a destructive change to forward-only table '"test_schema"."test_model"'s schema.""",
+        ):
+            evaluator.migrate([snapshot], {})
+
+    # WARN
+    model = SqlModel(
+        name="test_schema.test_model",
+        kind=IncrementalByTimeRangeKind(
+            time_column="a", on_destructive_change=OnDestructiveChange.WARN
+        ),
+        query=parse_one("SELECT c, a FROM tbl WHERE ds BETWEEN @start_ds and @end_ds"),
+    )
+    snapshot = make_snapshot(model, version="1")
+    snapshot.change_category = SnapshotChangeCategory.FORWARD_ONLY
+
+    logger = logging.getLogger("sqlmesh.core.snapshot.evaluator")
+    with patch.object(logger, "warning") as mock_logger:
+        evaluator.migrate([snapshot], {})
+        assert (
+            mock_logger.call_args[0][0]
+            == """Plan results in a destructive change to forward-only table '"test_schema"."test_model"'s schema."""
+        )
+
+    # allow destructive
+    with patch.object(logger, "warning") as mock_logger:
+        evaluator.migrate([snapshot], {}, {'"test_schema"."test_model"'})
+        assert mock_logger.call_count == 0
+
+
 def test_forward_only_snapshot_for_added_model(mocker: MockerFixture, adapter_mock, make_snapshot):
     adapter_mock.SUPPORTS_CLONING = False
     evaluator = SnapshotEvaluator(adapter_mock)
@@ -1589,6 +1676,77 @@ def test_create_incremental_by_unique_no_intervals(adapter_mock, make_snapshot):
     )
 
 
+def test_create_seed(mocker: MockerFixture, adapter_mock, make_snapshot):
+    expressions = d.parse(
+        """
+        MODEL (
+            name db.seed,
+            kind SEED (
+              path '../seeds/waiter_names.csv',
+              batch_size 5,
+            )
+        );
+    """
+    )
+
+    model = load_sql_based_model(expressions, path=Path("./examples/sushi/models/test_model.sql"))
+
+    snapshot = make_snapshot(model)
+    snapshot.categorize_as(SnapshotChangeCategory.BREAKING)
+
+    evaluator = SnapshotEvaluator(adapter_mock)
+    evaluator.create([snapshot], {})
+
+    common_create_kwargs: t.Dict[str, t.Any] = dict(
+        columns_to_types={"id": exp.DataType.build("bigint"), "name": exp.DataType.build("text")},
+        storage_format=None,
+        partitioned_by=[],
+        partition_interval_unit=IntervalUnit.DAY,
+        clustered_by=[],
+        table_properties={},
+        table_description=None,
+    )
+
+    adapter_mock.create_table.assert_has_calls(
+        [
+            call(
+                f"sqlmesh__db.db__seed__{snapshot.version}__temp",
+                column_descriptions=None,
+                **common_create_kwargs,
+            ),
+            call(
+                f"sqlmesh__db.db__seed__{snapshot.version}",
+                column_descriptions={},
+                **common_create_kwargs,
+            ),
+        ]
+    )
+
+    replace_query_calls = adapter_mock.replace_query.call_args_list
+    assert len(replace_query_calls) == 1
+
+    df = replace_query_calls[0][0][1]
+    assert list(df.itertuples(index=False)) == [
+        (0, "Toby"),
+        (1, "Tyson"),
+        (2, "Ryan"),
+        (3, "George"),
+        (4, "Chris"),
+    ]
+
+    insert_append_calls = adapter_mock.insert_append.call_args_list
+    assert len(insert_append_calls) == 1
+
+    df = insert_append_calls[0][0][1]
+    assert list(df.itertuples(index=False)) == [
+        (5, "Max"),
+        (6, "Vincent"),
+        (7, "Iaroslav"),
+        (8, "Emma"),
+        (9, "Maia"),
+    ]
+
+
 def test_standalone_audit(mocker: MockerFixture, adapter_mock, make_snapshot):
     evaluator = SnapshotEvaluator(adapter_mock)
 
@@ -1695,7 +1853,7 @@ def test_audit_wap(adapter_mock, make_snapshot):
     not_null_query = call_args[0][0][0]
     assert (
         not_null_query.sql(dialect="spark")
-        == "SELECT COUNT(*) FROM (SELECT * FROM (SELECT * FROM `spark_catalog`.`test_schema`.`test_table`.`branch_wap_test_wap_id` AS `branch_wap_test_wap_id`) AS `_q_0` WHERE `a` IS NULL) AS audit"
+        == "SELECT COUNT(*) FROM (SELECT * FROM (SELECT * FROM `spark_catalog`.`test_schema`.`test_table`.`branch_wap_test_wap_id` AS `branch_wap_test_wap_id`) AS `_q_0` WHERE `a` IS NULL AND TRUE) AS audit"
     )
 
     custom_audit_query = call_args[1][0][0]
@@ -1746,3 +1904,33 @@ def test_create_post_statements_use_deployable_table(
     post_calls = call_args[1][0][0]
     assert len(post_calls) == 1
     assert post_calls[0].sql(dialect="postgres") == expected_call
+
+
+def test_evaluate_incremental_by_partition(mocker: MockerFixture, make_snapshot, adapter_mock):
+    model = SqlModel(
+        name="test_schema.test_model",
+        query=parse_one("SELECT 1, ds, b FROM tbl_a"),
+        kind=IncrementalByPartitionKind(),
+        partitioned_by=["ds", "b"],
+    )
+    snapshot = make_snapshot(model)
+    snapshot.categorize_as(SnapshotChangeCategory.BREAKING)
+
+    evaluator = SnapshotEvaluator(adapter_mock)
+    evaluator.evaluate(
+        snapshot,
+        start="2020-01-01",
+        end="2020-01-02",
+        execution_time="2020-01-02",
+        snapshots={},
+    )
+
+    adapter_mock.insert_overwrite_by_partition.assert_called_once_with(
+        snapshot.table_name(),
+        model.render_query(),
+        partitioned_by=[
+            exp.to_column("ds", quoted=True),
+            exp.to_column("b", quoted=True),
+        ],
+        columns_to_types=model.columns_to_types,
+    )
