@@ -215,7 +215,6 @@ def test_snapshots_exists(state_sync: EngineAdapterStateSync, snapshots: t.List[
 
 @pytest.fixture
 def get_snapshot_intervals(state_sync) -> t.Callable[[Snapshot], t.Optional[SnapshotIntervals]]:
-
     def _get_snapshot_intervals(snapshot: Snapshot) -> t.Optional[SnapshotIntervals]:
         intervals = state_sync._get_snapshot_intervals([snapshot])[-1]
         return intervals[0] if intervals else None
@@ -479,6 +478,36 @@ def test_compact_intervals(
     assert get_snapshot_intervals(snapshot).intervals == expected_intervals
 
 
+def test_compact_intervals_delete_batches(
+    state_sync: EngineAdapterStateSync,
+    make_snapshot: t.Callable,
+    mocker: MockerFixture,
+) -> None:
+    snapshot = make_snapshot(
+        SqlModel(
+            name="a",
+            cron="@daily",
+            query=parse_one("select 1, ds"),
+        ),
+        version="a",
+    )
+
+    delete_from_mock = mocker.patch.object(state_sync.engine_adapter, "delete_from")
+    state_sync.INTERVAL_BATCH_SIZE = 2
+
+    state_sync.push_snapshots([snapshot])
+
+    state_sync.add_interval(snapshot, "2020-01-01", "2020-01-11")
+    state_sync.add_interval(snapshot, "2020-01-01", "2020-01-12")
+    state_sync.add_interval(snapshot, "2020-01-01", "2020-01-13")
+    state_sync.add_interval(snapshot, "2020-01-01", "2020-01-14")
+    state_sync.add_interval(snapshot, "2020-01-01", "2020-01-15")
+
+    state_sync.compact_intervals()
+
+    delete_from_mock.assert_has_calls([call(state_sync.intervals_table, mocker.ANY)] * 3)
+
+
 def test_promote_snapshots(state_sync: EngineAdapterStateSync, make_snapshot: t.Callable):
     snapshot_a = make_snapshot(
         SqlModel(
@@ -1065,18 +1094,12 @@ def test_delete_expired_snapshots_seed(
 
     state_sync.push_snapshots([snapshot])
     assert set(state_sync.get_snapshots(None)) == {snapshot.snapshot_id}
-    assert state_sync.engine_adapter.fetchall(
-        "SELECT name, version, content FROM sqlmesh._seeds"
-    ) == [
-        (snapshot.name, snapshot.version, snapshot.model.seed.content),
-    ]
 
     assert state_sync.delete_expired_snapshots() == [
         SnapshotTableCleanupTask(snapshot=snapshot.table_info, dev_table_only=False),
     ]
 
     assert not state_sync.get_snapshots(None)
-    assert not state_sync.engine_adapter.fetchall("SELECT * FROM sqlmesh._seeds")
 
 
 def test_delete_expired_snapshots_batching(
@@ -1450,7 +1473,12 @@ def test_version_sqlmesh(state_sync: EngineAdapterStateSync) -> None:
 
     # patch version sqlmesh doesn't matter
     major, minor, patch, *_ = SQLMESH_VERSION_TUPLE
-    sqlmesh_version_patch_bump = f"{major}.{minor}.{int(patch) + 1}"
+    new_patch = (
+        f"dev{int(patch[3:]) + 1}"  # type: ignore
+        if isinstance(patch, str) and patch.startswith("dev")
+        else f"{int(patch) + 1}"
+    )
+    sqlmesh_version_patch_bump = f"{major}.{minor}.{new_patch}"
     state_sync._update_versions(sqlmesh_version=sqlmesh_version_patch_bump)
     state_sync.get_versions(validate=False)
 
@@ -1584,7 +1612,6 @@ def test_first_migration_failure(duck_conn, mocker: MockerFixture, tmp_path) ->
     assert not state_sync.engine_adapter.table_exists(state_sync.snapshots_table)
     assert not state_sync.engine_adapter.table_exists(state_sync.environments_table)
     assert not state_sync.engine_adapter.table_exists(state_sync.versions_table)
-    assert not state_sync.engine_adapter.table_exists(state_sync.seeds_table)
     assert not state_sync.engine_adapter.table_exists(state_sync.intervals_table)
 
 
@@ -1737,20 +1764,11 @@ def test_seed_hydration(
     assert snapshot.model.is_hydrated
     assert snapshot.model.seed.content == "header\n1\n2"
 
-    stored_snapshot = state_sync.get_snapshots([snapshot.snapshot_id], hydrate_seeds=False)[
-        snapshot.snapshot_id
-    ]
+    stored_snapshot = state_sync.get_snapshots([snapshot.snapshot_id])[snapshot.snapshot_id]
     assert isinstance(stored_snapshot.model, SeedModel)
     assert not stored_snapshot.model.is_hydrated
     assert stored_snapshot.model.seed.content == ""
 
-    stored_snapshot = state_sync.get_snapshots([snapshot.snapshot_id], hydrate_seeds=True)[
-        snapshot.snapshot_id
-    ]
-    assert isinstance(stored_snapshot.model, SeedModel)
-    assert stored_snapshot.model.is_hydrated
-    assert stored_snapshot.model.seed.content == "header\n1\n2"
-
 
 def test_nodes_exist(state_sync: EngineAdapterStateSync, make_snapshot: t.Callable):
     snapshot = make_snapshot(
@@ -1855,7 +1873,7 @@ def test_cache(state_sync, make_snapshot, mocker):
     now_timestamp.return_value = to_timestamp("2023-01-01 00:00:05")
     with patch.object(state_sync, "get_snapshots") as mock:
         assert cache.get_snapshots([snapshot.snapshot_id])
-        mock.not_called()
+        mock.assert_not_called()
 
     # no cache hit
     now_timestamp.return_value = to_timestamp("2023-01-01 00:00:11")
@@ -2194,6 +2212,7 @@ def test_snapshot_batching(state_sync, mocker, make_snapshot):
                 ).json(),
                 "a",
                 "1",
+                "1",
             ],
             [
                 make_snapshot(
@@ -2201,6 +2220,7 @@ def test_snapshot_batching(state_sync, mocker, make_snapshot):
                 ).json(),
                 "a",
                 "2",
+                "2",
             ],
         ],
         [
@@ -2210,6 +2230,7 @@ def test_snapshot_batching(state_sync, mocker, make_snapshot):
                 ).json(),
                 "a",
                 "3",
+                "3",
             ],
         ],
     ]
@@ -2243,14 +2264,6 @@ def test_seed_model_metadata_update(
 
     state_sync.push_snapshots([snapshot])
 
-    seed_query = (
-        exp.select("COUNT(*)")
-        .from_("sqlmesh._seeds")
-        .where(exp.column("version").eq(snapshot.version))
-    )
-
-    assert state_sync.engine_adapter.fetchone(seed_query)[0] == 1
-
     model = model.copy(update={"owner": "jen"})
     new_snapshot = make_snapshot(model)
     new_snapshot.previous_versions = snapshot.all_versions
@@ -2260,5 +2273,4 @@ def test_seed_model_metadata_update(
     assert snapshot.version == new_snapshot.version
 
     state_sync.push_snapshots([new_snapshot])
-    assert state_sync.engine_adapter.fetchone(seed_query)[0] == 1
     assert len(state_sync.get_snapshots([new_snapshot, snapshot])) == 2
