@@ -12,12 +12,21 @@ from sqlglot import exp
 
 from sqlmesh.cli.example_project import init_example_project
 from sqlmesh.core import constants as c
-from sqlmesh.core.config import Config, DuckDBConnectionConfig, ModelDefaultsConfig
+from sqlmesh.core.config import (
+    Config,
+    DuckDBConnectionConfig,
+    SparkConnectionConfig,
+    GatewayConfig,
+    ModelDefaultsConfig,
+)
 from sqlmesh.core.context import Context
 from sqlmesh.core.dialect import parse
+from sqlmesh.core.engine_adapter import EngineAdapter
+from sqlmesh.core.macros import MacroEvaluator, macro
 from sqlmesh.core.model import Model, SqlModel, load_sql_based_model, model
 from sqlmesh.core.test.definition import ModelTest, PythonModelTest, SqlModelTest
-from sqlmesh.utils.errors import ConfigError
+from sqlmesh.utils.errors import ConfigError, TestError
+from sqlmesh.utils.yaml import dump as dump_yaml
 from sqlmesh.utils.yaml import load as load_yaml
 
 if t.TYPE_CHECKING:
@@ -37,7 +46,9 @@ def _create_test(
         test_name=test_name,
         model=model,
         models=context._models,
-        engine_adapter=context._test_engine_adapter,
+        engine_adapter=context._test_connection_config.create_engine_adapter(
+            register_comments_override=False
+        ),
         dialect=context.config.dialect,
         path=None,
         default_catalog=context.default_catalog,
@@ -276,9 +287,7 @@ test_foo:
 
 
 def test_row_order(sushi_context: Context, full_model_without_ctes: SqlModel) -> None:
-    model = t.cast(SqlModel, sushi_context.upsert_model(full_model_without_ctes))
-
-    # input and output rows are in different orders
+    # Input and output rows are in different orders
     body = load_yaml(
         """
 test_foo:
@@ -305,42 +314,66 @@ test_foo:
         """
     )
 
-    # model query without ORDER BY should pass unit test
-    result = _create_test(body, "test_foo", model, sushi_context).run()
-    _check_successful_or_raise(result)
+    # Model query without ORDER BY should pass unit test
+    _check_successful_or_raise(
+        _create_test(
+            body=body,
+            test_name="test_foo",
+            model=sushi_context.upsert_model(full_model_without_ctes),
+            context=sushi_context,
+        ).run()
+    )
 
-    # model query with ORDER BY should fail unit test
     full_model_without_ctes_dict = full_model_without_ctes.dict()
     full_model_without_ctes_dict["query"] = full_model_without_ctes.query.order_by("id")  # type: ignore
     full_model_without_ctes_orderby = SqlModel(**full_model_without_ctes_dict)
 
-    model = t.cast(SqlModel, sushi_context.upsert_model(full_model_without_ctes_orderby))
-    result = _create_test(body, "test_foo", model, sushi_context).run()
-
-    expected_failure_msg = (
-        "AssertionError: Data mismatch (exp: expected, act: actual)\n\n"
-        "   id     value      ds    \n"
-        "  exp act   exp act exp act\n"
-        "0   2   1     3   2   4   3\n"
-        "1   1   2     2   3   3   4\n"
+    # Model query with ORDER BY should fail unit test
+    _check_successful_or_raise(
+        _create_test(
+            body=body,
+            test_name="test_foo",
+            model=sushi_context.upsert_model(full_model_without_ctes_orderby),
+            context=sushi_context,
+        ).run(),
+        expected_msg=(
+            "AssertionError: Data mismatch (exp: expected, act: actual)\n\n"
+            "   id     value      ds    \n"
+            "  exp act   exp act exp act\n"
+            "0   2   1     3   2   4   3\n"
+            "1   1   2     2   3   3   4\n"
+        ),
     )
-    _check_successful_or_raise(result, expected_msg=expected_failure_msg)
 
 
-def test_partial_data(sushi_context: Context) -> None:
+@pytest.mark.parametrize(
+    "waiter_names_input",
+    [
+        """sushi.waiter_names:
+          - id: 1
+          - id: 2
+            name: null
+          - id: 3
+            name: 'bob'
+        """,
+        """sushi.waiter_names: 
+      format: csv
+      rows: |
+        id,name
+        1,
+        2,null
+        3,bob""",
+    ],
+)
+def test_partial_data(sushi_context: Context, waiter_names_input: str) -> None:
     _check_successful_or_raise(
         _create_test(
             body=load_yaml(
-                """
+                f"""
 test_foo:
   model: sushi.foo
   inputs:
-    sushi.waiter_names:
-      - id: 1
-      - id: 2
-        name: null
-      - id: 3
-        name: 'bob'
+    {waiter_names_input}
   outputs:
     ctes:
       source:
@@ -372,6 +405,203 @@ test_foo:
     )
 
 
+@pytest.mark.parametrize(
+    "waiter_names_input",
+    [
+        """sushi.waiter_names:
+        format: yaml
+        rows: 
+        - id: 1
+          name: alice
+        - id: 2
+          name: 'bob'
+        """,
+        """sushi.waiter_names: 
+      format: csv
+      rows: |
+        id,name
+        1,alice
+        2,bob""",
+        """sushi.waiter_names: 
+      format: csv
+      csv_settings: 
+        sep: "#"
+      rows: |
+        id#name
+        1#alice
+        2#bob""",
+    ],
+)
+def test_format_inline(sushi_context: Context, waiter_names_input: str) -> None:
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                f"""
+test_foo:
+  model: sushi.foo
+  inputs:
+    {waiter_names_input}
+  outputs:
+    query:
+      - id: 1
+        name: alice
+      - id: 2
+        name: 'bob'
+                """
+            ),
+            test_name="test_foo",
+            model=sushi_context.upsert_model(
+                _create_model(
+                    "SELECT id, name FROM sushi.waiter_names ",
+                    default_catalog=sushi_context.default_catalog,
+                )
+            ),
+            context=sushi_context,
+        ).run()
+    )
+
+
+@pytest.mark.parametrize(
+    ["input_data", "filename", "file_data"],
+    [
+        [
+            """sushi.waiter_names:
+        format: yaml
+        path: """,
+            "test_data.yaml",
+            """- id: 1
+  name: alice
+- id: 2
+  name: 'bob'
+""",
+        ],
+        [
+            """sushi.waiter_names:
+        path: """,
+            "test_data.yaml",
+            """rows:
+- id: 1
+  name: alice
+- id: 2
+  name: 'bob'
+""",
+        ],
+        [
+            """sushi.waiter_names:
+        format: csv
+        path: """,
+            "test_data.csv",
+            """id,name
+1,alice
+2,bob""",
+        ],
+    ],
+)
+def test_format_path(
+    sushi_context: Context, tmp_path: Path, input_data: str, filename: str, file_data: str
+) -> None:
+    test_csv_file = tmp_path / filename
+    test_csv_file.write_text(file_data)
+
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                f"""
+test_foo:
+  model: sushi.foo
+  inputs:
+    {input_data}{str(test_csv_file)}
+  outputs:
+    query:
+      - id: 1
+        name: alice
+      - id: 2
+        name: 'bob'
+                """
+            ),
+            test_name="test_foo",
+            model=sushi_context.upsert_model(
+                _create_model(
+                    "SELECT id, name FROM sushi.waiter_names ",
+                    default_catalog=sushi_context.default_catalog,
+                )
+            ),
+            context=sushi_context,
+        ).run()
+    )
+
+
+def test_unsupported_format_failure(
+    sushi_context: Context, full_model_without_ctes: SqlModel
+) -> None:
+    with pytest.raises(
+        TestError,
+        match="Unsupported data format 'xml' for 'sushi.waiter_names'",
+    ):
+        _create_test(
+            body=load_yaml(
+                """
+test_foo:
+  model: sushi.foo
+  description: XML format isn't supported to load data (fails intentionally)
+  inputs:
+    sushi.waiter_names:
+      format: xml
+      path: 'test_data.xml'
+  outputs:
+    query:
+      - id: 1
+        value: null
+                """
+            ),
+            test_name="test_foo",
+            model=sushi_context.upsert_model(full_model_without_ctes),
+            context=sushi_context,
+        )
+
+    with pytest.raises(
+        TestError,
+        match="Unsupported data format 'xml' for 'sushi.waiter_names'",
+    ):
+        _create_test(
+            body=load_yaml(
+                """
+test_foo:
+  model: sushi.foo
+  description: XML without path doesn't raise error
+  inputs:
+    sushi.waiter_names:
+      format: xml
+      rows: |
+        <rows>
+          <row>
+              <id>1</id>
+              <name>alice</name>
+          </row>
+          <row>
+            <id>2</id>
+            <name>bob</name>
+          </row>
+        </rows>
+  outputs:
+    query:
+      - id: 1
+        name: alice
+      - id: 2
+        name: 'bob'
+                """
+            ),
+            test_name="test_foo",
+            model=sushi_context.upsert_model(
+                _create_model(
+                    "SELECT id, name FROM sushi.waiter_names ",
+                    default_catalog=sushi_context.default_catalog,
+                )
+            ),
+            context=sushi_context,
+        )
+
+
 def test_partial_output_columns() -> None:
     _check_successful_or_raise(
         _create_test(
@@ -575,14 +805,15 @@ test_child:
         d: 2020-01-01
         """
     )
+
+    mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
     test = _create_test(body, "test_child", child, sushi_context)
-    test._test_schema = "sqlmesh_test_jzngz56a"
 
     spy_execute = mocker.spy(test.engine_adapter, "_execute")
     _check_successful_or_raise(test.run())
 
     spy_execute.assert_any_call(
-        f'CREATE OR REPLACE VIEW "memory"."sqlmesh_test_jzngz56a"."memory__sushi__parent" ("s", "a", "b") AS '
+        'CREATE OR REPLACE VIEW "memory"."sqlmesh_test_jzngz56a"."memory__sushi__parent" ("s", "a", "b") AS '
         "SELECT "
         'CAST("s" AS STRUCT("d" DATE)) AS "s", '
         'CAST("a" AS INT) AS "a", '
@@ -613,7 +844,7 @@ test_foo:
         expected_msg=(
             """Failed to infer the data type of column 'value' for '"raw"'. This issue can """
             "be mitigated by casting the column in the model definition, setting its type in "
-            "schema.yaml if it's an external model, setting the model's 'columns' property, "
+            "external_models.yaml if it's an external model, setting the model's 'columns' property, "
             "or setting its 'columns' mapping in the test itself\n"
         ),
     )
@@ -791,6 +1022,29 @@ test_foo:
         ).run()
     )
 
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_a:
+  model: a
+  inputs:
+    b:
+      columns:
+        x: "array(int)"
+      rows: []
+  outputs:
+    query: []
+                """
+            ),
+            test_name="test_a",
+            model=sushi_context.upsert_model(
+                _create_model("SELECT x FROM b", default_catalog="memory")
+            ),
+            context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+        ).run()
+    )
+
 
 @pytest.mark.parametrize("full_model_without_ctes", ["snowflake"], indirect=True)
 def test_normalization(full_model_without_ctes: SqlModel) -> None:
@@ -895,6 +1149,7 @@ test_foo:
 
 
 def test_freeze_time(mocker: MockerFixture) -> None:
+    mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
     test = _create_test(
         body=load_yaml(
             """
@@ -915,20 +1170,20 @@ test_foo:
         ),
         context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
     )
-    test._test_schema = f"sqlmesh_test_jzngz56a"
 
     spy_execute = mocker.spy(test.engine_adapter, "_execute")
     _check_successful_or_raise(test.run())
 
     spy_execute.assert_has_calls(
         [
+            call('CREATE SCHEMA IF NOT EXISTS "memory"."sqlmesh_test_jzngz56a"'),
             call(
                 "SELECT "
                 """CAST('2023-01-01 12:05:03+00:00' AS DATE) AS "cur_date", """
                 """CAST('2023-01-01 12:05:03+00:00' AS TIME) AS "cur_time", """
                 '''CAST('2023-01-01 12:05:03+00:00' AS TIMESTAMP) AS "cur_timestamp"'''
             ),
-            call('DROP SCHEMA IF EXISTS "sqlmesh_test_jzngz56a" CASCADE'),
+            call('DROP SCHEMA IF EXISTS "memory"."sqlmesh_test_jzngz56a" CASCADE'),
         ]
     )
 
@@ -970,7 +1225,8 @@ def test_successes(sushi_context: Context) -> None:
     assert "test_customer_revenue_by_day" in successful_tests
 
 
-def test_create_external_model_fixture(sushi_context: Context) -> None:
+def test_create_external_model_fixture(sushi_context: Context, mocker: MockerFixture) -> None:
+    mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
     test = _create_test(
         body=load_yaml(
             """
@@ -988,13 +1244,209 @@ test_foo:
         model=_create_model("SELECT x FROM c.db.external"),
         context=sushi_context,
     )
-    test._test_schema = f"sqlmesh_test_jzngz56a"
     _check_successful_or_raise(test.run())
 
-    assert len(test._fixture_table_cache) == len(sushi_context.models) + 1
+    assert len(test._fixture_table_cache) == 1
     for table in test._fixture_table_cache.values():
         assert table.catalog == "memory"
-        assert table.db == f"sqlmesh_test_jzngz56a"
+        assert table.db == "sqlmesh_test_jzngz56a"
+
+
+def test_runtime_stage() -> None:
+    @macro()
+    def test_macro(evaluator: MacroEvaluator) -> t.List[bool]:
+        return [evaluator.runtime_stage == "testing", evaluator.runtime_stage == "loading"]
+
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_foo:
+  model: foo
+  outputs:
+    query:
+      - c: [true, false]
+                """
+            ),
+            test_name="test_foo",
+            model=_create_model("SELECT [@test_macro()] AS c"),
+            context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+        ).run()
+    )
+
+
+def test_gateway(copy_to_temp_path: t.Callable, mocker: MockerFixture) -> None:
+    path = Path(copy_to_temp_path("examples/sushi")[0])
+    db_db_path = str(path / "db.db")
+    test_db_path = str(path / "test.db")
+
+    config = Config(
+        gateways={
+            "main": GatewayConfig(connection=DuckDBConnectionConfig(database=db_db_path)),
+            "test": GatewayConfig(test_connection=DuckDBConnectionConfig(database=test_db_path)),
+        },
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+
+    context = Context(paths=path, config=config)
+
+    test_path = path / c.TESTS / "test_customer_revenue_by_day.yaml"
+    test_dict = load_yaml(test_path)
+    test_dict["test_customer_revenue_by_day"]["gateway"] = "test"
+
+    with open(test_path, "w", encoding="utf-8") as file:
+        dump_yaml(test_dict, file)
+
+    spy_execute = mocker.spy(EngineAdapter, "_execute")
+    mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
+
+    result = context.test(tests=[f"{test_path}::test_customer_revenue_by_day"])
+    _check_successful_or_raise(result)
+
+    expected_view_sql = (
+        'CREATE OR REPLACE VIEW "test"."sqlmesh_test_jzngz56a"."db__sushi__orders" '
+        '("id", "customer_id", "waiter_id", "start_ts", "end_ts", "event_date") AS '
+        "SELECT "
+        'CAST("id" AS INT) AS "id", '
+        'CAST("customer_id" AS INT) AS "customer_id", '
+        'CAST("waiter_id" AS INT) AS "waiter_id", '
+        'CAST("start_ts" AS INT) AS "start_ts", '
+        'CAST("end_ts" AS INT) AS "end_ts", '
+        'CAST("event_date" AS DATE) AS "event_date" '
+        "FROM (VALUES "
+        "(1, 1, 1, 1641002340, 1641004140, CAST('2022-01-01' AS DATE)), "
+        "(2, 1, 2, 1641007740, 1641009540, CAST('2022-01-01' AS DATE))) "
+        'AS "t"("id", "customer_id", "waiter_id", "start_ts", "end_ts", "event_date")'
+    )
+    test_adapter = t.cast(ModelTest, result.successes[0]).engine_adapter
+    assert call(test_adapter, expected_view_sql) in spy_execute.mock_calls
+
+    _check_successful_or_raise(context.test())
+
+
+def test_generate_input_data_using_sql(mocker: MockerFixture, tmp_path: Path) -> None:
+    init_example_project(tmp_path, dialect="duckdb")
+    config = Config(
+        default_connection=DuckDBConnectionConfig(),
+        model_defaults=ModelDefaultsConfig(dialect="duckdb"),
+    )
+    context = Context(paths=tmp_path, config=config)
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_example_full_model_alt:
+  model: sqlmesh_example.full_model
+  inputs:
+    sqlmesh_example.incremental_model:
+      query: |
+        SELECT 1 AS id, 1 AS item_id
+        UNION ALL
+        SELECT 2 AS id, 1 AS item_id
+        UNION ALL
+        SELECT 3 AS id, 2 AS item_id
+  outputs:
+    query:
+      rows:
+      - item_id: 1
+        num_orders: 2
+      - item_id: 2
+        num_orders: 1
+                """
+            ),
+            test_name="test_example_full_model_alt",
+            model=context.get_model("sqlmesh_example.full_model"),
+            context=context,
+        ).run()
+    )
+
+    mocker.patch("sqlmesh.core.test.definition.random_id", return_value="jzngz56a")
+    test = _create_test(
+        body=load_yaml(
+            """
+test_foo:
+  model: xyz
+  inputs:
+    foo:
+      query: "SELECT {'x': 1, 'n': {'y': 2}} AS struct_value"
+  outputs:
+    query:
+      - struct_value: {'x': 1, 'n': {'y': 2}}
+            """
+        ),
+        test_name="test_foo",
+        model=_create_model("SELECT struct_value FROM foo"),
+        context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+    )
+    spy_execute = mocker.spy(test.engine_adapter, "_execute")
+    _check_successful_or_raise(test.run())
+
+    spy_execute.assert_any_call(
+        'CREATE OR REPLACE VIEW "memory"."sqlmesh_test_jzngz56a"."foo" AS '
+        '''SELECT {'x': 1, 'n': {'y': 2}} AS "struct_value"'''
+    )
+
+    with pytest.raises(
+        TestError,
+        match="Invalid test, cannot set both 'query' and 'rows' for 'foo'",
+    ):
+        _create_test(
+            body=load_yaml(
+                """
+test_foo:
+  model: xyz
+  inputs:
+    foo:
+      query: "SELECT {'x': 1, 'n': {'y': 2}} AS struct_value"
+      rows:
+        struct_value: {'x': 1, 'n': {'y': 2}}
+  outputs:
+    query:
+      - struct_value: {'x': 1, 'n': {'y': 2}}
+                """
+            ),
+            test_name="test_foo",
+            model=_create_model("SELECT struct_value FROM foo"),
+            context=Context(config=Config(model_defaults=ModelDefaultsConfig(dialect="duckdb"))),
+        )
+
+
+def test_pyspark_python_model() -> None:
+    spark_connection_config = SparkConnectionConfig(
+        config={
+            "spark.master": "local",
+            "spark.sql.warehouse.dir": "/tmp/data_dir",
+            "spark.driver.extraJavaOptions": "-Dderby.system.home=/tmp/derby_dir",
+        },
+    )
+    config = Config(
+        gateways=GatewayConfig(test_connection=spark_connection_config),
+        model_defaults=ModelDefaultsConfig(dialect="spark"),
+    )
+    context = Context(config=config)
+
+    @model("pyspark_model", columns={"col": "int"})
+    def execute(context, start, end, execution_time, **kwargs):
+        return context.spark.sql("SELECT 1 AS col")
+
+    _check_successful_or_raise(
+        _create_test(
+            body=load_yaml(
+                """
+test_pyspark_model:
+  model: pyspark_model
+  outputs:
+    query:
+      - col: 1
+                """
+            ),
+            test_name="test_pyspark_model",
+            model=model.get_registry()["pyspark_model"].model(
+                module_path=Path("."), path=Path(".")
+            ),
+            context=context,
+        ).run()
+    )
 
 
 def test_test_generation(tmp_path: Path) -> None:
@@ -1052,8 +1504,7 @@ def test_test_generation(tmp_path: Path) -> None:
     assert "ctes" in test["test_full_model"]["outputs"]
     assert "cte" in test["test_full_model"]["outputs"]["ctes"]
 
-    result = context.test()
-    _check_successful_or_raise(result)
+    _check_successful_or_raise(context.test())
 
     context.create_test(
         "sqlmesh_example.full_model",
