@@ -19,7 +19,7 @@ WITH order_total AS (
   WHERE
     oi.ds > (select max(ds) from {{ this }})
 {% endif %}
-{% if sqlmesh is defined %}
+{% if sqlmesh_incremental is defined %}
   WHERE
       oi.ds BETWEEN '{{ start_ds }}' AND '{{ end_ds }}'
 {% endif %}
@@ -38,7 +38,7 @@ LEFT JOIN order_total AS ot
 WHERE
   o.ds > (select max(ds) from {{ this }})
 {% endif %}
-{% if sqlmesh is defined %}
+{% if sqlmesh_incremental is defined %}
   WHERE
       o.ds BETWEEN '{{ start_ds }}' AND '{{ end_ds }}'
 {% endif %}
