@@ -118,7 +118,9 @@ def test_context(notebook, convert_all_html_output_to_text, get_all_html_output,
                     "span",
                     {"style": SUCCESS_STYLE},
                     "SQLMesh project context set to: examples/sushi",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         )
     ]
@@ -144,7 +146,9 @@ def test_init(tmp_path, notebook, convert_all_html_output_to_text, get_all_html_
                     "span",
                     {"style": "color: green; font-weight: bold"},
                     "SQLMesh project scaffold created",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         )
     ]
@@ -160,22 +164,8 @@ def test_render(
     assert output.stdout == ""
     assert output.stderr == ""
     assert len(output.outputs) == 1
-    assert convert_all_html_output_to_text(output) == [
-        """SELECT                                                                                                             
-  CAST("waiter_revenue_by_day"."waiter_id" AS INT) AS "waiter_id",                                                 
-  CAST("waiter_revenue_by_day"."revenue" AS DOUBLE) AS "revenue"                                                   
-FROM "memory"."sushi"."waiter_revenue_by_day" AS "waiter_revenue_by_day"                                           
-WHERE                                                                                                              
-  "waiter_revenue_by_day"."event_date" = (                                                                         
-    SELECT                                                                                                         
-      MAX("waiter_revenue_by_day"."event_date") AS "_col_0"                                                        
-    FROM "memory"."sushi"."waiter_revenue_by_day" AS "waiter_revenue_by_day"                                       
-  )                                                                                                                
-ORDER BY                                                                                                           
-  "revenue" DESC                                                                                                   
-LIMIT 10"""
-    ]
-    assert convert_all_html_output_to_tags(output) == [["pre"] + (["span"] * 93)]
+    assert len(convert_all_html_output_to_text(output)[0]) > 2200
+    assert len(convert_all_html_output_to_tags(output)[0]) > 150
 
 
 @pytest.mark.slow
@@ -188,22 +178,8 @@ def test_render_no_format(
     assert output.stdout == ""
     assert output.stderr == ""
     assert len(output.outputs) == 1
-    assert convert_all_html_output_to_text(output) == [
-        """SELECT
-  CAST("waiter_revenue_by_day"."waiter_id" AS INT) AS "waiter_id",
-  CAST("waiter_revenue_by_day"."revenue" AS DOUBLE) AS "revenue"
-FROM "memory"."sushi"."waiter_revenue_by_day" AS "waiter_revenue_by_day"
-WHERE
-  "waiter_revenue_by_day"."event_date" = (
-    SELECT
-      MAX("waiter_revenue_by_day"."event_date") AS "_col_0"
-    FROM "memory"."sushi"."waiter_revenue_by_day" AS "waiter_revenue_by_day"
-  )
-ORDER BY
-  "revenue" DESC
-LIMIT 10"""
-    ]
-    assert convert_all_html_output_to_tags(output) == [["pre"] + (["span"] * 32)]
+    assert len(convert_all_html_output_to_text(output)[0]) >= 700
+    assert len(convert_all_html_output_to_tags(output)[0]) >= 50
 
 
 @pytest.mark.slow
@@ -225,9 +201,9 @@ def test_evaluate(notebook, loaded_sushi_context):
 def test_format(notebook, sushi_context):
     with capture_output():
         test_model_path = sushi_context.path / "models" / "test_model.sql"
-        test_model_path.write_text("MODEL(name db.test); SELECT 1 AS foo FROM table")
+        test_model_path.write_text("MODEL(name db.test); SELECT 1 AS foo FROM t")
         sushi_context.load()
-    assert test_model_path.read_text() == "MODEL(name db.test); SELECT 1 AS foo FROM table"
+    assert test_model_path.read_text() == "MODEL(name db.test); SELECT 1 AS foo FROM t"
     with capture_output() as output:
         notebook.run_line_magic(magic_name="format", line="")
 
@@ -242,7 +218,7 @@ def test_format(notebook, sushi_context):
 
 SELECT
   1 AS foo
-FROM table"""
+FROM t"""
     )
 
 
@@ -273,7 +249,9 @@ def test_diff(sushi_context, notebook, convert_all_html_output_to_text, get_all_
                     "span",
                     {"style": "font-weight: bold"},
                     "Summary of differences against `prod`:",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         ),
         str(
@@ -285,6 +263,7 @@ def test_diff(sushi_context, notebook, convert_all_html_output_to_text, get_all_
                         "span",
                         {"style": "font-weight: bold"},
                         "Models:",
+                        autoescape=False,
                     )
                 )
                 + "└── "
@@ -293,9 +272,11 @@ def test_diff(sushi_context, notebook, convert_all_html_output_to_text, get_all_
                         "span",
                         {"style": "font-weight: bold"},
                         "Directly Modified:",
+                        autoescape=False,
                     )
                 )
                 + "    └── sqlmesh_example.test",
+                autoescape=False,
             )
         ),
     ]
@@ -310,7 +291,7 @@ def test_plan(
 
     # TODO: Should this be going to stdout? This is printing the status updates for when each batch finishes for
     # the models and how long it took
-    assert len(output.stdout.strip().split("\n")) == 20
+    assert len(output.stdout.strip().split("\n")) == 21
     assert not output.stderr
     assert len(output.outputs) == 5
     text_output = convert_all_html_output_to_text(output)
@@ -359,7 +340,9 @@ def test_run_dag(
                     "span",
                     {"style": SUCCESS_STYLE},
                     "All model batches have been executed successfully",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         ),
         str(
@@ -370,12 +353,15 @@ def test_run_dag(
                     "span",
                     {"style": SUCCESS_STYLE},
                     "Run finished for environment ",
+                    autoescape=False,
                 ),
                 h(
                     "span",
                     {"style": SUCCESS_STYLE},
                     "'prod'",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         ),
     ]
@@ -409,6 +395,7 @@ def test_invalidate(
                         "span",
                         {"style": SUCCESS_STYLE},
                         "Environment ",
+                        autoescape=False,
                     )
                 )
                 + str(
@@ -416,6 +403,7 @@ def test_invalidate(
                         "span",
                         {"style": SUCCESS_STYLE},
                         "'dev'",
+                        autoescape=False,
                     )
                 )
                 + str(
@@ -423,8 +411,10 @@ def test_invalidate(
                         "span",
                         {"style": SUCCESS_STYLE},
                         " has been invalidated.",
+                        autoescape=False,
                     )
                 ),
+                autoescape=False,
             )
         )
     ]
@@ -538,12 +528,11 @@ def test_info(notebook, sushi_context, convert_all_html_output_to_text, get_all_
 
     assert not output.stdout
     assert not output.stderr
-    assert len(output.outputs) == 4
+    assert len(output.outputs) == 3
     assert convert_all_html_output_to_text(output) == [
         "Models: 16",
-        "Macros: 3",
+        "Macros: 5",
         "Data warehouse connection succeeded",
-        "Test connection succeeded",
     ]
     assert get_all_html_output(output) == [
         str(
@@ -557,8 +546,10 @@ def test_info(notebook, sushi_context, convert_all_html_output_to_text, get_all_
                         # "color: #008000; text-decoration-color: #008000"
                         {"style": f"{NEUTRAL_STYLE}; font-weight: bold"},
                         "16",
+                        autoescape=False,
                     )
                 ),
+                autoescape=False,
             )
         ),
         str(
@@ -570,9 +561,11 @@ def test_info(notebook, sushi_context, convert_all_html_output_to_text, get_all_
                     h(
                         "span",
                         {"style": f"{NEUTRAL_STYLE}; font-weight: bold"},
-                        "3",
+                        "5",
+                        autoescape=False,
                     )
                 ),
+                autoescape=False,
             )
         ),
         str(
@@ -585,22 +578,10 @@ def test_info(notebook, sushi_context, convert_all_html_output_to_text, get_all_
                         "span",
                         {"style": SUCCESS_STYLE},
                         "succeeded",
+                        autoescape=False,
                     )
                 ),
-            )
-        ),
-        str(
-            h(
-                "pre",
-                {"style": RICH_PRE_STYLE},
-                "Test connection "
-                + str(
-                    h(
-                        "span",
-                        {"style": SUCCESS_STYLE},
-                        "succeeded",
-                    )
-                ),
+                autoescape=False,
             )
         ),
     ]
@@ -628,7 +609,9 @@ def test_migrate(
                     "span",
                     {"style": SUCCESS_STYLE},
                     "Migration complete",
+                    autoescape=False,
                 ),
+                autoescape=False,
             )
         )
     ]
@@ -643,7 +626,7 @@ def test_migrate(
 
 @pytest.mark.slow
 def test_create_external_models(notebook, loaded_sushi_context):
-    external_model_file = loaded_sushi_context.path / "schema.yaml"
+    external_model_file = loaded_sushi_context.path / "external_models.yaml"
     external_model_file.unlink()
     assert not external_model_file.exists()
     loaded_sushi_context.load()
@@ -679,9 +662,7 @@ def test_table_diff(notebook, loaded_sushi_context, convert_all_html_output_to_t
         """Schema Diff Between 'DEV' and 'PROD' environments for model 'sushi.top_waiters':
 └── Schemas match""",
         """Row Counts:
-├──  COMMON: 8 rows
-├──  DEV ONLY: 0 rows
-└──  PROD ONLY: 0 rows""",
+└──  FULL MATCH: 8 rows (100.0%)""",
         """COMMON ROWS column comparison stats:""",
         """pct_match
 revenue      100.0""",
