@@ -95,6 +95,7 @@ def test_apply_plan(mocker: MockerFixture, snapshot: Snapshot):
                         "name": "INCREMENTAL_BY_TIME_RANGE",
                         "time_column": {"column": "`ds`"},
                         "forward_only": False,
+                        "on_destructive_change": "ERROR",
                         "disable_restatement": False,
                         "dialect": "spark",
                     },
@@ -160,16 +161,16 @@ def test_apply_plan(mocker: MockerFixture, snapshot: Snapshot):
         "skip_backfill": False,
         "notification_targets": [],
         "request_id": request_id,
-        "restatements": {},
         "backfill_concurrent_tasks": 1,
         "ddl_concurrent_tasks": 1,
         "users": [],
         "is_dev": False,
         "forward_only": False,
+        "allow_destructive_snapshots": [],
         "models_to_backfill": ['"test_model"'],
         "end_bounded": False,
         "ensure_finalized_snapshots": False,
-        "directly_modified_snapshots": [{"identifier": "2298286125", "name": '"test_model"'}],
+        "directly_modified_snapshots": [{"identifier": "349602315", "name": '"test_model"'}],
         "indirectly_modified_snapshots": {},
         "removed_snapshots": [],
         "restatements": {'"test_model"': [to_timestamp("2024-01-01"), to_timestamp("2024-01-02")]},
