@@ -1,12 +1,13 @@
 import os
-from distutils.core import run_setup, setup
+
+import setuptools
 
 os.chdir(os.path.join(os.path.dirname(__file__), ".."))
-sqlmesh_dist = run_setup("setup.py", stop_after="init")
+sqlmesh_dist = setuptools.distutils.core.run_setup("setup.py", stop_after="init")
 requirements = sqlmesh_dist.install_requires + sqlmesh_dist.extras_require["dev"]  # type: ignore
 os.chdir(os.path.dirname(__file__))
 
-setup(
+setuptools.setup(
     name="sqlmesh-tests",
     description="Tests for SQLMesh",
     url="https://github.com/TobikoData/sqlmesh",
