@@ -60,7 +60,7 @@ X = 1
 Y = 2
 Z = 3
 
-my_lambda = lambda: print("z")
+my_lambda = lambda: print("z")  # noqa: E731
 
 KLASS_X = 1
 KLASS_Y = 2
@@ -140,19 +140,19 @@ def test_func_globals() -> None:
 def test_normalize_source() -> None:
     assert (
         normalize_source(main_func)
-        == """def main_func(y):
+        == """def main_func(y: int):
     sqlglot.parse_one('1')
     MyClass()
     DataClass(x=y)
 
-    def closure(z):
+    def closure(z: int):
         return z + Z
     return closure(y) + other_func(Y)"""
     )
 
     assert (
         normalize_source(other_func)
-        == """def other_func(a):
+        == """def other_func(a: int):
     import sqlglot
     sqlglot.parse_one('1')
     pd.DataFrame([{'x': 1}])
@@ -163,7 +163,7 @@ def test_normalize_source() -> None:
 
 
 def test_serialize_env_error() -> None:
-    with pytest.raises(SQLMeshError) as e:
+    with pytest.raises(SQLMeshError):
         # pretend to be the module pandas
         serialize_env({"test_date": test_date}, path=Path("tests/utils"))
 
@@ -179,12 +179,12 @@ def test_serialize_env() -> None:
             name="main_func",
             alias="MAIN",
             path="test_metaprogramming.py",
-            payload="""def main_func(y):
+            payload="""def main_func(y: int):
     sqlglot.parse_one('1')
     MyClass()
     DataClass(x=y)
 
-    def closure(z):
+    def closure(z: int):
         return z + Z
     return closure(y) + other_func(Y)""",
         ),
@@ -231,12 +231,12 @@ class DataClass:
         "my_lambda": Executable(
             name="my_lambda",
             path="test_metaprogramming.py",
-            payload=f"my_lambda = lambda : print('z')",
+            payload="my_lambda = lambda : print('z')",
         ),
         "other_func": Executable(
             name="other_func",
             path="test_metaprogramming.py",
-            payload="""def other_func(a):
+            payload="""def other_func(a: int):
     import sqlglot
     sqlglot.parse_one('1')
     pd.DataFrame([{'x': 1}])
