@@ -13,12 +13,11 @@ def test_yaml() -> None:
     prod:
       type: postgres
       host: 127.0.0.1
-      # IMPORTANT: Make sure to quote the entire Jinja string here
       user: "{{ env_var('__SQLMESH_TEST_ENV_USER__') }}"
       password: "{{ env_var('__SQLMESH_TEST_ENV_PASSWORD__') }}"
 """
 
-    assert contents == yaml.dumps(yaml.load(contents, render_jinja=False))
+    assert contents == yaml.dump(yaml.load(contents, render_jinja=False))
 
     expected_contents = """profile:
   target: prod
@@ -26,7 +25,6 @@ def test_yaml() -> None:
     prod:
       type: postgres
       host: 127.0.0.1
-      # IMPORTANT: Make sure to quote the entire Jinja string here
       user: user
       password: password
 """
@@ -34,7 +32,7 @@ def test_yaml() -> None:
     os.environ["__SQLMESH_TEST_ENV_USER__"] = "user"
     os.environ["__SQLMESH_TEST_ENV_PASSWORD__"] = "password"
 
-    assert expected_contents == yaml.dumps(yaml.load(contents))
+    assert expected_contents == yaml.dump(yaml.load(contents))
 
     # Return the environment to its previous state
     del os.environ["__SQLMESH_TEST_ENV_USER__"]
