@@ -1,3 +1,4 @@
+import { isNil } from '@utils/index'
 import { tableFromIPC } from 'apache-arrow'
 
 const baseURL = window.location.origin
@@ -13,7 +14,7 @@ export interface FetchOptionsWithSignal {
 
 export interface FetchOptions<B extends object = any> {
   url: string
-  method: 'get' | 'post' | 'put' | 'delete' | 'patch'
+  method: 'GET' | 'POST' | 'PUT' | 'DELETE' | 'PATCH'
   data?: B
   responseType?: string
   headers?: Record<string, string>
@@ -26,7 +27,7 @@ export interface FetchOptions<B extends object = any> {
     | 'no-cache'
     | 'force-cache'
     | 'only-if-cached'
-  params?: Record<string, string>
+  params?: Record<string, Maybe<string | number | boolean>>
 }
 
 export async function fetchAPI<T = any, B extends object = any>(
@@ -45,7 +46,7 @@ export async function fetchAPI<T = any, B extends object = any>(
       ...params,
     }).reduce((acc: Record<string, string>, [key, param]) => {
       if (param != null) {
-        acc[key] = param
+        acc[key] = String(param)
       }
 
       return acc
@@ -64,15 +65,26 @@ export async function fetchAPI<T = any, B extends object = any>(
       signal: options?.signal,
     })
       .then(async response => {
-        const headerContentType = response.headers.get('Content-Type')
-
-        if (headerContentType == null)
-          return { ok: false, detail: 'Empty response' }
-        if (response.status >= 500) throw new Error(response.statusText)
-        if (response.status >= 400)
-          throw new Error({ ...(await response.json()) }.detail)
         if (response.status === 204) return { ok: true }
 
+        const headerContentType = response.headers.get('Content-Type')
+        if (isNil(headerContentType))
+          return { ok: false, message: 'Empty response' }
+        if (response.status >= 400) {
+          try {
+            const json = await response.json()
+
+            json.status = json.status ?? response.status
+
+            throw json
+          } catch (error: any) {
+            throw {
+              message: response.statusText,
+              ...error,
+            } as unknown as Error
+          }
+        }
+
         const isEventStream = headerContentType.includes('text/event-stream')
         const isApplicationJson = headerContentType.includes('application/json')
         const isArrowStream = headerContentType.includes(
@@ -89,13 +101,7 @@ export async function fetchAPI<T = any, B extends object = any>(
         }
       })
       .then(resolve)
-      .catch(error => {
-        if (error.name === 'AbortError') {
-          console.log(['Request aborted', 'fetchAPI', method, url])
-        } else {
-          reject(error)
-        }
-      })
+      .catch(reject)
   })
 }
 
