@@ -1,20 +1,20 @@
 import { useMemo } from 'react'
 import {
-  type BodyApplyApiCommandsApplyPostCategories,
+  type BodyInitiateApplyApiCommandsApplyPostCategoriesAnyOf,
+  type BodyInitiateApplyApiCommandsApplyPostCategories,
   type PlanDates,
   type PlanOptions,
+  SnapshotChangeCategory,
 } from '~/api/client'
-import { type ModelEnvironment } from '~/models/environment'
-import { isFalse, isStringEmptyOrNil } from '~/utils'
+import { isNil, isStringEmptyOrNil, isTrue } from '~/utils'
 import { usePlan } from './context'
+import { useStoreContext } from '@context/context'
 
-export function usePlanPayload({
-  environment,
-  isInitialPlanRun,
-}: {
-  environment: ModelEnvironment
-  isInitialPlanRun: boolean
-}): { planDates?: PlanDates; planOptions: PlanOptions } {
+export function usePlanPayload(options?: PlanOptions): {
+  planDates?: PlanDates
+  planOptions: PlanOptions
+  categories: BodyInitiateApplyApiCommandsApplyPostCategories
+} {
   const {
     start,
     end,
@@ -22,13 +22,21 @@ export function usePlanPayload({
     no_gaps,
     skip_backfill,
     forward_only,
+    include_unmodified,
     no_auto_categorization,
     restate_models,
+    auto_apply,
     create_from,
+    change_categorization,
   } = usePlan()
 
+  const environment = useStoreContext(s => s.environment)
+
+  const isInitialPlanRun =
+    isNil(environment?.isDefault) || isTrue(environment?.isDefault)
+
   const planDates = useMemo(() => {
-    if (environment.isDefault) return
+    if (environment.isProd) return
 
     return {
       start,
@@ -40,43 +48,65 @@ export function usePlanPayload({
   }, [environment, start, end, isInitialPlanRun, restate_models])
 
   const planOptions = useMemo(() => {
-    if (environment.isInitial) return { skip_tests: false }
-    if (environment.isDefault) return { skip_tests }
-
-    return {
-      no_gaps,
-      skip_backfill,
-      forward_only,
-      create_from,
-      no_auto_categorization,
-      skip_tests,
-      restate_models,
-    }
+    return environment.isInitialProd
+      ? {
+          include_unmodified: true,
+          no_gaps: true,
+          skip_tests,
+          auto_apply,
+        }
+      : {
+          skip_tests,
+          no_gaps,
+          skip_backfill,
+          forward_only,
+          create_from,
+          no_auto_categorization,
+          restate_models,
+          include_unmodified,
+          auto_apply,
+        }
   }, [
     environment,
     no_gaps,
     skip_backfill,
     forward_only,
+    include_unmodified,
     create_from,
     no_auto_categorization,
     skip_tests,
     restate_models,
+    auto_apply,
   ])
 
+  const categories = useMemo(() => {
+    return Array.from(
+      change_categorization.values(),
+    ).reduce<BodyInitiateApplyApiCommandsApplyPostCategoriesAnyOf>(
+      (acc, { category, change }) => {
+        acc[change.displayName] =
+          category?.value ?? SnapshotChangeCategory.NUMBER_1
+
+        return acc
+      },
+      {},
+    )
+  }, [change_categorization])
+
   return {
-    planOptions,
+    planOptions: {
+      ...planOptions,
+      ...options,
+    },
     planDates,
+    categories,
   }
 }
 
-export function useApplyPayload({
-  isInitialPlanRun,
-}: {
-  isInitialPlanRun: boolean
-}): {
+export function useApplyPayload(): {
   planDates?: PlanDates
   planOptions: PlanOptions
-  categories: BodyApplyApiCommandsApplyPostCategories
+  categories: BodyInitiateApplyApiCommandsApplyPostCategories
 } {
   const {
     start,
@@ -85,28 +115,34 @@ export function useApplyPayload({
     no_gaps,
     skip_backfill,
     forward_only,
+    include_unmodified,
     no_auto_categorization,
     restate_models,
-    hasBackfills,
     create_from,
     change_categorization,
   } = usePlan()
 
+  const environment = useStoreContext(s => s.environment)
+
+  const isInitialPlanRun =
+    isNil(environment?.isDefault) || isTrue(environment?.isDefault)
+
   const planDates = useMemo(() => {
-    if (isInitialPlanRun || isFalse(hasBackfills)) return
+    if (isInitialPlanRun) return
 
     return {
       start,
       end,
     }
-  }, [hasBackfills, start, end, isInitialPlanRun])
+  }, [start, end, isInitialPlanRun])
 
   const categories = useMemo(() => {
     return Array.from(
       change_categorization.values(),
-    ).reduce<BodyApplyApiCommandsApplyPostCategories>(
+    ).reduce<BodyInitiateApplyApiCommandsApplyPostCategoriesAnyOf>(
       (acc, { category, change }) => {
-        acc[change.model_name] = category.value
+        acc[change.displayName] =
+          category?.value ?? SnapshotChangeCategory.NUMBER_1
 
         return acc
       },
@@ -120,6 +156,7 @@ export function useApplyPayload({
       no_gaps,
       skip_backfill,
       forward_only,
+      include_unmodified,
       create_from,
       no_auto_categorization,
       skip_tests,
